@@ -19,13 +19,16 @@ set -euo pipefail
 source "$(dirname "$0")/../../lib/init.sh"
 source module ci/cloudbuild/builds/lib/bazel.sh
 source module ci/cloudbuild/builds/lib/integration.sh
+source module ci/lib/io.sh
 
 export CC=clang
 export CXX=clang++
 
 mapfile -t args < <(bazel::common_args)
 args+=("--config=tsan")
-bazel test "${args[@]}" --test_tag_filters=-integration-test ...
+# report_atomic_races=0: https://github.com/google/sanitizers/issues/953
+args+=("--test_env=TSAN_OPTIONS=suppressions=${PROJECT_ROOT}/ci/tsan_suppressions.txt:halt_on_error=1:second_deadlock_stack=1:report_atomic_races=0")
+io::run bazel test "${args[@]}" --test_tag_filters=-integration-test ...
 
 mapfile -t integration_args < <(integration::bazel_args)
 integration::bazel_with_emulators test "${args[@]}" "${integration_args[@]}"
