@@ -12,9 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/accesscontextmanager/access_context_manager_client.h"
+//! [all]
+#include "google/cloud/accesscontextmanager/v1/access_context_manager_client.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -22,18 +22,19 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace accesscontextmanager = ::google::cloud::accesscontextmanager;
+  namespace accesscontextmanager = ::google::cloud::accesscontextmanager_v1;
   auto client = accesscontextmanager::AccessContextManagerClient(
       accesscontextmanager::MakeAccessContextManagerConnection());
 
   auto const parent = std::string("accessPolicies/") + argv[1];
   for (auto r : client.ListAccessLevels(parent)) {
-    if (!r) throw std::runtime_error(r.status().message());
+    if (!r) throw std::move(r).status();
     std::cout << r->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
