@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/automl/auto_ml_client.h"
+//! [all]
+#include "google/cloud/automl/v1/auto_ml_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 3) {
@@ -22,18 +23,19 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace automl = ::google::cloud::automl;
+  auto const location = google::cloud::Location(argv[1], argv[2]);
+
+  namespace automl = ::google::cloud::automl_v1;
   auto client = automl::AutoMlClient(automl::MakeAutoMlConnection());
 
-  auto const parent =
-      std::string{"projects/"} + argv[1] + "/locations/" + argv[2];
-  for (auto m : client.ListModels(parent)) {
-    if (!m) throw std::runtime_error(m.status().message());
+  for (auto m : client.ListModels(location.FullName())) {
+    if (!m) throw std::move(m).status();
     std::cout << m->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
