@@ -1,4 +1,4 @@
-// Copyright 2017 Google Inc.
+// Copyright 2017 Google LLC
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -23,6 +23,8 @@
 #include "absl/types/optional.h"
 #include <gmock/gmock.h>
 #include <thread>
+// ClientOptions is deprecated. We need to suppress the compiler warnings.
+#include "google/cloud/internal/disable_deprecation_warnings.inc"
 
 namespace google {
 namespace cloud {
@@ -125,7 +127,7 @@ TEST(ClientOptionsTest, CustomBackgroundThreadsOption) {
   bool invoked = false;
   auto factory = [&invoked] {
     invoked = true;
-    return absl::make_unique<Fake>();
+    return std::make_unique<Fake>();
   };
 
   auto options =
@@ -439,7 +441,7 @@ TEST(ClientOptionsTest, SetSslTargetNameOverride) {
 
 TEST(ClientOptionsTest, UserAgentPrefix) {
   std::string const actual = ClientOptions::UserAgentPrefix();
-  EXPECT_THAT(actual, HasSubstr("gcloud-cpp/"));
+  EXPECT_THAT(actual, HasSubstr("gl-cpp/"));
 }
 
 TEST(ClientOptionsTest, RefreshPeriod) {
