@@ -14,15 +14,15 @@
 
 #include "google/cloud/bigtable/examples/bigtable_examples_common.h"
 #include "google/cloud/bigtable/resource_names.h"
-#include "google/cloud/bigtable/testing/cleanup_stale_resources.h"
-#include "google/cloud/bigtable/testing/random_names.h"
 //! [bigtable includes]
 #include "google/cloud/bigtable/table.h"
 //! [bigtable includes]
+#include "google/cloud/bigtable/testing/cleanup_stale_resources.h"
+#include "google/cloud/bigtable/testing/random_names.h"
 #include "google/cloud/internal/format_time_point.h"
 #include "google/cloud/internal/getenv.h"
 #include "google/cloud/internal/random.h"
-#include "google/cloud/testing_util/crash_handler.h"
+#include "google/cloud/log.h"
 #include <chrono>
 #include <sstream>
 
@@ -103,7 +103,7 @@ void PrepareReadSamples(google::cloud::bigtable::Table table) {
   for (auto const& f : failures) {
     std::cerr << "index[" << f.original_index() << "]=" << f.status() << "\n";
   }
-  throw std::runtime_error(failures.front().status().message());
+  throw google::cloud::Status(failures.front().status());
 }
 
 void ReadRowsWithLimit(google::cloud::bigtable::Table table,
@@ -122,8 +122,8 @@ void ReadRowsWithLimit(google::cloud::bigtable::Table table,
                                        "connected_wifi"),
         cbt::Filter::Latest(1));
     // Read and print the first rows in the range, within the row limit.
-    for (StatusOr<cbt::Row> const& row : table.ReadRows(range, limit, filter)) {
-      if (!row) throw std::runtime_error(row.status().message());
+    for (auto& row : table.ReadRows(range, limit, filter)) {
+      if (!row) throw std::move(row).status();
       if (row->cells().size() != 1) {
         std::ostringstream os;
         os << "Unexpected number of cells in " << row->row_key();
@@ -145,7 +145,8 @@ void ReadKeysSet(std::vector<std::string> argv) {
   }
 
   google::cloud::bigtable::Table table(
-      google::cloud::bigtable::MakeDataClient(argv[0], argv[1]), argv[2]);
+      google::cloud::bigtable::MakeDataConnection(),
+      google::cloud::bigtable::TableResource(argv[0], argv[1], argv[2]));
   argv.erase(argv.begin(), argv.begin() + 3);
 
   // [START bigtable_read_keys_set]
@@ -160,7 +161,7 @@ void ReadKeysSet(std::vector<std::string> argv) {
 
     cbt::Filter filter = cbt::Filter::Latest(1);
     for (StatusOr<cbt::Row>& row : table.ReadRows(std::move(row_set), filter)) {
-      if (!row) throw std::runtime_error(row.status().message());
+      if (!row) throw std::move(row).status();
       std::cout << row->row_key() << ":\n";
       for (auto const& cell : row->cells()) {
         std::cout << "\t" << cell.family_name() << ":"
@@ -188,7 +189,7 @@ void ReadPrefixList(google::cloud::bigtable::Table table,
     }
 
     for (StatusOr<cbt::Row>& row : table.ReadRows(std::move(row_set), filter)) {
-      if (!row) throw std::runtime_error(row.status().message());
+      if (!row) throw std::move(row).status();
       std::cout << row->row_key() << ":\n";
       for (cbt::Cell const& cell : row->cells()) {
         std::cout << "\t" << cell.family_name() << ":"
@@ -210,7 +211,7 @@ void ReadRow(google::cloud::bigtable::Table table,
   [](google::cloud::bigtable::Table table, std::string const& row_key) {
     StatusOr<std::pair<bool, cbt::Row>> tuple =
         table.ReadRow(row_key, cbt::Filter::PassAllFilter());
-    if (!tuple) throw std::runtime_error(tuple.status().message());
+    if (!tuple) throw std::move(tuple).status();
     if (!tuple->first) {
       std::cout << "Row " << row_key << " not found\n";
       return;
@@ -229,7 +230,7 @@ void ReadRowPartial(google::cloud::bigtable::Table table,
   [](google::cloud::bigtable::Table table, std::string const& row_key) {
     StatusOr<std::pair<bool, cbt::Row>> tuple = table.ReadRow(
         row_key, cbt::Filter::ColumnName("stats_summary", "os_build"));
-    if (!tuple) throw std::runtime_error(tuple.status().message());
+    if (!tuple) throw std::move(tuple).status();
     if (!tuple->first) {
       std::cout << "Row " << row_key << " not found\n";
       return;
@@ -247,10 +248,10 @@ void ReadRows(google::cloud::bigtable::Table table,
   using ::google::cloud::StatusOr;
   [](cbt::Table table) {
     // Read and print the rows.
-    for (StatusOr<cbt::Row> const& row : table.ReadRows(
+    for (StatusOr<cbt::Row>& row : table.ReadRows(
              cbt::RowSet("phone#4c410523#20190501", "phone#4c410523#20190502"),
              cbt::Filter::PassAllFilter())) {
-      if (!row) throw std::runtime_error(row.status().message());
+      if (!row) throw std::move(row).status();
       PrintRow(*row);
     }
   }
@@ -265,11 +266,11 @@ void ReadRowRange(google::cloud::bigtable::Table table,
   using ::google::cloud::StatusOr;
   [](cbt::Table table) {
     // Read and print the rows.
-    for (StatusOr<cbt::Row> const& row :
+    for (StatusOr<cbt::Row>& row :
          table.ReadRows(cbt::RowRange::Range("phone#4c410523#20190501",
                                              "phone#4c410523#201906201"),
                         cbt::Filter::PassAllFilter())) {
-      if (!row) throw std::runtime_error(row.status().message());
+      if (!row) throw std::move(row).status();
       PrintRow(*row);
     }
   }
@@ -284,13 +285,13 @@ void ReadRowRanges(google::cloud::bigtable::Table table,
   using ::google::cloud::StatusOr;
   [](cbt::Table table) {
     // Read and print the rows.
-    for (StatusOr<cbt::Row> const& row : table.ReadRows(
+    for (StatusOr<cbt::Row>& row : table.ReadRows(
              cbt::RowSet({cbt::RowRange::Range("phone#4c410523#20190501",
                                                "phone#4c410523#20190601"),
                           cbt::RowRange::Range("phone#5c10102#20190501",
                                                "phone#5c10102#20190601")}),
              cbt::Filter::PassAllFilter())) {
-      if (!row) throw std::runtime_error(row.status().message());
+      if (!row) throw std::move(row).status();
       PrintRow(*row);
     }
   }
@@ -305,9 +306,9 @@ void ReadRowPrefix(google::cloud::bigtable::Table table,
   using ::google::cloud::StatusOr;
   [](cbt::Table table) {
     // Read and print the rows.
-    for (StatusOr<cbt::Row> const& row : table.ReadRows(
+    for (StatusOr<cbt::Row>& row : table.ReadRows(
              cbt::RowRange::Prefix("phone"), cbt::Filter::PassAllFilter())) {
-      if (!row) throw std::runtime_error(row.status().message());
+      if (!row) throw std::move(row).status();
       PrintRow(*row);
     }
   }
@@ -322,10 +323,10 @@ void ReadFilter(google::cloud::bigtable::Table table,
   using ::google::cloud::StatusOr;
   [](cbt::Table table) {
     // Read and print the rows.
-    for (StatusOr<cbt::Row> const& row :
+    for (StatusOr<cbt::Row>& row :
          table.ReadRows(cbt::RowRange::InfiniteRange(),
                         cbt::Filter::ValueRegex("PQ2A.*"))) {
-      if (!row) throw std::runtime_error(row.status().message());
+      if (!row) throw std::move(row).status();
       PrintRow(*row);
     }
   }
@@ -333,6 +334,27 @@ void ReadFilter(google::cloud::bigtable::Table table,
   (std::move(table));
 }
 
+void ReadRowsReverse(google::cloud::bigtable::Table table,
+                     std::vector<std::string> const&) {
+  //! [reverse scan] [START bigtable_reverse_scan]
+  namespace cbt = ::google::cloud::bigtable;
+  using ::google::cloud::Options;
+  using ::google::cloud::StatusOr;
+  [](cbt::Table table) {
+    // Read and print the rows.
+    auto reader = table.ReadRows(
+        cbt::RowRange::RightOpen("phone#5c10102", "phone#5c10103"), 3,
+        cbt::Filter::PassAllFilter(),
+        Options{}.set<cbt::ReverseScanOption>(true));
+    for (StatusOr<cbt::Row>& row : reader) {
+      if (!row) throw std::move(row).status();
+      PrintRow(*row);
+    }
+  }
+  //! [reverse scan] [END bigtable_reverse_scan]
+  (std::move(table));
+}
+
 void RunAll(std::vector<std::string> const& argv) {
   namespace examples = ::google::cloud::bigtable::examples;
   namespace cbt = ::google::cloud::bigtable;
@@ -362,16 +384,15 @@ void RunAll(std::vector<std::string> const& argv) {
 
   // Create a table to run the tests on
   std::cout << "Creating table " << table_id << std::endl;
-  google::bigtable::admin::v2::GcRule gc;
-  gc.set_max_num_versions(10);
   google::bigtable::admin::v2::Table t;
   auto& families = *t.mutable_column_families();
-  *families["stats_summary"].mutable_gc_rule() = std::move(gc);
+  families["stats_summary"].mutable_gc_rule()->set_max_num_versions(10);
   auto schema = admin.CreateTable(cbt::InstanceName(project_id, instance_id),
                                   table_id, std::move(t));
-  if (!schema) throw std::runtime_error(schema.status().message());
+  if (!schema) throw std::move(schema).status();
 
-  cbt::Table table(cbt::MakeDataClient(project_id, instance_id), table_id);
+  cbt::Table table(cbt::MakeDataConnection(),
+                   cbt::TableResource(project_id, instance_id, table_id));
 
   std::cout << "Preparing data for read examples" << std::endl;
   PrepareReadSamples(table);
@@ -391,6 +412,10 @@ void RunAll(std::vector<std::string> const& argv) {
   ReadFilter(table, {});
   std::cout << "Running ReadRowsWithLimit() example" << std::endl;
   ReadRowsWithLimit(table, {"5"});
+  if (!google::cloud::bigtable::examples::UsingEmulator()) {
+    std::cout << "Running ReadRowsReverse() example" << std::endl;
+    ReadRowsReverse(table, {});
+  }
 
   std::cout << "Running ReadKeySet() example" << std::endl;
   ReadKeysSet({table.project_id(), table.instance_id(), table.table_id(),
@@ -403,9 +428,7 @@ void RunAll(std::vector<std::string> const& argv) {
 
 }  // anonymous namespace
 
-int main(int argc, char* argv[]) {
-  google::cloud::testing_util::InstallCrashHandler(argv[0]);
-
+int main(int argc, char* argv[]) try {
   using ::google::cloud::bigtable::examples::MakeCommandEntry;
   google::cloud::bigtable::examples::Commands commands = {
       MakeCommandEntry("read-row", {"<row-key>"}, ReadRow),
@@ -417,9 +440,14 @@ int main(int argc, char* argv[]) {
       MakeCommandEntry("read-row-ranges", {}, ReadRowRanges),
       MakeCommandEntry("read-row-prefix", {}, ReadRowPrefix),
       MakeCommandEntry("read-filter", {}, ReadFilter),
+      MakeCommandEntry("read-rows-reverse", {}, ReadRowsReverse),
       {"auto", RunAll},
   };
 
   google::cloud::bigtable::examples::Example example(std::move(commands));
   return example.Run(argc, argv);
+} catch (std::exception const& ex) {
+  std::cerr << ex.what() << "\n";
+  ::google::cloud::LogSink::Instance().Flush();
+  return 1;
 }
