@@ -17,9 +17,9 @@
 #include "google/cloud/bigtable/table_admin.h"
 #include "google/cloud/bigtable/testing/cleanup_stale_resources.h"
 #include "google/cloud/bigtable/testing/random_names.h"
+#include "google/cloud/bigtable/wait_for_consistency.h"
 #include "google/cloud/internal/getenv.h"
-#include "google/cloud/testing_util/crash_handler.h"
-#include "google/bigtable/admin/v2/bigtable_table_admin.pb.h"
+#include "google/cloud/log.h"
 #include <chrono>
 #include <sstream>
 #include <thread>
@@ -38,16 +38,13 @@ void CreateTable(google::cloud::bigtable_admin::BigtableTableAdminClient admin,
      std::string const& instance_id, std::string const& table_id) {
     std::string instance_name = cbt::InstanceName(project_id, instance_id);
     // Example garbage collection rule.
-    google::bigtable::admin::v2::GcRule gc;
-    gc.set_max_num_versions(10);
-
     google::bigtable::admin::v2::Table t;
     auto& families = *t.mutable_column_families();
-    *families["fam"].mutable_gc_rule() = std::move(gc);
+    families["fam"].mutable_gc_rule()->set_max_num_versions(10);
 
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.CreateTable(instance_name, table_id, std::move(t));
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Table successfully created: " << schema->DebugString()
               << "\n";
   }
@@ -71,8 +68,8 @@ void ListTables(google::cloud::bigtable_admin::BigtableTableAdminClient admin,
 
     StreamRange<google::bigtable::admin::v2::Table> tables =
         admin.ListTables(std::move(r));
-    for (auto const& table : tables) {
-      if (!table) throw std::runtime_error(table.status().message());
+    for (auto& table : tables) {
+      if (!table) throw std::move(table).status();
       std::cout << table->name() << "\n";
     }
   }
@@ -96,7 +93,7 @@ void GetTable(google::cloud::bigtable_admin::BigtableTableAdminClient admin,
 
     StatusOr<google::bigtable::admin::v2::Table> table =
         admin.GetTable(std::move(r));
-    if (!table) throw std::runtime_error(table.status().message());
+    if (!table) throw std::move(table).status();
     std::cout << table->name() << " details=\n" << table->DebugString() << "\n";
   }
   //! [get table] [END bigtable_get_table_metadata]
@@ -125,7 +122,7 @@ void CheckTableExists(
         std::cout << "Table " << table_id << " does not exist\n";
         return;
       }
-      throw std::runtime_error(table.status().message());
+      throw std::move(table).status();
     }
 
     std::cout << "Table " << table_id << " was found\n";
@@ -155,12 +152,12 @@ void GetOrCreateTable(
         table.status().code() == google::cloud::StatusCode::kNotFound) {
       // The table does not exist, try to create the table.
       table = admin.CreateTable(instance_name, table_id, {});
-      if (!table) throw std::runtime_error(table.status().message());
+      if (!table) throw std::move(table).status();
       // The schema returned by a `CreateTable()` request does not include all
       // the metadata for a table, we need to explicitly request the rest:
       table = admin.GetTable(std::move(r));
     }
-    if (!table) throw std::runtime_error(table.status().message());
+    if (!table) throw std::move(table).status();
     std::cout << "Table metadata: " << table->DebugString() << "\n";
   }
   // [END bigtable_get_or_create_table]
@@ -205,22 +202,19 @@ void ModifyTable(google::cloud::bigtable_admin::BigtableTableAdminClient admin,
     // Update
     ModifyColumnFamiliesRequest::Modification m2;
     m2.set_id("fam");
-    google::bigtable::admin::v2::GcRule gc2;
-    gc2.set_max_num_versions(5);
-    *m2.mutable_update()->mutable_gc_rule() = std::move(gc2);
+    m2.mutable_update()->mutable_gc_rule()->set_max_num_versions(5);
 
     // Create
     ModifyColumnFamiliesRequest::Modification m3;
     m3.set_id("fam");
-    google::bigtable::admin::v2::GcRule gc3;
-    gc3.mutable_max_age()->set_seconds(7 * kSecondsPerDay);
-    *m3.mutable_update()->mutable_gc_rule() = std::move(gc3);
+    m3.mutable_update()->mutable_gc_rule()->mutable_max_age()->set_seconds(
+        7 * kSecondsPerDay);
 
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.ModifyColumnFamilies(
             table_name, {std::move(m1), std::move(m2), std::move(m3)});
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Schema modified to: " << schema->DebugString() << "\n";
   }
   //! [modify table]
@@ -244,14 +238,13 @@ void CreateMaxAgeFamily(
 
     ModifyColumnFamiliesRequest::Modification mod;
     mod.set_id(family_name);
-    google::bigtable::admin::v2::GcRule gc;
-    gc.mutable_max_age()->set_seconds(5 * kSecondsPerDay);
-    *mod.mutable_create()->mutable_gc_rule() = std::move(gc);
+    mod.mutable_create()->mutable_gc_rule()->mutable_max_age()->set_seconds(
+        5 * kSecondsPerDay);
 
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.ModifyColumnFamilies(table_name, {std::move(mod)});
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Schema modified to: " << schema->DebugString() << "\n";
   }
   // [END bigtable_create_family_gc_max_age]
@@ -273,14 +266,12 @@ void CreateMaxVersionsFamily(
 
     ModifyColumnFamiliesRequest::Modification mod;
     mod.set_id(family_name);
-    google::bigtable::admin::v2::GcRule gc;
-    gc.set_max_num_versions(2);
-    *mod.mutable_create()->mutable_gc_rule() = std::move(gc);
+    mod.mutable_create()->mutable_gc_rule()->set_max_num_versions(2);
 
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.ModifyColumnFamilies(table_name, {std::move(mod)});
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Schema modified to: " << schema->DebugString() << "\n";
   }
   // [END bigtable_create_family_gc_max_versions]
@@ -302,24 +293,16 @@ void CreateUnionFamily(
     auto constexpr kSecondsPerDay =
         std::chrono::seconds(std::chrono::hours(24)).count();
 
-    google::bigtable::admin::v2::GcRule gc1;
-    gc1.set_max_num_versions(1);
-
-    google::bigtable::admin::v2::GcRule gc2;
-    gc2.mutable_max_age()->set_seconds(5 * kSecondsPerDay);
-
-    google::bigtable::admin::v2::GcRule gc_union;
-    *gc_union.mutable_union_()->add_rules() = std::move(gc1);
-    *gc_union.mutable_union_()->add_rules() = std::move(gc2);
-
     ModifyColumnFamiliesRequest::Modification mod;
     mod.set_id(family_name);
-    *mod.mutable_create()->mutable_gc_rule() = std::move(gc_union);
+    auto& gc_union = *mod.mutable_create()->mutable_gc_rule()->mutable_union_();
+    gc_union.add_rules()->set_max_num_versions(1);
+    gc_union.add_rules()->mutable_max_age()->set_seconds(5 * kSecondsPerDay);
 
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.ModifyColumnFamilies(table_name, {std::move(mod)});
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Schema modified to: " << schema->DebugString() << "\n";
   }
   // [END bigtable_create_family_gc_union]
@@ -341,24 +324,17 @@ void CreateIntersectionFamily(
     auto constexpr kSecondsPerDay =
         std::chrono::seconds(std::chrono::hours(24)).count();
 
-    google::bigtable::admin::v2::GcRule gc1;
-    gc1.set_max_num_versions(1);
-
-    google::bigtable::admin::v2::GcRule gc2;
-    gc2.mutable_max_age()->set_seconds(5 * kSecondsPerDay);
-
-    google::bigtable::admin::v2::GcRule gc_intersection;
-    *gc_intersection.mutable_intersection()->add_rules() = std::move(gc1);
-    *gc_intersection.mutable_intersection()->add_rules() = std::move(gc2);
-
     ModifyColumnFamiliesRequest::Modification mod;
     mod.set_id(family_name);
-    *mod.mutable_create()->mutable_gc_rule() = std::move(gc_intersection);
+    auto& gc_int =
+        *mod.mutable_create()->mutable_gc_rule()->mutable_intersection();
+    gc_int.add_rules()->set_max_num_versions(1);
+    gc_int.add_rules()->mutable_max_age()->set_seconds(5 * kSecondsPerDay);
 
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.ModifyColumnFamilies(table_name, {std::move(mod)});
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Schema modified to: " << schema->DebugString() << "\n";
   }
   // [END bigtable_create_family_gc_intersection]
@@ -380,30 +356,22 @@ void CreateNestedFamily(
     auto constexpr kSecondsPerDay =
         std::chrono::seconds(std::chrono::hours(24)).count();
 
-    google::bigtable::admin::v2::GcRule gc1;
-    gc1.set_max_num_versions(10);
-
-    google::bigtable::admin::v2::GcRule gc2_1;
-    gc2_1.set_max_num_versions(1);
-    google::bigtable::admin::v2::GcRule gc2_2;
-    gc2_2.mutable_max_age()->set_seconds(5 * kSecondsPerDay);
-
-    google::bigtable::admin::v2::GcRule gc2;
-    *gc2.mutable_intersection()->add_rules() = std::move(gc2_1);
-    *gc2.mutable_intersection()->add_rules() = std::move(gc2_2);
-
-    google::bigtable::admin::v2::GcRule gc;
-    *gc.mutable_union_()->add_rules() = std::move(gc1);
-    *gc.mutable_union_()->add_rules() = std::move(gc2);
-
     ModifyColumnFamiliesRequest::Modification mod;
     mod.set_id(family_name);
-    *mod.mutable_create()->mutable_gc_rule() = std::move(gc);
+    auto& gc = *mod.mutable_create()->mutable_gc_rule();
+    auto& gc_1 = *gc.mutable_union_()->add_rules();
+    auto& gc_2 = *gc.mutable_union_()->add_rules();
+    auto& gc_2_1 = *gc_2.mutable_intersection()->add_rules();
+    auto& gc_2_2 = *gc_2.mutable_intersection()->add_rules();
+
+    gc_1.set_max_num_versions(10);
+    gc_2_1.set_max_num_versions(1);
+    gc_2_2.mutable_max_age()->set_seconds(5 * kSecondsPerDay);
 
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.ModifyColumnFamilies(table_name, {std::move(mod)});
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Schema modified to: " << schema->DebugString() << "\n";
   }
   // [END bigtable_create_family_gc_nested]
@@ -430,7 +398,7 @@ void GetFamilyMetadata(
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.GetTable(std::move(r));
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     auto pos = schema->column_families().find(family_name);
     if (pos == schema->column_families().end()) {
       std::cout << "Cannot find family <" << family_name << "> in table\n";
@@ -464,18 +432,16 @@ void GetOrCreateFamily(
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.GetTable(std::move(r));
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     auto pos = schema->column_families().find(family_name);
     if (pos == schema->column_families().end()) {
       // Try to create the column family instead:
       ModifyColumnFamiliesRequest::Modification mod;
       mod.set_id(family_name);
-      google::bigtable::admin::v2::GcRule gc;
-      gc.set_max_num_versions(5);
-      *mod.mutable_create()->mutable_gc_rule() = std::move(gc);
+      mod.mutable_create()->mutable_gc_rule()->set_max_num_versions(5);
 
       auto modified = admin.ModifyColumnFamilies(table_name, {std::move(mod)});
-      if (!modified) throw std::runtime_error(schema.status().message());
+      if (!modified) throw std::move(schema).status();
       schema = *std::move(modified);
       pos = schema->column_families().find(family_name);
     }
@@ -512,7 +478,7 @@ void DeleteColumnFamily(
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.ModifyColumnFamilies(table_name, {std::move(mod)});
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Schema modified to: " << schema->DebugString() << "\n";
   }
   // [END bigtable_delete_family]
@@ -538,7 +504,7 @@ void CheckFamilyExists(
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.GetTable(std::move(r));
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     auto pos = schema->column_families().find(family_name);
     if (pos == schema->column_families().end()) {
       std::cout << "The column family <" << family_name << "> does not exist";
@@ -568,7 +534,7 @@ void ListColumnFamilies(
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.GetTable(std::move(r));
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     for (auto const& kv : schema->column_families()) {
       std::string const& column_family_name = kv.first;
       google::bigtable::admin::v2::ColumnFamily const& family = kv.second;
@@ -594,14 +560,12 @@ void UpdateGcRule(google::cloud::bigtable_admin::BigtableTableAdminClient admin,
 
     ModifyColumnFamiliesRequest::Modification mod;
     mod.set_id(family_name);
-    google::bigtable::admin::v2::GcRule gc;
-    gc.set_max_num_versions(1);
-    *mod.mutable_update()->mutable_gc_rule() = std::move(gc);
+    mod.mutable_update()->mutable_gc_rule()->set_max_num_versions(1);
 
     StatusOr<google::bigtable::admin::v2::Table> schema =
         admin.ModifyColumnFamilies(table_name, {std::move(mod)});
 
-    if (!schema) throw std::runtime_error(schema.status().message());
+    if (!schema) throw std::move(schema).status();
     std::cout << "Schema modified to: " << schema->DebugString() << "\n";
   }
   // [END bigtable_update_gc_rule]
@@ -659,48 +623,43 @@ void DropRowsByPrefix(
   (std::move(admin), argv.at(0), argv.at(1), argv.at(2), argv.at(3));
 }
 
-// TODO(#7732) - update this sample to use the helper method
 void WaitForConsistencyCheck(
-    google::cloud::bigtable_admin::BigtableTableAdminClient const&,
+    google::cloud::bigtable_admin::BigtableTableAdminClient admin,
     std::vector<std::string> const& argv) {
-  auto old_admin = google::cloud::bigtable::TableAdmin(
-      google::cloud::bigtable::MakeAdminClient(argv.at(0)), argv.at(1));
-
   //! [wait for consistency check]
   namespace cbt = ::google::cloud::bigtable;
+  namespace cbta = ::google::cloud::bigtable_admin;
+  using ::google::cloud::CompletionQueue;
   using ::google::cloud::future;
+  using ::google::cloud::Status;
   using ::google::cloud::StatusOr;
-  [](cbt::TableAdmin admin, std::string const& table_id) {
-    StatusOr<std::string> consistency_token =
-        admin.GenerateConsistencyToken(table_id);
+  [](cbta::BigtableTableAdminClient admin, std::string const& project_id,
+     std::string const& instance_id, std::string const& table_id) {
+    std::string table_name = cbt::TableName(project_id, instance_id, table_id);
+    StatusOr<google::bigtable::admin::v2::GenerateConsistencyTokenResponse>
+        consistency_token = admin.GenerateConsistencyToken(table_name);
     if (!consistency_token) {
-      throw std::runtime_error(consistency_token.status().message());
+      throw std::move(consistency_token).status();
     }
-    future<StatusOr<cbt::Consistency>> consistent_future =
-        admin.WaitForConsistency(table_id, *consistency_token);
-    future<void> fut = consistent_future.then(
-        [&consistency_token](future<StatusOr<cbt::Consistency>> f) {
-          auto is_consistent = f.get();
-          if (!is_consistent) {
-            throw std::runtime_error(is_consistent.status().message());
-          }
-          if (*is_consistent == cbt::Consistency::kConsistent) {
-            std::cout << "Table is consistent with token " << *consistency_token
-                      << "\n";
-          } else {
-            std::cout
-                << "Table is not yet consistent, Please try again later with"
-                << " the same token (" << *consistency_token << ")\n";
-          }
-        });
-    fut.get();  // simplify example by blocking until operation is done.
+    // Start a thread to perform the background work.
+    CompletionQueue cq;
+    std::thread cq_runner([&cq] { cq.Run(); });
+
+    std::string token = consistency_token->consistency_token();
+    future<Status> consistent_future =
+        cbta::AsyncWaitForConsistency(cq, admin, table_name, token);
+
+    // Simplify the example by blocking until the operation is done.
+    Status status = consistent_future.get();
+    if (!status.ok()) throw std::runtime_error(status.message());
+    std::cout << "Table is consistent with token " << token << "\n";
+
+    // Shutdown the work queue and join the background thread
+    cq.Shutdown();
+    cq_runner.join();
   }
   //! [wait for consistency check]
-  (old_admin, argv.at(2));
-
-  // TODO(#7740) - remove this sleep when lifetime issues are fixed.
-  // Extend the lifetime of old_admin long enough to avoid a crash.
-  std::this_thread::sleep_for(std::chrono::milliseconds(500));
+  (std::move(admin), argv.at(0), argv.at(1), argv.at(2));
 }
 
 void CheckConsistency(
@@ -716,7 +675,7 @@ void CheckConsistency(
     std::string table_name = cbt::TableName(project_id, instance_id, table_id);
     StatusOr<google::bigtable::admin::v2::CheckConsistencyResponse> result =
         admin.CheckConsistency(table_name, consistency_token);
-    if (!result) throw std::runtime_error(result.status().message());
+    if (!result) throw std::move(result).status();
     if (result->consistent()) {
       std::cout << "Table is consistent with token " << consistency_token
                 << "\n";
@@ -742,7 +701,7 @@ void GenerateConsistencyToken(
     std::string table_name = cbt::TableName(project_id, instance_id, table_id);
     StatusOr<google::bigtable::admin::v2::GenerateConsistencyTokenResponse>
         token = admin.GenerateConsistencyToken(table_name);
-    if (!token) throw std::runtime_error(token.status().message());
+    if (!token) throw std::move(token).status();
     std::cout << "generated token is : " << token->consistency_token() << "\n";
   }
   //! [generate consistency token]
@@ -785,16 +744,12 @@ void RunAll(std::vector<std::string> const& argv) {
   // Create a table to run the tests on.
   google::bigtable::admin::v2::Table t;
   auto& families = *t.mutable_column_families();
-  google::bigtable::admin::v2::GcRule gc1;
-  gc1.set_max_num_versions(10);
-  *families["fam"].mutable_gc_rule() = std::move(gc1);
-  google::bigtable::admin::v2::GcRule gc2;
-  gc2.set_max_num_versions(3);
-  *families["foo"].mutable_gc_rule() = std::move(gc2);
+  families["fam"].mutable_gc_rule()->set_max_num_versions(10);
+  families["foo"].mutable_gc_rule()->set_max_num_versions(3);
 
   auto table_1 = admin.CreateTable(cbt::InstanceName(project_id, instance_id),
                                    table_id_1, std::move(t));
-  if (!table_1) throw std::runtime_error(table_1.status().message());
+  if (!table_1) throw std::move(table_1).status();
 
   std::cout << "\nRunning ListTables() example" << std::endl;
   ListTables(admin, {project_id, instance_id});
@@ -873,7 +828,7 @@ void RunAll(std::vector<std::string> const& argv) {
   GenerateConsistencyToken(admin, {project_id, instance_id, table_id_1});
 
   auto token = admin.GenerateConsistencyToken(table_1->name());
-  if (!token) throw std::runtime_error(token.status().message());
+  if (!token) throw std::move(token).status();
 
   std::cout << "\nRunning CheckConsistency() example" << std::endl;
   CheckConsistency(
@@ -891,9 +846,7 @@ void RunAll(std::vector<std::string> const& argv) {
 
 }  // anonymous namespace
 
-int main(int argc, char* argv[]) {
-  google::cloud::testing_util::InstallCrashHandler(argv[0]);
-
+int main(int argc, char* argv[]) try {
   namespace examples = ::google::cloud::bigtable::examples;
   google::cloud::bigtable::examples::Example example({
       examples::MakeCommandEntry("create-table", {"<table-id>"}, CreateTable),
@@ -949,4 +902,8 @@ int main(int argc, char* argv[]) {
       {"auto", RunAll},
   });
   return example.Run(argc, argv);
+} catch (std::exception const& ex) {
+  std::cerr << ex.what() << "\n";
+  ::google::cloud::LogSink::Instance().Flush();
+  return 1;
 }
