@@ -1,4 +1,4 @@
-// Copyright 2018 Google Inc.
+// Copyright 2018 Google LLC
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -23,11 +23,12 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
 using ::google::cloud::testing_util::IsOk;
-using ::google::cloud::testing_util::chrono_literals::operator"" _us;
+using ::google::cloud::testing_util::chrono_literals::operator"" _us;  // NOLINT
 using ::testing::Not;
 
 using MutationIntegrationTest =
     ::google::cloud::bigtable::testing::TableIntegrationTest;
+
 /**
  * This function creates Cell by ignoring the timestamp.
  * In this case Cloud Bigtable will insert the default server
@@ -61,7 +62,7 @@ std::string const kColumnFamily3 = "family3";
  * Cloud Bigtable
  */
 TEST_F(MutationIntegrationTest, SetCellTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
 
   // Create a vector of cells which will be inserted into bigtable
   std::string const row_key = "SetCellRowKey";
@@ -85,7 +86,7 @@ TEST_F(MutationIntegrationTest, SetCellTest) {
  * correctly inserted into Cloud Bigtable
  */
 TEST_F(MutationIntegrationTest, SetCellNumericValueTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
 
   // Create a vector of cells which will be inserted into bigtable
   std::string const row_key = "SetCellNumRowKey";
@@ -128,7 +129,7 @@ TEST_F(MutationIntegrationTest, SetCellNumericValueErrorTest) {
  * correctly inserted into Cloud Bigtable.
  */
 TEST_F(MutationIntegrationTest, SetCellIgnoreTimestampTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
 
   // Create a vector of cell which will be inserted into bigtable
   std::string const row_key = "SetCellRowKey";
@@ -166,7 +167,8 @@ TEST_F(MutationIntegrationTest, SetCellIgnoreTimestampTest) {
  * Bigtable.
  */
 TEST_F(MutationIntegrationTest, DeleteFromColumnForTimestampRangeTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
+
   // Create a vector of cell which will be inserted into bigtable
   std::string const row_key = "DeleteColumn-Key";
   std::vector<Cell> created_cells{
@@ -210,7 +212,8 @@ TEST_F(MutationIntegrationTest, DeleteFromColumnForTimestampRangeTest) {
  * We expect the server (and not the client library) to reject invalid ranges.
  */
 TEST_F(MutationIntegrationTest, DeleteFromColumnForReversedTimestampRangeTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
+
   // Create a vector of cell which will be inserted into bigtable
   std::string const key = "row";
   std::vector<Cell> created_cells{
@@ -242,7 +245,8 @@ TEST_F(MutationIntegrationTest, DeleteFromColumnForReversedTimestampRangeTest) {
  * We expect the server (and not the client library) to reject invalid ranges.
  */
 TEST_F(MutationIntegrationTest, DeleteFromColumnForEmptyTimestampRangeTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
+
   // Create a vector of cell which will be inserted into bigtable
   std::string const key = "row";
   std::vector<Cell> created_cells{
@@ -266,7 +270,8 @@ TEST_F(MutationIntegrationTest, DeleteFromColumnForEmptyTimestampRangeTest) {
  * is deleting all records only for that column_identifier.
  */
 TEST_F(MutationIntegrationTest, DeleteFromColumnForAllTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
+
   // Create a vector of cell which will be inserted into bigtable
   std::string const row_key = "DeleteColumnForAll-Key";
   std::vector<Cell> created_cells{
@@ -299,7 +304,8 @@ TEST_F(MutationIntegrationTest, DeleteFromColumnForAllTest) {
  * timestamp only.
  */
 TEST_F(MutationIntegrationTest, DeleteFromColumnStartingFromTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
+
   // Create a vector of cell which will be inserted into bigtable
   std::string const row_key = "DeleteColumnStartingFrom-Key";
   std::vector<Cell> created_cells{
@@ -335,7 +341,8 @@ TEST_F(MutationIntegrationTest, DeleteFromColumnStartingFromTest) {
  * timestamp only. end_timestamp is not inclusive.
  */
 TEST_F(MutationIntegrationTest, DeleteFromColumnEndingAtTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
+
   // Create a vector of cell which will be inserted into bigtable cloud
   std::string const row_key = "DeleteColumnEndingAt-Key";
   std::vector<Cell> created_cells{
@@ -372,7 +379,7 @@ TEST_F(MutationIntegrationTest, DeleteFromColumnEndingAtTest) {
  * records for that family only
  */
 TEST_F(MutationIntegrationTest, DeleteFromFamilyTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
 
   // Create a vector of cell which will be inserted into bigtable
   std::string const row_key = "DeleteFamily-Key";
@@ -404,7 +411,7 @@ TEST_F(MutationIntegrationTest, DeleteFromFamilyTest) {
  * records for that row only
  */
 TEST_F(MutationIntegrationTest, DeleteFromRowTest) {
-  auto table = GetTable();
+  auto table = GetTable("with-data-connection");
 
   // Create a vector of cell which will be inserted into bigtable
   std::string const row_key1 = "DeleteRowKey1";
