@@ -12,10 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/cloudbuild/cloud_build_client.h"
-#include "google/cloud/project.h"
+//! [all]
+#include "google/cloud/cloudbuild/v1/cloud_build_client.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -23,19 +22,18 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace cloudbuild = ::google::cloud::cloudbuild;
+  namespace cloudbuild = ::google::cloud::cloudbuild_v1;
   auto client =
       cloudbuild::CloudBuildClient(cloudbuild::MakeCloudBuildConnection());
-
-  auto const project = google::cloud::Project(argv[1]);
-  auto const filter = "";  // Any filter expression
-  for (auto r : client.ListBuilds(project.FullName(), filter)) {
-    if (!r) throw std::runtime_error(r.status().message());
-    std::cout << r->DebugString() << "\n";
+  auto const* filter = R"""(status="WORKING")""";  // List only running builds
+  for (auto b : client.ListBuilds(argv[1], filter)) {
+    if (!b) throw std::move(b).status();
+    std::cout << b->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
