@@ -24,8 +24,8 @@ namespace cloud {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 using ::testing::HasSubstr;
-using testing_util::chrono_literals::operator"" _ms;
-using testing_util::ExpectFutureError;
+using ::google::cloud::testing_util::chrono_literals::operator"" _ms;  // NOLINT
+using ::google::cloud::testing_util::ExpectFutureError;
 
 TEST(FutureTestInt, ThenSimple) {
   promise<int> p;
@@ -682,19 +682,9 @@ TEST(FutureTestString, conform_2_10_4_2_b) {
 //// @test Verify conformance with section 2.10 of the Concurrency TS.
 // NOLINTNEXTLINE(google-readability-avoid-underscore-in-googletest-name)
 TEST(FutureTestString, conform_2_10_4_2_c) {
-#if 1
   using V =
       internal::make_ready_return<std::reference_wrapper<std::string>>::type;
   static_assert(std::is_same<V, std::string&>::value, "Expected std::string&");
-#else
-  // TODO(#1410) - Implement future<R&> specialization.
-  // When T is a reference wrapper get R&.
-  std::string value("42");
-  future<std::string&> f = make_ready_future(std::ref(value));
-  EXPECT_TRUE(f.valid());
-  ASSERT_EQ(std::future_status::ready, f.wait_for(0_ms));
-  EXPECT_EQ("42", f.get());
-#endif  // 1
 }
 
 class MockFunctor {
