@@ -24,7 +24,7 @@ namespace cloud {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 using ::testing::HasSubstr;
-using testing_util::chrono_literals::operator"" _ms;
+using testing_util::chrono_literals::operator"" _ms;  // NOLINT
 using testing_util::ExpectFutureError;
 
 TEST(FutureTestVoid, ThenSimple) {
