@@ -12,28 +12,30 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/gkehub/gke_hub_client.h"
+//! [all]
+#include "google/cloud/gkehub/v1/gke_hub_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
-    std::cerr << "Usage: " << argv[0] << " project-id location-id\n";
+    std::cerr << "Usage: " << argv[0] << " project-id\n";
     return 1;
   }
 
-  namespace gkehub = ::google::cloud::gkehub;
+  auto const location = google::cloud::Location(argv[1], "-");
+
+  namespace gkehub = ::google::cloud::gkehub_v1;
   auto client = gkehub::GkeHubClient(gkehub::MakeGkeHubConnection());
 
-  auto const location =
-      "projects/" + std::string(argv[0]) + "/locations/" + std::string(argv[1]);
-  for (auto r : client.ListMemberships(location)) {
-    if (!r) throw std::runtime_error(r.status().message());
-    std::cout << r->DebugString() << "\n";
+  for (auto m : client.ListMemberships(location.FullName())) {
+    if (!m) throw std::move(m).status();
+    std::cout << m->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
