@@ -14,13 +14,16 @@
 
 #include "google/cloud/internal/async_polling_loop.h"
 #include "google/cloud/grpc_options.h"
+#include "google/cloud/internal/make_status.h"
+#include "google/cloud/internal/opentelemetry.h"
 #include "google/cloud/testing_util/async_sequencer.h"
 #include "google/cloud/testing_util/is_proto_equal.h"
 #include "google/cloud/testing_util/mock_completion_queue_impl.h"
+#include "google/cloud/testing_util/opentelemetry_matchers.h"
 #include "google/cloud/testing_util/status_matchers.h"
-#include "absl/memory/memory.h"
-#include <google/bigtable/admin/v2/bigtable_instance_admin.pb.h>
+#include <google/protobuf/timestamp.pb.h>
 #include <gmock/gmock.h>
+#include <chrono>
 
 namespace google {
 namespace cloud {
@@ -28,28 +31,34 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace internal {
 namespace {
 
-using ::google::bigtable::admin::v2::Instance;
 using ::google::cloud::testing_util::AsyncSequencer;
 using ::google::cloud::testing_util::IsOk;
 using ::google::cloud::testing_util::IsProtoEqual;
 using ::google::cloud::testing_util::MockCompletionQueueImpl;
 using ::google::cloud::testing_util::StatusIs;
-using ::google::longrunning::Operation;
 using ::testing::AtLeast;
 using ::testing::Eq;
 using ::testing::HasSubstr;
 using ::testing::Not;
 using ::testing::Return;
+using TimerType = StatusOr<std::chrono::system_clock::time_point>;
+using Response = google::protobuf::Timestamp;
+
+struct StringOption {
+  using Type = std::string;
+};
 
 class MockStub {
  public:
-  MOCK_METHOD(future<StatusOr<Operation>>, AsyncGetOperation,
-              (CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
-               google::longrunning::GetOperationRequest const&),
+  MOCK_METHOD(future<StatusOr<google::longrunning::Operation>>,
+              AsyncGetOperation,
+              (CompletionQueue&, std::shared_ptr<grpc::ClientContext>,
+               Options const&, google::longrunning::GetOperationRequest const&),
               ());
 
   MOCK_METHOD(future<Status>, AsyncCancelOperation,
-              (CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+              (CompletionQueue&, std::shared_ptr<grpc::ClientContext>,
+               Options const&,
                google::longrunning::CancelOperationRequest const&),
               ());
 };
@@ -62,25 +71,27 @@ class MockPollingPolicy : public PollingPolicy {
 };
 
 AsyncPollLongRunningOperation MakePoll(std::shared_ptr<MockStub> const& mock) {
-  return
-      [mock](CompletionQueue& cq, std::unique_ptr<grpc::ClientContext> context,
-             google::longrunning::GetOperationRequest const& request) {
-        return mock->AsyncGetOperation(cq, std::move(context), request);
-      };
+  return [mock](CompletionQueue& cq, auto context, Options const& options,
+                google::longrunning::GetOperationRequest const& request) {
+    return mock->AsyncGetOperation(cq, std::move(context), options, request);
+  };
 }
 
 AsyncCancelLongRunningOperation MakeCancel(
     std::shared_ptr<MockStub> const& mock) {
-  return
-      [mock](CompletionQueue& cq, std::unique_ptr<grpc::ClientContext> context,
-             google::longrunning::CancelOperationRequest const& request) {
-        return mock->AsyncCancelOperation(cq, std::move(context), request);
-      };
+  return [mock](CompletionQueue& cq, auto context, Options const& options,
+                google::longrunning::CancelOperationRequest const& request) {
+    return mock->AsyncCancelOperation(cq, std::move(context), options, request);
+  };
+}
+
+std::string CurrentTestName() {
+  return testing::UnitTest::GetInstance()->current_test_info()->name();
 }
 
 TEST(AsyncPollingLoopTest, ImmediateSuccess) {
-  Instance expected;
-  expected.set_name("test-instance-name");
+  Response expected;
+  expected.set_seconds(123456);
   google::longrunning::Operation op;
   op.set_name("test-op-name");
   op.set_done(true);
@@ -88,15 +99,17 @@ TEST(AsyncPollingLoopTest, ImmediateSuccess) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation).Times(0);
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure).Times(0);
   EXPECT_CALL(*policy, WaitPeriod).Times(0);
   CompletionQueue cq;
-  auto actual = AsyncPollingLoop(cq, make_ready_future(make_status_or(op)),
-                                 MakePoll(mock), MakeCancel(mock),
-                                 std::move(policy), "test-function")
-                    .get();
+  auto current = MakeImmutableOptions(Options{});
+  auto actual =
+      AsyncPollingLoop(cq, current, make_ready_future(make_status_or(op)),
+                       MakePoll(mock), MakeCancel(mock), std::move(policy),
+                       "test-function")
+          .get();
   ASSERT_THAT(actual, IsOk());
   EXPECT_THAT(*actual, IsProtoEqual(op));
 }
@@ -112,21 +125,26 @@ TEST(AsyncPollingLoopTest, ImmediateCancel) {
             make_status_or(std::chrono::system_clock::now()));
       });
   CompletionQueue cq(mock_cq);
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([](CompletionQueue&, auto, Options const& options,
                    google::longrunning::GetOperationRequest const&) {
-        return make_ready_future(StatusOr<Operation>(Status{
-            StatusCode::kCancelled, "test-function: operation cancelled"}));
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        return make_ready_future(
+            StatusOr<google::longrunning::Operation>(Status{
+                StatusCode::kCancelled, "test-function: operation cancelled"}));
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([](CompletionQueue&, auto, Options const& options,
                    google::longrunning::CancelOperationRequest const& request) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         EXPECT_EQ(request.name(), "test-op-name");
         return make_ready_future(Status{});
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure).WillOnce([](Status const& status) {
     EXPECT_THAT(status, StatusIs(StatusCode::kCancelled));
@@ -137,24 +155,23 @@ TEST(AsyncPollingLoopTest, ImmediateCancel) {
 
   promise<StatusOr<google::longrunning::Operation>> p;
   auto pending =
-      AsyncPollingLoop(cq, p.get_future(), MakePoll(mock), MakeCancel(mock),
-                       std::move(policy), "test-function");
-  pending.cancel();
+      AsyncPollingLoop(cq, current, p.get_future(), MakePoll(mock),
+                       MakeCancel(mock), std::move(policy), "test-function");
+  {
+    OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+    pending.cancel();
+  }
   p.set_value(starting_op);
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
   auto actual = pending.get();
   EXPECT_THAT(actual, StatusIs(StatusCode::kCancelled,
                                AllOf(HasSubstr("test-function"),
                                      HasSubstr("operation cancelled"))));
 }
 
-TEST(AsyncPollingLoopTest, PollThenSuccess) {
-  Instance instance;
-  instance.set_name("test-instance-name");
+TEST(AsyncPollingLoopTest, ImmediateCancelImplicitOptions) {
   google::longrunning::Operation starting_op;
   starting_op.set_name("test-op-name");
-  google::longrunning::Operation expected = starting_op;
-  expected.set_done(true);
-  expected.mutable_metadata()->PackFrom(instance);
 
   auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
   EXPECT_CALL(*mock_cq, MakeRelativeTimer)
@@ -166,20 +183,92 @@ TEST(AsyncPollingLoopTest, PollThenSuccess) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([](CompletionQueue&, auto, Options const& options,
+                   google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(CurrentOptions().get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        return make_ready_future(
+            StatusOr<google::longrunning::Operation>(Status{
+                StatusCode::kCancelled, "test-function: operation cancelled"}));
+      });
+  EXPECT_CALL(*mock, AsyncCancelOperation)
+      .WillOnce([](CompletionQueue&, auto, Options const& options,
+                   google::longrunning::CancelOperationRequest const& request) {
+        EXPECT_EQ(CurrentOptions().get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        EXPECT_EQ(request.name(), "test-op-name");
+        return make_ready_future(Status{});
+      });
+  auto policy = std::make_unique<MockPollingPolicy>();
+  EXPECT_CALL(*policy, clone()).Times(0);
+  EXPECT_CALL(*policy, OnFailure).WillOnce([](Status const& status) {
+    EXPECT_THAT(status, StatusIs(StatusCode::kCancelled));
+    return false;
+  });
+  EXPECT_CALL(*policy, WaitPeriod)
+      .WillOnce(Return(std::chrono::milliseconds(1)));
+
+  promise<StatusOr<google::longrunning::Operation>> p;
+  OptionsSpan span(Options{}.set<StringOption>(CurrentTestName()));
+  auto pending = AsyncPollingLoop(
+      cq, p.get_future(),
+      [mock](auto& cq, auto context, auto const& request) {
+        return mock->AsyncGetOperation(cq, std::move(context), CurrentOptions(),
+                                       request);
+      },
+      [mock](auto& cq, auto context, auto const& request) {
+        return mock->AsyncCancelOperation(cq, std::move(context),
+                                          CurrentOptions(), request);
+      },
+      std::move(policy), "test-function");
+  {
+    OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+    pending.cancel();
+  }
+  p.set_value(starting_op);
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto actual = pending.get();
+  EXPECT_THAT(actual, StatusIs(StatusCode::kCancelled,
+                               AllOf(HasSubstr("test-function"),
+                                     HasSubstr("operation cancelled"))));
+}
+
+TEST(AsyncPollingLoopTest, PollThenSuccess) {
+  Response response;
+  response.set_seconds(123456);
+  google::longrunning::Operation starting_op;
+  starting_op.set_name("test-op-name");
+  google::longrunning::Operation expected = starting_op;
+  expected.set_done(true);
+  expected.mutable_metadata()->PackFrom(response);
+
+  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
+  EXPECT_CALL(*mock_cq, MakeRelativeTimer)
+      .WillOnce([](std::chrono::nanoseconds) {
+        return make_ready_future(
+            make_status_or(std::chrono::system_clock::now()));
+      });
+  CompletionQueue cq(mock_cq);
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
+
+  auto mock = std::make_shared<MockStub>();
+  EXPECT_CALL(*mock, AsyncGetOperation)
+      .WillOnce([&](CompletionQueue&, auto, Options const& options,
                     google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(expected));
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure).Times(0);
   EXPECT_CALL(*policy, WaitPeriod)
       .WillRepeatedly(Return(std::chrono::milliseconds(1)));
-  auto actual =
-      AsyncPollingLoop(cq, make_ready_future(make_status_or(starting_op)),
-                       MakePoll(mock), MakeCancel(mock), std::move(policy),
-                       "test-function")
-          .get();
+  auto pending = AsyncPollingLoop(
+      cq, current, make_ready_future(make_status_or(starting_op)),
+      MakePoll(mock), MakeCancel(mock), std::move(policy), "test-function");
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto actual = pending.get();
   ASSERT_THAT(actual, IsOk());
   EXPECT_THAT(*actual, IsProtoEqual(expected));
 }
@@ -196,31 +285,32 @@ TEST(AsyncPollingLoopTest, PollThenTimerError) {
                 Status{StatusCode::kCancelled, "cq shutdown"}));
       });
   CompletionQueue cq(mock_cq);
+  auto current = MakeImmutableOptions(Options{});
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation).Times(0);
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure).Times(0);
   EXPECT_CALL(*policy, WaitPeriod)
       .WillRepeatedly(Return(std::chrono::milliseconds(1)));
-  auto actual =
-      AsyncPollingLoop(cq, make_ready_future(make_status_or(starting_op)),
-                       MakePoll(mock), MakeCancel(mock), std::move(policy),
-                       "test-function")
-          .get();
+  auto actual = AsyncPollingLoop(cq, current,
+                                 make_ready_future(make_status_or(starting_op)),
+                                 MakePoll(mock), MakeCancel(mock),
+                                 std::move(policy), "test-function")
+                    .get();
   EXPECT_THAT(actual,
               StatusIs(StatusCode::kCancelled, HasSubstr("cq shutdown")));
 }
 
 TEST(AsyncPollingLoopTest, PollThenEventualSuccess) {
-  Instance instance;
-  instance.set_name("test-instance-name");
+  Response response;
+  response.set_seconds(123456);
   google::longrunning::Operation starting_op;
   starting_op.set_name("test-op-name");
   google::longrunning::Operation expected = starting_op;
   expected.set_done(true);
-  expected.mutable_metadata()->PackFrom(instance);
+  expected.mutable_metadata()->PackFrom(response);
 
   auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
   EXPECT_CALL(*mock_cq, MakeRelativeTimer)
@@ -232,45 +322,51 @@ TEST(AsyncPollingLoopTest, PollThenEventualSuccess) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([&](CompletionQueue&, auto, Options const& options,
                     google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(starting_op));
       })
-      .WillOnce([](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([](CompletionQueue&, auto, Options const& options,
                    google::longrunning::GetOperationRequest const&) {
-        return make_ready_future(
-            StatusOr<Operation>(Status(StatusCode::kUnavailable, "try-again")));
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        return make_ready_future(StatusOr<google::longrunning::Operation>(
+            Status(StatusCode::kUnavailable, "try-again")));
       })
-      .WillOnce([&](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([&](CompletionQueue&, auto, Options const& options,
                     google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(starting_op));
       })
-      .WillOnce([&](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([&](CompletionQueue&, auto, Options const& options,
                     google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(expected));
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure).WillRepeatedly(Return(true));
   EXPECT_CALL(*policy, WaitPeriod)
       .WillRepeatedly(Return(std::chrono::milliseconds(1)));
-  auto actual =
-      AsyncPollingLoop(cq, make_ready_future(make_status_or(starting_op)),
-                       MakePoll(mock), MakeCancel(mock), std::move(policy),
-                       "test-function")
-          .get();
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
+  auto pending = AsyncPollingLoop(
+      cq, current, make_ready_future(make_status_or(starting_op)),
+      MakePoll(mock), MakeCancel(mock), std::move(policy), "test-function");
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto actual = pending.get();
   ASSERT_THAT(actual, IsOk());
   EXPECT_THAT(*actual, IsProtoEqual(expected));
 }
 
 TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicy) {
-  Instance instance;
-  instance.set_name("test-instance-name");
+  Response response;
+  response.set_seconds(123456);
   google::longrunning::Operation starting_op;
   starting_op.set_name("test-op-name");
   google::longrunning::Operation expected = starting_op;
   expected.set_done(true);
-  expected.mutable_metadata()->PackFrom(instance);
+  expected.mutable_metadata()->PackFrom(response);
 
   auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
   EXPECT_CALL(*mock_cq, MakeRelativeTimer)
@@ -283,12 +379,12 @@ TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicy) {
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(2))
-      .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<grpc::ClientContext>,
+      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
                           google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return make_ready_future(make_status_or(starting_op));
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure)
       .WillOnce(Return(true))
@@ -296,11 +392,14 @@ TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicy) {
       .WillOnce(Return(false));
   EXPECT_CALL(*policy, WaitPeriod)
       .WillRepeatedly(Return(std::chrono::milliseconds(1)));
-  auto actual =
-      AsyncPollingLoop(cq, make_ready_future(make_status_or(starting_op)),
-                       MakePoll(mock), MakeCancel(mock), std::move(policy),
-                       "test-function")
-          .get();
+
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
+  auto pending = AsyncPollingLoop(
+      cq, current, make_ready_future(make_status_or(starting_op)),
+      MakePoll(mock), MakeCancel(mock), std::move(policy), "test-function");
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto actual = pending.get();
   EXPECT_THAT(actual,
               StatusIs(Not(Eq(StatusCode::kOk)),
                        AllOf(HasSubstr("test-function"),
@@ -308,13 +407,13 @@ TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicy) {
 }
 
 TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicyWithFailure) {
-  Instance instance;
-  instance.set_name("test-instance-name");
+  Response response;
+  response.set_seconds(123456);
   google::longrunning::Operation starting_op;
   starting_op.set_name("test-op-name");
   google::longrunning::Operation expected = starting_op;
   expected.set_done(true);
-  expected.mutable_metadata()->PackFrom(instance);
+  expected.mutable_metadata()->PackFrom(response);
 
   auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
   EXPECT_CALL(*mock_cq, MakeRelativeTimer)
@@ -327,13 +426,13 @@ TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicyWithFailure) {
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(2))
-      .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<grpc::ClientContext>,
+      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
                           google::longrunning::GetOperationRequest const&) {
-        return make_ready_future(
-            StatusOr<Operation>(Status{StatusCode::kUnavailable, "try-again"}));
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
+        return make_ready_future(StatusOr<google::longrunning::Operation>(
+            Status{StatusCode::kUnavailable, "try-again"}));
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure)
       .WillOnce(Return(true))
@@ -341,25 +440,27 @@ TEST(AsyncPollingLoopTest, PollThenExhaustedPollingPolicyWithFailure) {
       .WillOnce(Return(false));
   EXPECT_CALL(*policy, WaitPeriod)
       .WillRepeatedly(Return(std::chrono::milliseconds(1)));
-  auto actual =
-      AsyncPollingLoop(cq, make_ready_future(make_status_or(starting_op)),
-                       MakePoll(mock), MakeCancel(mock), std::move(policy),
-                       "test-function")
-          .get();
+
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
+  auto pending = AsyncPollingLoop(
+      cq, current, make_ready_future(make_status_or(starting_op)),
+      MakePoll(mock), MakeCancel(mock), std::move(policy), "test-function");
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto actual = pending.get();
   ASSERT_THAT(actual,
               StatusIs(StatusCode::kUnavailable, HasSubstr("try-again")));
 }
 
 TEST(AsyncPollingLoopTest, PollLifetime) {
-  Instance instance;
-  instance.set_name("test-instance-name");
+  Response response;
+  response.set_seconds(123456);
   google::longrunning::Operation starting_op;
   starting_op.set_name("test-op-name");
   google::longrunning::Operation expected = starting_op;
   expected.set_done(true);
-  expected.mutable_metadata()->PackFrom(instance);
+  expected.mutable_metadata()->PackFrom(response);
 
-  using TimerType = StatusOr<std::chrono::system_clock::time_point>;
   AsyncSequencer<TimerType> timer_sequencer;
   auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
   EXPECT_CALL(*mock_cq, MakeRelativeTimer)
@@ -368,24 +469,26 @@ TEST(AsyncPollingLoopTest, PollLifetime) {
           [&](std::chrono::nanoseconds) { return timer_sequencer.PushBack(); });
   CompletionQueue cq(mock_cq);
 
-  AsyncSequencer<StatusOr<Operation>> get_sequencer;
+  AsyncSequencer<StatusOr<google::longrunning::Operation>> get_sequencer;
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(4)
-      .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<grpc::ClientContext>,
+      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
                           google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return get_sequencer.PushBack();
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure).WillRepeatedly(Return(true));
   EXPECT_CALL(*policy, WaitPeriod)
       .WillRepeatedly(Return(std::chrono::milliseconds(1)));
 
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
   auto pending = AsyncPollingLoop(
-      cq, make_ready_future(make_status_or(starting_op)), MakePoll(mock),
-      MakeCancel(mock), std::move(policy), "test-function");
+      cq, current, make_ready_future(make_status_or(starting_op)),
+      MakePoll(mock), MakeCancel(mock), std::move(policy), "test-function");
 
   for (int i = 0; i != 3; ++i) {
     timer_sequencer.PopFront().set_value(std::chrono::system_clock::now());
@@ -393,6 +496,7 @@ TEST(AsyncPollingLoopTest, PollLifetime) {
   }
   timer_sequencer.PopFront().set_value(std::chrono::system_clock::now());
   get_sequencer.PopFront().set_value(expected);
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
   auto actual = pending.get();
   ASSERT_THAT(actual, IsOk());
   EXPECT_THAT(*actual, IsProtoEqual(expected));
@@ -402,7 +506,6 @@ TEST(AsyncPollingLoopTest, PollThenCancelDuringTimer) {
   google::longrunning::Operation starting_op;
   starting_op.set_name("test-op-name");
 
-  using TimerType = StatusOr<std::chrono::system_clock::time_point>;
   AsyncSequencer<TimerType> timer_sequencer;
   auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
   EXPECT_CALL(*mock_cq, MakeRelativeTimer)
@@ -411,21 +514,22 @@ TEST(AsyncPollingLoopTest, PollThenCancelDuringTimer) {
           [&](std::chrono::nanoseconds) { return timer_sequencer.PushBack(); });
   CompletionQueue cq(mock_cq);
 
-  AsyncSequencer<StatusOr<Operation>> get_sequencer;
+  AsyncSequencer<StatusOr<google::longrunning::Operation>> get_sequencer;
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
-      .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<grpc::ClientContext>,
+      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
                           google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return get_sequencer.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([&](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([&](CompletionQueue&, auto, Options const& options,
                     google::longrunning::CancelOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return make_ready_future(Status{});
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure)
       .Times(2)
@@ -435,18 +539,24 @@ TEST(AsyncPollingLoopTest, PollThenCancelDuringTimer) {
   EXPECT_CALL(*policy, WaitPeriod)
       .WillRepeatedly(Return(std::chrono::milliseconds(1)));
 
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
   auto pending = AsyncPollingLoop(
-      cq, make_ready_future(make_status_or(starting_op)), MakePoll(mock),
-      MakeCancel(mock), std::move(policy), "test-function");
+      cq, current, make_ready_future(make_status_or(starting_op)),
+      MakePoll(mock), MakeCancel(mock), std::move(policy), "test-function");
 
   timer_sequencer.PopFront().set_value(std::chrono::system_clock::now());
   get_sequencer.PopFront().set_value(starting_op);
   auto t = timer_sequencer.PopFront();
-  pending.cancel();
+  {
+    OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+    pending.cancel();
+  }
   t.set_value(std::chrono::system_clock::now());
   get_sequencer.PopFront().set_value(
       Status{StatusCode::kCancelled, "test-function: operation cancelled"});
 
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
   auto actual = pending.get();
   EXPECT_THAT(actual, StatusIs(StatusCode::kCancelled,
                                AllOf(HasSubstr("test-function"),
@@ -457,7 +567,6 @@ TEST(AsyncPollingLoopTest, PollThenCancelDuringPoll) {
   google::longrunning::Operation starting_op;
   starting_op.set_name("test-op-name");
 
-  using TimerType = StatusOr<std::chrono::system_clock::time_point>;
   AsyncSequencer<TimerType> timer_sequencer;
   auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
   EXPECT_CALL(*mock_cq, MakeRelativeTimer)
@@ -466,21 +575,22 @@ TEST(AsyncPollingLoopTest, PollThenCancelDuringPoll) {
           [&](std::chrono::nanoseconds) { return timer_sequencer.PushBack(); });
   CompletionQueue cq(mock_cq);
 
-  AsyncSequencer<StatusOr<Operation>> get_sequencer;
+  AsyncSequencer<StatusOr<google::longrunning::Operation>> get_sequencer;
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
       .Times(AtLeast(1))
-      .WillRepeatedly([&](CompletionQueue&,
-                          std::unique_ptr<grpc::ClientContext>,
+      .WillRepeatedly([&](CompletionQueue&, auto, Options const& options,
                           google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return get_sequencer.PushBack();
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([&](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
+      .WillOnce([&](CompletionQueue&, auto, Options const& options,
                     google::longrunning::CancelOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         return make_ready_future(Status{});
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure)
       .Times(2)
@@ -490,18 +600,24 @@ TEST(AsyncPollingLoopTest, PollThenCancelDuringPoll) {
   EXPECT_CALL(*policy, WaitPeriod)
       .WillRepeatedly(Return(std::chrono::milliseconds(1)));
 
+  auto current =
+      MakeImmutableOptions(Options{}.set<StringOption>(CurrentTestName()));
   auto pending = AsyncPollingLoop(
-      cq, make_ready_future(make_status_or(starting_op)), MakePoll(mock),
-      MakeCancel(mock), std::move(policy), "test-function");
+      cq, current, make_ready_future(make_status_or(starting_op)),
+      MakePoll(mock), MakeCancel(mock), std::move(policy), "test-function");
 
   timer_sequencer.PopFront().set_value(std::chrono::system_clock::now());
   get_sequencer.PopFront().set_value(starting_op);
   timer_sequencer.PopFront().set_value(std::chrono::system_clock::now());
   auto g = get_sequencer.PopFront();
-  pending.cancel();
+  {
+    OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+    pending.cancel();
+  }
   g.set_value(
       Status{StatusCode::kCancelled, "test-function: operation cancelled"});
 
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
   auto actual = pending.get();
   EXPECT_THAT(actual, StatusIs(StatusCode::kCancelled,
                                AllOf(HasSubstr("test-function"),
@@ -524,26 +640,27 @@ TEST(AsyncPollingLoopTest, ConfigurePollContext) {
 
   auto mock = std::make_shared<MockStub>();
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<grpc::ClientContext> context,
+      .WillOnce([](CompletionQueue&, auto context, Options const& options,
                    google::longrunning::GetOperationRequest const&) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         // Ensure that our options have taken affect on the ClientContext before
         // we start using it.
         EXPECT_EQ(GRPC_COMPRESS_DEFLATE, context->compression_algorithm());
-        return make_ready_future(StatusOr<Operation>(Status{
-            StatusCode::kCancelled, "test-function: operation cancelled"}));
+        return make_ready_future(
+            StatusOr<google::longrunning::Operation>(Status{
+                StatusCode::kCancelled, "test-function: operation cancelled"}));
       });
   EXPECT_CALL(*mock, AsyncCancelOperation)
-      .WillOnce([](CompletionQueue&,
-                   std::unique_ptr<grpc::ClientContext> context,
+      .WillOnce([](CompletionQueue&, auto context, Options const& options,
                    google::longrunning::CancelOperationRequest const& request) {
+        EXPECT_EQ(options.get<StringOption>(), CurrentTestName());
         // Ensure that our options have taken affect on the ClientContext before
         // we start using it.
         EXPECT_EQ(GRPC_COMPRESS_DEFLATE, context->compression_algorithm());
         EXPECT_EQ(request.name(), "test-op-name");
         return make_ready_future(Status{});
       });
-  auto policy = absl::make_unique<MockPollingPolicy>();
+  auto policy = std::make_unique<MockPollingPolicy>();
   EXPECT_CALL(*policy, clone()).Times(0);
   EXPECT_CALL(*policy, OnFailure).WillOnce([](Status const& status) {
     EXPECT_THAT(status, StatusIs(StatusCode::kCancelled));
@@ -555,20 +672,159 @@ TEST(AsyncPollingLoopTest, ConfigurePollContext) {
   auto setup_poll = [](grpc::ClientContext& context) {
     context.set_compression_algorithm(GRPC_COMPRESS_DEFLATE);
   };
-  OptionsSpan span(Options{}.set<internal::GrpcSetupPollOption>(setup_poll));
-
+  auto current =
+      MakeImmutableOptions(Options{}
+                               .set<GrpcSetupPollOption>(setup_poll)
+                               .set<StringOption>(CurrentTestName()));
   promise<StatusOr<google::longrunning::Operation>> p;
   auto pending =
-      AsyncPollingLoop(cq, p.get_future(), MakePoll(mock), MakeCancel(mock),
-                       std::move(policy), "test-function");
-  pending.cancel();
+      AsyncPollingLoop(cq, current, p.get_future(), MakePoll(mock),
+                       MakeCancel(mock), std::move(policy), "test-function");
+  {
+    OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+    pending.cancel();
+  }
   p.set_value(starting_op);
+  OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
   auto actual = pending.get();
   EXPECT_THAT(actual, StatusIs(StatusCode::kCancelled,
                                AllOf(HasSubstr("test-function"),
                                      HasSubstr("operation cancelled"))));
 }
 
+#ifdef GOOGLE_CLOUD_CPP_HAVE_OPENTELEMETRY
+using ::google::cloud::testing_util::EnableTracing;
+using ::google::cloud::testing_util::IsActive;
+using ::google::cloud::testing_util::OTelAttribute;
+using ::google::cloud::testing_util::SpanHasAttributes;
+using ::google::cloud::testing_util::SpanNamed;
+using ::testing::AllOf;
+using ::testing::Each;
+using ::testing::SizeIs;
+
+TEST(AsyncPollingLoopTest, TracedAsyncBackoff) {
+  auto span_catcher = testing_util::InstallSpanCatcher();
+
+  google::longrunning::Operation starting_op;
+  starting_op.set_name("test-op-name");
+
+  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
+  EXPECT_CALL(*mock_cq, MakeRelativeTimer).WillRepeatedly([](auto) {
+    return make_ready_future<TimerType>(std::chrono::system_clock::now());
+  });
+  CompletionQueue cq(mock_cq);
+
+  auto mock = std::make_shared<MockStub>();
+  EXPECT_CALL(*mock, AsyncGetOperation).WillRepeatedly([] {
+    return make_ready_future<StatusOr<google::longrunning::Operation>>(
+        UnavailableError("try again"));
+  });
+
+  auto policy = std::make_unique<MockPollingPolicy>();
+  EXPECT_CALL(*policy, clone()).Times(0);
+  EXPECT_CALL(*policy, OnFailure)
+      .WillOnce(Return(true))
+      .WillOnce(Return(true))
+      .WillOnce(Return(true))
+      .WillOnce(Return(false));
+  EXPECT_CALL(*policy, WaitPeriod)
+      .WillRepeatedly(Return(std::chrono::milliseconds(1)));
+
+  (void)AsyncPollingLoop(cq, internal::MakeImmutableOptions(EnableTracing({})),
+                         make_ready_future(make_status_or(starting_op)),
+                         MakePoll(mock), MakeCancel(mock), std::move(policy),
+                         "test-function")
+      .get();
+
+  // The polling loop waits once initially, and once for each of the three retry
+  // attempts. So we expect a total of 4 backoffs.
+  auto spans = span_catcher->GetSpans();
+  EXPECT_THAT(spans, AllOf(SizeIs(4), Each(SpanNamed("Async Backoff"))));
+}
+
+TEST(AsyncPollingLoopTest, SpanActiveDuringCancel) {
+  auto span_catcher = testing_util::InstallSpanCatcher();
+
+  auto span = MakeSpan("span");
+  google::longrunning::Operation starting_op;
+  starting_op.set_name("test-op-name");
+
+  AsyncSequencer<TimerType> timer_sequencer;
+  auto mock_cq = std::make_shared<MockCompletionQueueImpl>();
+  EXPECT_CALL(*mock_cq, MakeRelativeTimer).Times(2).WillRepeatedly([&] {
+    return timer_sequencer.PushBack();
+  });
+  CompletionQueue cq(mock_cq);
+
+  AsyncSequencer<StatusOr<google::longrunning::Operation>> get_sequencer;
+  auto mock = std::make_shared<MockStub>();
+  EXPECT_CALL(*mock, AsyncGetOperation).Times(2).WillRepeatedly([&] {
+    return get_sequencer.PushBack();
+  });
+  EXPECT_CALL(*mock, AsyncCancelOperation)
+      .WillOnce([&](CompletionQueue&, auto, Options const&,
+                    google::longrunning::CancelOperationRequest const&) {
+        EXPECT_THAT(span, IsActive());
+        return make_ready_future(Status{});
+      });
+  auto policy = std::make_unique<MockPollingPolicy>();
+  EXPECT_CALL(*policy, clone()).Times(0);
+  EXPECT_CALL(*policy, OnFailure)
+      .Times(2)
+      .WillRepeatedly([](Status const& status) {
+        return status.code() != StatusCode::kCancelled;
+      });
+  EXPECT_CALL(*policy, WaitPeriod)
+      .WillRepeatedly(Return(std::chrono::milliseconds(1)));
+
+  OTelScope scope(span);
+  auto pending = AsyncPollingLoop(
+      cq, MakeImmutableOptions(EnableTracing({})),
+      make_ready_future(make_status_or(starting_op)), MakePoll(mock),
+      MakeCancel(mock), std::move(policy), "test-function");
+
+  timer_sequencer.PopFront().set_value(std::chrono::system_clock::now());
+  get_sequencer.PopFront().set_value(starting_op);
+  timer_sequencer.PopFront().set_value(std::chrono::system_clock::now());
+  auto g = get_sequencer.PopFront();
+  {
+    auto overlay = opentelemetry::trace::Scope(MakeSpan("overlay"));
+    pending.cancel();
+  }
+  g.set_value(CancelledError("cancelled"));
+
+  auto overlay = opentelemetry::trace::Scope(MakeSpan("overlay"));
+  (void)pending.get();
+}
+
+TEST(AsyncPollingLoopTest, TraceCapturesOperationName) {
+  auto span_catcher = testing_util::InstallSpanCatcher();
+
+  google::longrunning::Operation op;
+  op.set_name("test-op-name");
+  op.set_done(true);
+
+  auto span = MakeSpan("span");
+  auto mock = std::make_shared<MockStub>();
+  auto policy = std::make_unique<MockPollingPolicy>();
+  CompletionQueue cq;
+
+  OTelScope scope(span);
+  (void)AsyncPollingLoop(cq, internal::MakeImmutableOptions(EnableTracing({})),
+                         make_ready_future(make_status_or(op)), MakePoll(mock),
+                         MakeCancel(mock), std::move(policy), "test-function")
+      .get();
+  span->End();
+
+  auto spans = span_catcher->GetSpans();
+  EXPECT_THAT(spans,
+              ElementsAre(AllOf(SpanNamed("span"),
+                                SpanHasAttributes(OTelAttribute<std::string>(
+                                    "gl-cpp.LRO_name", "test-op-name")))));
+}
+
+#endif  // GOOGLE_CLOUD_CPP_HAVE_OPENTELEMETRY
+
 }  // namespace
 }  // namespace internal
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
