@@ -12,26 +12,29 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/logging/logging_service_v2_client.h"
+//! [all]
+#include "google/cloud/logging/v2/logging_service_v2_client.h"
+#include "google/cloud/project.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
-  if (argc != 4) {
-    std::cerr << "Usage: " << argv[0] << " project-id \n";
+  if (argc != 2) {
+    std::cerr << "Usage: " << argv[0] << " project-id\n";
     return 1;
   }
 
-  namespace logging = ::google::cloud::logging;
+  namespace logging = ::google::cloud::logging_v2;
   auto client = logging::LoggingServiceV2Client(
       logging::MakeLoggingServiceV2Connection());
-  auto const parent = std::string("projects/") + argv[1];
-  for (auto const& logs : client.ListLogs(parent)) {
-    std::cout << logs.value() << "\n";
+  auto const project = google::cloud::Project(argv[1]);
+  for (auto l : client.ListLogs(project.FullName())) {
+    if (!l) throw std::move(l).status();
+    std::cout << *l << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
