@@ -12,9 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/oslogin/os_login_client.h"
+//! [all]
+#include "google/cloud/oslogin/v1/os_login_client.h"
 #include <iostream>
-#include <stdexcept>
 #include <string>
 
 int main(int argc, char* argv[]) try {
@@ -23,17 +23,18 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace oslogin = ::google::cloud::oslogin;
+  namespace oslogin = ::google::cloud::oslogin_v1;
   auto client =
       oslogin::OsLoginServiceClient(oslogin::MakeOsLoginServiceConnection());
 
   auto const name = "users/" + std::string{argv[1]};
   auto lp = client.GetLoginProfile(name);
-  if (!lp) throw std::runtime_error(lp.status().message());
+  if (!lp) throw std::move(lp).status();
   std::cout << lp->DebugString() << "\n";
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
