@@ -15,6 +15,7 @@
 #include "google/cloud/pubsub/internal/batching_publisher_connection.h"
 #include "google/cloud/pubsub/internal/defaults.h"
 #include "google/cloud/pubsub/testing/mock_batch_sink.h"
+#include "google/cloud/pubsub/testing/mock_message_batch.h"
 #include "google/cloud/future.h"
 #include "google/cloud/internal/random.h"
 #include "google/cloud/testing_util/async_sequencer.h"
@@ -40,6 +41,7 @@ using ::testing::AtLeast;
 using ::testing::Contains;
 using ::testing::ElementsAre;
 using ::testing::HasSubstr;
+using ::testing::NiceMock;
 
 google::pubsub::v1::PublishResponse MakeResponse(
     google::pubsub::v1::PublishRequest const& request) {
@@ -60,8 +62,48 @@ std::vector<std::string> MessagesData(
   return data;
 }
 
+TEST(BatchingPublisherConnectionTest, FastDestructor) {
+  auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
+  pubsub::Topic const topic("test-project", "test-topic");
+
+  AsyncSequencer<void> async;
+  // This test will never get a chance to flush its message.
+  EXPECT_CALL(*mock, AsyncPublish).Times(0);
+
+  google::cloud::internal::AutomaticallyCreatedBackgroundThreads background;
+  // Make this so large that the test times out before the message hold expires.
+  // This ensures that the two messages will be sent in one batch.
+  auto constexpr kMaxHoldTime = std::chrono::hours(24);
+  auto const ordering_key = std::string{};
+  auto publisher = BatchingPublisherConnection::Create(
+      topic,
+      DefaultPublisherOptions(
+          Options{}
+              .set<pubsub::MaxBatchMessagesOption>(4)
+              .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
+      ordering_key, mock, background.cq(), mock_batch);
+
+  // Publishing a message starts the batch timer.
+  auto pending = publisher->Publish(
+      {pubsub::MessageBuilder{}.SetData("test-data-0").Build()});
+
+  auto const start = std::chrono::steady_clock::now();
+  publisher.reset();
+  auto const elapsed = std::chrono::steady_clock::now() - start;
+  // Considering that the timer is configured to wait 24 hours, shutting down in
+  // 30s is good enough. It also avoids flakiness introduced by more precise
+  // measurements.
+  EXPECT_LE(elapsed, std::chrono::seconds(30));
+}
+
 TEST(BatchingPublisherConnectionTest, DefaultMakesProgress) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   AsyncSequencer<void> async;
@@ -92,7 +134,7 @@ TEST(BatchingPublisherConnectionTest, DefaultMakesProgress) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, background.cq());
+      ordering_key, mock, background.cq(), mock_batch);
 
   // We expect the responses to be satisfied in the context of the completion
   // queue threads. This is an important property, the processing of any
@@ -129,6 +171,9 @@ TEST(BatchingPublisherConnectionTest, DefaultMakesProgress) {
 
 TEST(BatchingPublisherConnectionTest, BatchByMessageCount) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   EXPECT_CALL(*mock, AsyncPublish)
@@ -156,7 +201,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageCount) {
               .set<pubsub::MaxBatchMessagesOption>(2)
               .set<pubsub::MaxBatchBytesOption>(kMaxBytes)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, background.cq());
+      ordering_key, mock, background.cq(), mock_batch);
   auto r0 =
       publisher
           ->Publish({pubsub::MessageBuilder{}.SetData("test-data-0").Build()})
@@ -181,6 +226,9 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageCount) {
 
 TEST(BatchingPublisherConnectionTest, BatchByMessageSize) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   EXPECT_CALL(*mock, AsyncPublish)
@@ -211,7 +259,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSize) {
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxBatchBytesOption>(max_bytes)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, background.cq());
+      ordering_key, mock, background.cq(), mock_batch);
   auto r0 = publisher->Publish({m0}).then([](future<StatusOr<std::string>> f) {
     auto r = f.get();
     ASSERT_STATUS_OK(r);
@@ -229,6 +277,9 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSize) {
 }
 
 TEST(BatchingPublisherConnectionTest, BatchByMessageSizeLargeMessageBreak) {
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto constexpr kSinglePayload = 128;
@@ -269,7 +320,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeLargeMessageBreak) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(100)
               .set<pubsub::MaxBatchBytesOption>(kBatchLimit)),
-      ordering_key, mock, cq);
+      ordering_key, mock, cq, mock_batch);
   std::vector<future<Status>> results;
   for (int i = 0; i != 3; ++i) {
     results.push_back(
@@ -297,6 +348,9 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeLargeMessageBreak) {
 }
 
 TEST(BatchingPublisherConnectionTest, BatchByMessageSizeOversizedSingleton) {
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto constexpr kSinglePayload = 128;
@@ -347,7 +401,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeOversizedSingleton) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(100)
               .set<pubsub::MaxBatchBytesOption>(kBatchLimit)),
-      ordering_key, mock, cq);
+      ordering_key, mock, cq, mock_batch);
   std::vector<future<Status>> results;
   auto publish_single = [&] {
     results.push_back(
@@ -379,6 +433,9 @@ TEST(BatchingPublisherConnectionTest, BatchByMessageSizeOversizedSingleton) {
 }
 
 TEST(BatchingPublisherConnectionTest, BatchTorture) {
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto constexpr kMaxMessages = 20;
@@ -414,7 +471,7 @@ TEST(BatchingPublisherConnectionTest, BatchTorture) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(kMaxMessages)
               .set<pubsub::MaxBatchBytesOption>(kMaxPayload)),
-      ordering_key, mock, background.cq());
+      ordering_key, mock, background.cq(), mock_batch);
 
   auto worker = [&](int iterations) {
     auto gen = google::cloud::internal::DefaultPRNG(std::random_device{}());
@@ -443,6 +500,9 @@ TEST(BatchingPublisherConnectionTest, BatchTorture) {
 
 TEST(BatchingPublisherConnectionTest, BatchByMaximumHoldTime) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   EXPECT_CALL(*mock, AsyncPublish)
@@ -466,7 +526,7 @@ TEST(BatchingPublisherConnectionTest, BatchByMaximumHoldTime) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxHoldTimeOption>(std::chrono::milliseconds(5))),
-      /*ordering_key=*/{}, mock, cq);
+      /*ordering_key=*/{}, mock, cq, mock_batch);
   auto r0 =
       publisher
           ->Publish({pubsub::MessageBuilder{}.SetData("test-data-0").Build()})
@@ -497,6 +557,9 @@ TEST(BatchingPublisherConnectionTest, BatchByMaximumHoldTime) {
 
 TEST(BatchingPublisherConnectionTest, BatchByFlush) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   EXPECT_CALL(*mock, AsyncPublish)
@@ -531,7 +594,7 @@ TEST(BatchingPublisherConnectionTest, BatchByFlush) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(4)
               .set<pubsub::MaxHoldTimeOption>(std::chrono::milliseconds(5))),
-      ordering_key, mock, cq);
+      ordering_key, mock, cq, mock_batch);
 
   std::vector<future<void>> results;
   for (auto i : {0, 1}) {
@@ -574,6 +637,9 @@ TEST(BatchingPublisherConnectionTest, BatchByFlush) {
 
 TEST(BatchingPublisherConnectionTest, HandleError) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto const error_status = Status(StatusCode::kPermissionDenied, "uh-oh");
@@ -588,7 +654,7 @@ TEST(BatchingPublisherConnectionTest, HandleError) {
   auto publisher = BatchingPublisherConnection::Create(
       topic,
       DefaultPublisherOptions(Options{}.set<pubsub::MaxBatchMessagesOption>(2)),
-      ordering_key, mock, bg.cq());
+      ordering_key, mock, bg.cq(), mock_batch);
   auto r0 = publisher->Publish(
       {pubsub::MessageBuilder{}.SetData("test-data-0").Build()});
   auto r1 = publisher->Publish(
@@ -602,6 +668,9 @@ TEST(BatchingPublisherConnectionTest, HandleError) {
 
 TEST(BatchingPublisherConnectionTest, HandleInvalidResponse) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   EXPECT_CALL(*mock, AsyncPublish)
@@ -614,7 +683,7 @@ TEST(BatchingPublisherConnectionTest, HandleInvalidResponse) {
   auto publisher = BatchingPublisherConnection::Create(
       topic,
       DefaultPublisherOptions(Options{}.set<pubsub::MaxBatchMessagesOption>(2)),
-      "test-ordering-key", mock, background.cq());
+      "test-ordering-key", mock, background.cq(), mock_batch);
   auto r0 = publisher->Publish(
       {pubsub::MessageBuilder{}.SetData("test-data-0").Build()});
   auto r1 = publisher->Publish(
@@ -628,6 +697,9 @@ TEST(BatchingPublisherConnectionTest, HandleInvalidResponse) {
 
 TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingPartialBatch) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
 
   auto const error_status = Status(StatusCode::kPermissionDenied, "uh-oh");
@@ -648,7 +720,7 @@ TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingPartialBatch) {
       topic,
       DefaultPublisherOptions(
           Options{}.set<pubsub::MaxBatchMessagesOption>(kBatchSize)),
-      ordering_key, mock, cq);
+      ordering_key, mock, cq, mock_batch);
   std::vector<future<StatusOr<std::string>>> results;
   // Create a full batch (by message count) and a partial batch.
   for (int i = 0; i != kBatchSize + kBatchSize / 2; ++i) {
@@ -676,6 +748,9 @@ TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingPartialBatch) {
 
 TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingResume) {
   auto mock = std::make_shared<pubsub_testing::MockBatchSink>();
+  auto mock_batch =
+      std::make_shared<NiceMock<pubsub_testing::MockMessageBatch>>();
+  EXPECT_CALL(*mock_batch, Flush).WillRepeatedly([] { return [](auto) {}; });
   pubsub::Topic const topic("test-project", "test-topic");
   auto const ordering_key = std::string{"test-key"};
 
@@ -709,7 +784,7 @@ TEST(BatchingPublisherConnectionTest, HandleErrorWithOrderingResume) {
           Options{}
               .set<pubsub::MaxBatchMessagesOption>(kBatchSize)
               .set<pubsub::MaxHoldTimeOption>(kMaxHoldTime)),
-      ordering_key, mock, cq);
+      ordering_key, mock, cq, mock_batch);
   std::vector<future<StatusOr<std::string>>> results;
   // Create a full batch (by size).
   for (int i = 0; i != kBatchSize; ++i) {
