@@ -24,8 +24,8 @@ namespace pubsub_testing {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
 std::unique_ptr<pubsub_testing::MockAsyncPullStream> FakeAsyncStreamingPull(
-    google::cloud::CompletionQueue& cq, std::unique_ptr<grpc::ClientContext>,
-    google::pubsub::v1::StreamingPullRequest const&);
+    google::cloud::CompletionQueue const& completion_queue,
+    std::shared_ptr<grpc::ClientContext>);
 
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace pubsub_testing
