@@ -12,10 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/redis/cloud_redis_client.h"
-#include "google/cloud/project.h"
+//! [all]
+#include "google/cloud/redis/v1/cloud_redis_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
@@ -23,18 +23,19 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace redis = ::google::cloud::redis;
+  auto const location = google::cloud::Location(argv[1], "-");
+
+  namespace redis = ::google::cloud::redis_v1;
   auto client = redis::CloudRedisClient(redis::MakeCloudRedisConnection());
 
-  auto const project_id = std::string(argv[1]);
-  auto const parent = "projects/" + project_id + "locations/-";
-  for (auto r : client.ListInstances(parent)) {
-    if (!r) throw std::runtime_error(r.status().message());
-    std::cout << r->DebugString() << "\n";
+  for (auto i : client.ListInstances(location.FullName())) {
+    if (!i) throw std::move(i).status();
+    std::cout << i->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
