@@ -12,28 +12,31 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/scheduler/cloud_scheduler_client.h"
-#include "google/cloud/project.h"
+//! [all]
+#include "google/cloud/scheduler/v1/cloud_scheduler_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
-  if (argc != 2) {
-    std::cerr << "Usage: " << argv[0] << " project-id\n";
+  if (argc != 3) {
+    std::cerr << "Usage: " << argv[0] << " project-id location-id\n";
     return 1;
   }
 
-  namespace scheduler = ::google::cloud::scheduler;
+  auto const location = google::cloud::Location(argv[1], argv[2]);
+
+  namespace scheduler = ::google::cloud::scheduler_v1;
   auto client = scheduler::CloudSchedulerClient(
       scheduler::MakeCloudSchedulerConnection());
-  auto const project = google::cloud::Project(argv[1]);
-  for (auto j : client.ListJobs(project.FullName())) {
-    if (!j) throw std::runtime_error(j.status().message());
+
+  for (auto j : client.ListJobs(location.FullName())) {
+    if (!j) throw std::move(j).status();
     std::cout << j->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
