@@ -12,29 +12,30 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/securitycenter/security_center_client.h"
+//! [all]
+#include "google/cloud/securitycenter/v1/security_center_client.h"
 #include "google/cloud/project.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 2) {
-    std::cerr << "Usage: " << argv[0] << " organization-id\n";
+    std::cerr << "Usage: " << argv[0] << " project-id\n";
     return 1;
   }
 
-  namespace securitycenter = ::google::cloud::securitycenter;
+  namespace securitycenter = ::google::cloud::securitycenter_v1;
   auto client = securitycenter::SecurityCenterClient(
       securitycenter::MakeSecurityCenterConnection());
 
-  auto const parent = std::string{"organizations/"} + argv[1];
-  for (auto c : client.ListNotificationConfigs(parent)) {
-    if (!c) throw std::runtime_error(c.status().message());
+  auto const project = google::cloud::Project(argv[1]);
+  for (auto c : client.ListSources(project.FullName())) {
+    if (!c) throw std::move(c).status();
     std::cout << c->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
