@@ -12,9 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/shell/cloud_shell_client.h"
+//! [all]
+#include "google/cloud/shell/v1/cloud_shell_client.h"
 #include <iostream>
-#include <stdexcept>
 #include <string>
 
 int main(int argc, char* argv[]) try {
@@ -23,17 +23,18 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace shell = ::google::cloud::shell;
+  namespace shell = ::google::cloud::shell_v1;
   auto client =
       shell::CloudShellServiceClient(shell::MakeCloudShellServiceConnection());
 
   auto const name = "users/" + std::string{argv[1]} + "/environments/default";
   auto env = client.GetEnvironment(name);
-  if (!env) throw std::runtime_error(env.status().message());
+  if (!env) throw std::move(env).status();
   std::cout << env->DebugString() << "\n";
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
