@@ -16,8 +16,9 @@
 #include "google/cloud/spanner/benchmarks/benchmarks_config.h"
 #include "google/cloud/spanner/client.h"
 #include "google/cloud/spanner/internal/defaults.h"
+#include "google/cloud/spanner/internal/route_to_leader.h"
 #include "google/cloud/spanner/internal/session_pool.h"
-#include "google/cloud/spanner/internal/spanner_stub.h"
+#include "google/cloud/spanner/internal/spanner_stub_factory.h"
 #include "google/cloud/spanner/testing/pick_random_instance.h"
 #include "google/cloud/spanner/testing/random_database_name.h"
 #include "google/cloud/grpc_error_delegate.h"
@@ -27,7 +28,6 @@
 #include "google/cloud/internal/random.h"
 #include "google/cloud/internal/unified_grpc_credentials.h"
 #include "google/cloud/testing_util/timer.h"
-#include "absl/memory/memory.h"
 #include "absl/time/civil_time.h"
 #include <google/spanner/v1/result_set.pb.h>
 #include <grpcpp/grpcpp.h>
@@ -291,9 +291,12 @@ class ExperimentImpl {
               << std::flush;
 
     auto connection = spanner::MakeConnection(
-        database, spanner::ConnectionOptions().set_num_channels(num_channels),
+        database,
         // This pre-creates all the Sessions we will need (one per thread).
-        spanner::SessionPoolOptions().set_min_sessions(config.maximum_threads));
+        google::cloud::Options{}
+            .set<google::cloud::GrpcNumChannelsOption>(num_channels)
+            .set<spanner::SessionPoolMinSessionsOption>(
+                config.maximum_threads));
     return spanner::Client(std::move(connection));
   }
 
@@ -303,8 +306,9 @@ class ExperimentImpl {
     std::cout << "# Creating " << num_channels << " stub"
               << (num_channels != 1 ? "s" : "") << "\n"
               << std::flush;
-    auto opts = google::cloud::internal::MakeOptions(
-        spanner::ConnectionOptions().set_num_channels(num_channels));
+    auto opts =
+        google::cloud::Options{}.set<google::cloud::GrpcNumChannelsOption>(
+            num_channels);
     opts = spanner_internal::DefaultOptions(std::move(opts));
     auto auth = google::cloud::internal::CreateAuthenticationStrategy(
         opts.get<google::cloud::GrpcCredentialOption>());
@@ -317,12 +321,6 @@ class ExperimentImpl {
     return stubs;
   }
 
-  /// Get a snapshot of the random bit generator
-  google::cloud::internal::DefaultPRNG Generator() const {
-    std::lock_guard<std::mutex> lk(mu_);
-    return generator_;
-  };
-
   void DumpSamples(std::vector<RowCpuSample> const& samples) const {
     std::lock_guard<std::mutex> lk(mu_);
     std::copy(samples.begin(), samples.end(),
@@ -461,6 +459,8 @@ class BasicExperiment : public Experiment {
   }
 
  protected:
+  // Note that by bypassing the Client layer we are not instantiating
+  // an `OptionsSpan`, so `CurrentOptions()` will be empty if called.
   virtual std::vector<RowCpuSample> ViaStub(
       Config const& config, int thread_count, int channel_count,
       spanner::Database const& database,
@@ -480,7 +480,7 @@ class BasicExperiment : public Experiment {
     int num_stubs = static_cast<int>(stubs.size());
     int task_id = 0;
     for (auto& t : tasks) {
-      auto client = stubs[task_id++ % num_stubs];
+      auto const& client = stubs[task_id++ % num_stubs];
       t = std::async(std::launch::async, [this, &config, thread_count,
                                           num_stubs, client] {
         return ViaStub(config, thread_count, num_stubs,
@@ -542,6 +542,7 @@ class ReadExperiment : public BasicExperiment<Traits> {
       Status last_status;
       for (int i = 0; i != 10; ++i) {
         grpc::ClientContext context;
+        spanner_internal::RouteToLeader(context);  // always for CreateSession
         google::spanner::v1::CreateSessionRequest request{};
         request.set_database(database.FullName());
         auto response = stub->CreateSession(context, request);
@@ -588,15 +589,25 @@ class ReadExperiment : public BasicExperiment<Traits> {
       int row_count = 0;
       google::spanner::v1::PartialResultSet result;
       std::vector<google::protobuf::Value> row;
-      grpc::ClientContext context;
-      auto stream = stub->StreamingRead(context, request);
-      for (bool success = stream->Read(&result); success;
-           success = stream->Read(&result)) {
+      row.resize(columns.size());
+      auto stream = stub->StreamingRead(std::make_shared<grpc::ClientContext>(),
+                                        google::cloud::Options{}, request);
+      for (;;) {
+        auto read = stream->Read();
+        if (absl::holds_alternative<Status>(read)) {
+          auto status = absl::get<Status>(std::move(read));
+          auto const usage = timer.Sample();
+          samples.push_back(RowCpuSample{channel_count, thread_count, true,
+                                         row_count, usage.elapsed_time,
+                                         usage.cpu_time, std::move(status)});
+          break;
+        }
+        auto result =
+            absl::get<google::spanner::v1::PartialResultSet>(std::move(read));
         if (result.chunked_value()) {
           // We do not handle chunked values in the benchmark.
           continue;
         }
-        row.resize(columns.size());
         std::size_t index = 0;
         for (auto& value : *result.mutable_values()) {
           row[index] = std::move(value);
@@ -606,11 +617,6 @@ class ReadExperiment : public BasicExperiment<Traits> {
           }
         }
       }
-      auto final = stream->Finish();
-      auto const usage = timer.Sample();
-      samples.push_back(RowCpuSample{
-          channel_count, thread_count, true, row_count, usage.elapsed_time,
-          usage.cpu_time, google::cloud::MakeStatusFromRpcError(final)});
     }
     return samples;
   }
@@ -681,6 +687,7 @@ class SelectExperiment : public BasicExperiment<Traits> {
       Status last_status;
       for (int i = 0; i != ExperimentImpl<Traits>::kColumnCount; ++i) {
         grpc::ClientContext context;
+        spanner_internal::RouteToLeader(context);  // always for CreateSession
         google::spanner::v1::CreateSessionRequest request{};
         request.set_database(database.FullName());
         auto response = stub->CreateSession(context, request);
@@ -731,15 +738,26 @@ class SelectExperiment : public BasicExperiment<Traits> {
       int row_count = 0;
       google::spanner::v1::PartialResultSet result;
       std::vector<google::protobuf::Value> row;
-      grpc::ClientContext context;
-      auto stream = stub->ExecuteStreamingSql(context, request);
-      for (bool success = stream->Read(&result); success;
-           success = stream->Read(&result)) {
+      row.resize(ExperimentImpl<Traits>::kColumnCount);
+      auto stream =
+          stub->ExecuteStreamingSql(std::make_shared<grpc::ClientContext>(),
+                                    google::cloud::Options{}, request);
+      for (;;) {
+        auto read = stream->Read();
+        if (absl::holds_alternative<Status>(read)) {
+          auto status = absl::get<Status>(std::move(read));
+          auto const usage = timer.Sample();
+          samples.push_back(RowCpuSample{channel_count, thread_count, true,
+                                         row_count, usage.elapsed_time,
+                                         usage.cpu_time, std::move(status)});
+          break;
+        }
+        auto result =
+            absl::get<google::spanner::v1::PartialResultSet>(std::move(read));
         if (result.chunked_value()) {
           // We do not handle chunked values in the benchmark.
           continue;
         }
-        row.resize(ExperimentImpl<Traits>::kColumnCount);
         std::size_t index = 0;
         for (auto& value : *result.mutable_values()) {
           row[index] = std::move(value);
@@ -749,11 +767,6 @@ class SelectExperiment : public BasicExperiment<Traits> {
           }
         }
       }
-      auto final = stream->Finish();
-      auto const usage = timer.Sample();
-      samples.push_back(RowCpuSample{
-          channel_count, thread_count, true, row_count, usage.elapsed_time,
-          usage.cpu_time, google::cloud::MakeStatusFromRpcError(final)});
     }
     return samples;
   }
@@ -840,6 +853,7 @@ class UpdateExperiment : public BasicExperiment<Traits> {
       Status last_status;
       for (int i = 0; i != 10; ++i) {
         grpc::ClientContext context;
+        spanner_internal::RouteToLeader(context);  // always for CreateSession
         google::spanner::v1::CreateSessionRequest request{};
         request.set_database(database.FullName());
         auto response = stub->CreateSession(context, request);
@@ -1025,6 +1039,7 @@ class MutationExperiment : public BasicExperiment<Traits> {
       Status last_status;
       for (int i = 0; i != 10; ++i) {
         grpc::ClientContext context;
+        spanner_internal::RouteToLeader(context);  // always for CreateSession
         google::spanner::v1::CreateSessionRequest request{};
         request.set_database(database.FullName());
         auto response = stub->CreateSession(context, request);
@@ -1175,14 +1190,6 @@ class RunAllExperiment : public Experiment {
     for (auto& kv : AvailableExperiments()) {
       // Do not recurse, skip this experiment.
       if (kv.first == "run-all") continue;
-      // TODO(#5024): Remove this check when the emulator supports NUMERIC.
-      if (google::cloud::internal::GetEnv("SPANNER_EMULATOR_HOST")
-              .has_value()) {
-        auto pos = kv.first.rfind("-numeric");
-        if (pos != std::string::npos) {
-          continue;
-        }
-      }
 
       Config config = cfg;
       config.experiment = kv.first;
@@ -1228,30 +1235,30 @@ class RunAllExperiment : public Experiment {
 template <typename Trait>
 ExperimentFactory MakeReadFactory() {
   using G = ::google::cloud::internal::DefaultPRNG;
-  return [](G g) { return absl::make_unique<ReadExperiment<Trait>>(g); };
+  return [](G g) { return std::make_unique<ReadExperiment<Trait>>(g); };
 }
 
 template <typename Trait>
 ExperimentFactory MakeSelectFactory() {
   using G = ::google::cloud::internal::DefaultPRNG;
-  return [](G g) { return absl::make_unique<SelectExperiment<Trait>>(g); };
+  return [](G g) { return std::make_unique<SelectExperiment<Trait>>(g); };
 }
 
 template <typename Trait>
 ExperimentFactory MakeUpdateFactory() {
   using G = ::google::cloud::internal::DefaultPRNG;
-  return [](G g) { return absl::make_unique<UpdateExperiment<Trait>>(g); };
+  return [](G g) { return std::make_unique<UpdateExperiment<Trait>>(g); };
 }
 
 template <typename Trait>
 ExperimentFactory MakeMutationFactory() {
   using G = ::google::cloud::internal::DefaultPRNG;
-  return [](G g) { return absl::make_unique<MutationExperiment<Trait>>(g); };
+  return [](G g) { return std::make_unique<MutationExperiment<Trait>>(g); };
 }
 
 std::map<std::string, ExperimentFactory> AvailableExperiments() {
   auto make_run_all = [](google::cloud::internal::DefaultPRNG g) {
-    return absl::make_unique<RunAllExperiment>(g);
+    return std::make_unique<RunAllExperiment>(g);
   };
 
   return {
@@ -1359,13 +1366,6 @@ int main(int argc, char* argv[]) {
   request.set_create_statement(
       absl::StrCat("CREATE DATABASE `", database.database_id(), "`"));
   for (auto const& kv : available) {
-    // TODO(#5024): Remove this check when the emulator supports NUMERIC.
-    if (google::cloud::internal::GetEnv("SPANNER_EMULATOR_HOST").has_value()) {
-      auto pos = kv.first.rfind("-numeric");
-      if (pos != std::string::npos) {
-        continue;
-      }
-    }
     auto experiment = kv.second(generator);
     auto s = experiment->AdditionalDdlStatement();
     if (s.empty()) continue;
