@@ -17,6 +17,7 @@
 set -euo pipefail
 
 source "$(dirname "$0")/../../../../ci/lib/init.sh"
+source module /ci/cloudbuild/builds/lib/ctest.sh
 source module /ci/etc/integration-tests-config.sh
 source module /ci/lib/io.sh
 source module /google/cloud/spanner/ci/lib/spanner_emulator.sh
@@ -32,13 +33,17 @@ CMAKE_BINARY_DIR="$(realpath "${1}")"
 readonly CMAKE_BINARY_DIR
 shift
 
+if ctest::has_no_tests "${CMAKE_BINARY_DIR}" "^spanner_" "${ctest_args[@]}"; then
+  exit 0
+fi
+
 # Any additional arguments for the ctest invocation.
 ctest_args=("$@")
 
 cd "${CMAKE_BINARY_DIR}"
 
 # Starts the emulator and arranges to kill it.
-spanner_emulator::start
+spanner_emulator::start 8787
 trap spanner_emulator::kill EXIT
 
 ctest -R "^spanner_" "${ctest_args[@]}"
