@@ -14,12 +14,13 @@
 
 #include "google/cloud/spanner/client.h"
 #include "google/cloud/spanner/internal/connection_impl.h"
-#include "google/cloud/spanner/internal/spanner_stub.h"
+#include "google/cloud/spanner/internal/spanner_stub_factory.h"
 #include "google/cloud/spanner/internal/status_utils.h"
 #include "google/cloud/spanner/options.h"
 #include "google/cloud/spanner/retry_policy.h"
 #include "google/cloud/spanner/transaction.h"
 #include "google/cloud/backoff_policy.h"
+#include "google/cloud/credentials.h"
 #include "google/cloud/internal/getenv.h"
 #include "google/cloud/internal/retry_loop.h"
 #include "google/cloud/log.h"
@@ -34,163 +35,182 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
 namespace {
 
-// Extracts an option value as an `absl::optional`.
+// Extracts (removes and returns) an option value from `opts`. If
+// `OptionType` is not present, returns a default-constructed value.
 template <typename OptionType>
-absl::optional<typename OptionType::Type> OptOpt(Options const& opts) {
-  absl::optional<typename OptionType::Type> optopt;
-  if (opts.has<OptionType>()) optopt = opts.get<OptionType>();
-  return optopt;
+typename OptionType::Type ExtractOpt(Options& opts) {
+  auto option = internal::ExtractOption<OptionType>(opts);
+  return option ? *std::move(option) : typename OptionType::Type();
 }
 
 }  // namespace
 
 RowStream Client::Read(std::string table, KeySet keys,
-                       std::vector<std::string> columns,
-                       ReadOptions read_options) {
+                       std::vector<std::string> columns, Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
   return conn_->Read({spanner_internal::MakeSingleUseTransaction(
                           Transaction::ReadOnlyOptions()),
-                      std::move(table),
-                      std::move(keys),
-                      std::move(columns),
-                      std::move(read_options),
-                      {}});
+                      std::move(table), std::move(keys), std::move(columns),
+                      ToReadOptions(internal::CurrentOptions()), absl::nullopt,
+                      false, std::move(directed_read_option)});
 }
 
 RowStream Client::Read(Transaction::SingleUseOptions transaction_options,
                        std::string table, KeySet keys,
-                       std::vector<std::string> columns,
-                       ReadOptions read_options) {
+                       std::vector<std::string> columns, Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
   return conn_->Read({spanner_internal::MakeSingleUseTransaction(
                           std::move(transaction_options)),
-                      std::move(table),
-                      std::move(keys),
-                      std::move(columns),
-                      std::move(read_options),
-                      {}});
+                      std::move(table), std::move(keys), std::move(columns),
+                      ToReadOptions(internal::CurrentOptions()), absl::nullopt,
+                      false, std::move(directed_read_option)});
 }
 
 RowStream Client::Read(Transaction transaction, std::string table, KeySet keys,
-                       std::vector<std::string> columns,
-                       ReadOptions read_options) {
-  return conn_->Read({std::move(transaction),
-                      std::move(table),
-                      std::move(keys),
+                       std::vector<std::string> columns, Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->Read({std::move(transaction), std::move(table), std::move(keys),
                       std::move(columns),
-                      std::move(read_options),
-                      {}});
+                      ToReadOptions(internal::CurrentOptions()), absl::nullopt,
+                      false, std::move(directed_read_option)});
 }
 
-RowStream Client::Read(ReadPartition const& read_partition) {
-  return conn_->Read(spanner_internal::MakeReadParams(read_partition));
+RowStream Client::Read(ReadPartition const& read_partition, Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->Read(spanner_internal::MakeReadParams(
+      read_partition, std::move(directed_read_option)));
 }
 
 StatusOr<std::vector<ReadPartition>> Client::PartitionRead(
     Transaction transaction, std::string table, KeySet keys,
-    std::vector<std::string> columns, ReadOptions read_options,
-    PartitionOptions const& partition_options) {
-  return conn_->PartitionRead({{std::move(transaction),
-                                std::move(table),
-                                std::move(keys),
-                                std::move(columns),
-                                std::move(read_options),
-                                {}},
-                               partition_options});
+    std::vector<std::string> columns, Options opts) {
+  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
+  return conn_->PartitionRead(
+      {{std::move(transaction), std::move(table), std::move(keys),
+        std::move(columns), ToReadOptions(internal::CurrentOptions()),
+        absl::nullopt, false, DirectedReadOption::Type{}},
+       ToPartitionOptions(internal::CurrentOptions())});
 }
 
-RowStream Client::ExecuteQuery(SqlStatement statement,
-                               QueryOptions const& opts) {
-  return conn_->ExecuteQuery({spanner_internal::MakeSingleUseTransaction(
-                                  Transaction::ReadOnlyOptions()),
-                              std::move(statement),
-                              OverlayQueryOptions(opts),
-                              {}});
+RowStream Client::ExecuteQuery(SqlStatement statement, Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->ExecuteQuery(
+      {spanner_internal::MakeSingleUseTransaction(
+           Transaction::ReadOnlyOptions()),
+       std::move(statement), QueryOptions(internal::CurrentOptions()),
+       absl::nullopt, false, std::move(directed_read_option)});
 }
 
 RowStream Client::ExecuteQuery(
     Transaction::SingleUseOptions transaction_options, SqlStatement statement,
-    QueryOptions const& opts) {
-  return conn_->ExecuteQuery({spanner_internal::MakeSingleUseTransaction(
-                                  std::move(transaction_options)),
-                              std::move(statement),
-                              OverlayQueryOptions(opts),
-                              {}});
+    Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->ExecuteQuery(
+      {spanner_internal::MakeSingleUseTransaction(
+           std::move(transaction_options)),
+       std::move(statement), QueryOptions(internal::CurrentOptions()),
+       absl::nullopt, false, std::move(directed_read_option)});
 }
 
 RowStream Client::ExecuteQuery(Transaction transaction, SqlStatement statement,
-                               QueryOptions const& opts) {
-  return conn_->ExecuteQuery({std::move(transaction),
-                              std::move(statement),
-                              OverlayQueryOptions(opts),
-                              {}});
+                               Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->ExecuteQuery({std::move(transaction), std::move(statement),
+                              QueryOptions(internal::CurrentOptions()),
+                              absl::nullopt, false,
+                              std::move(directed_read_option)});
 }
 
-RowStream Client::ExecuteQuery(QueryPartition const& partition,
-                               QueryOptions const& opts) {
-  auto params = spanner_internal::MakeSqlParams(partition);
-  params.query_options = OverlayQueryOptions(opts);
-  return conn_->ExecuteQuery(std::move(params));
+RowStream Client::ExecuteQuery(QueryPartition const& partition, Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->ExecuteQuery(spanner_internal::MakeSqlParams(
+      partition, QueryOptions(internal::CurrentOptions()),
+      std::move(directed_read_option)));
 }
 
-ProfileQueryResult Client::ProfileQuery(SqlStatement statement,
-                                        QueryOptions const& opts) {
-  return conn_->ProfileQuery({spanner_internal::MakeSingleUseTransaction(
-                                  Transaction::ReadOnlyOptions()),
-                              std::move(statement),
-                              OverlayQueryOptions(opts),
-                              {}});
+ProfileQueryResult Client::ProfileQuery(SqlStatement statement, Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->ProfileQuery(
+      {spanner_internal::MakeSingleUseTransaction(
+           Transaction::ReadOnlyOptions()),
+       std::move(statement), QueryOptions(internal::CurrentOptions()),
+       absl::nullopt, false, std::move(directed_read_option)});
 }
 
 ProfileQueryResult Client::ProfileQuery(
     Transaction::SingleUseOptions transaction_options, SqlStatement statement,
-    QueryOptions const& opts) {
-  return conn_->ProfileQuery({spanner_internal::MakeSingleUseTransaction(
-                                  std::move(transaction_options)),
-                              std::move(statement),
-                              OverlayQueryOptions(opts),
-                              {}});
+    Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->ProfileQuery(
+      {spanner_internal::MakeSingleUseTransaction(
+           std::move(transaction_options)),
+       std::move(statement), QueryOptions(internal::CurrentOptions()),
+       absl::nullopt, false, std::move(directed_read_option)});
 }
 
 ProfileQueryResult Client::ProfileQuery(Transaction transaction,
-                                        SqlStatement statement,
-                                        QueryOptions const& opts) {
-  return conn_->ProfileQuery({std::move(transaction),
-                              std::move(statement),
-                              OverlayQueryOptions(opts),
-                              {}});
+                                        SqlStatement statement, Options opts) {
+  opts = internal::MergeOptions(std::move(opts), opts_);
+  auto directed_read_option = ExtractOpt<DirectedReadOption>(opts);
+  internal::OptionsSpan span(std::move(opts));
+  return conn_->ProfileQuery({std::move(transaction), std::move(statement),
+                              QueryOptions(internal::CurrentOptions()),
+                              absl::nullopt, false,
+                              std::move(directed_read_option)});
 }
 
 StatusOr<std::vector<QueryPartition>> Client::PartitionQuery(
-    Transaction transaction, SqlStatement statement,
-    PartitionOptions const& partition_options) {
+    Transaction transaction, SqlStatement statement, Options opts) {
+  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
   return conn_->PartitionQuery(
-      {std::move(transaction), std::move(statement), partition_options});
+      {std::move(transaction), std::move(statement),
+       ToPartitionOptions(internal::CurrentOptions())});
 }
 
 StatusOr<DmlResult> Client::ExecuteDml(Transaction transaction,
-                                       SqlStatement statement,
-                                       QueryOptions const& opts) {
-  return conn_->ExecuteDml({std::move(transaction),
-                            std::move(statement),
-                            OverlayQueryOptions(opts),
-                            {}});
+                                       SqlStatement statement, Options opts) {
+  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
+  return conn_->ExecuteDml({std::move(transaction), std::move(statement),
+                            QueryOptions(internal::CurrentOptions()),
+                            absl::nullopt, false, DirectedReadOption::Type{}});
 }
 
 StatusOr<ProfileDmlResult> Client::ProfileDml(Transaction transaction,
                                               SqlStatement statement,
-                                              QueryOptions const& opts) {
-  return conn_->ProfileDml({std::move(transaction),
-                            std::move(statement),
-                            OverlayQueryOptions(opts),
-                            {}});
+                                              Options opts) {
+  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
+  return conn_->ProfileDml({std::move(transaction), std::move(statement),
+                            QueryOptions(internal::CurrentOptions()),
+                            absl::nullopt, false, DirectedReadOption::Type{}});
 }
 
 StatusOr<ExecutionPlan> Client::AnalyzeSql(Transaction transaction,
                                            SqlStatement statement,
-                                           QueryOptions const& opts) {
-  return conn_->AnalyzeSql({std::move(transaction),
-                            std::move(statement),
-                            OverlayQueryOptions(opts),
-                            {}});
+                                           Options opts) {
+  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
+  return conn_->AnalyzeSql({std::move(transaction), std::move(statement),
+                            QueryOptions(internal::CurrentOptions()),
+                            absl::nullopt, false, DirectedReadOption::Type{}});
 }
 
 StatusOr<BatchDmlResult> Client::ExecuteBatchDml(
@@ -212,9 +232,9 @@ StatusOr<CommitResult> Client::Commit(
   using RerunnablePolicy = spanner_internal::SafeTransactionRerun;
 
   auto const txn_opts = Transaction::ReadWriteOptions().WithTag(
-      OptOpt<TransactionTagOption>(internal::CurrentOptions()));
+      internal::FetchOption<TransactionTagOption>(internal::CurrentOptions()));
   Transaction txn = MakeReadWriteTransaction(txn_opts);
-  for (int rerun = 0;; ++rerun) {
+  for (;;) {
     StatusOr<Mutations> mutations;
 #if GOOGLE_CLOUD_CPP_HAVE_EXCEPTIONS
     try {
@@ -263,7 +283,7 @@ StatusOr<CommitResult> Client::Commit(
       spanner_internal::Visit(
           txn, [](spanner_internal::SessionHolder& s,
                   StatusOr<google::spanner::v1::TransactionSelector> const&,
-                  std::string const&, std::int64_t) {
+                  spanner_internal::TransactionContext const&) {
             if (s) s->set_bad();
             return true;
           });
@@ -310,42 +330,43 @@ StatusOr<CommitResult> Client::Commit(Transaction transaction,
                         CommitOptions(internal::CurrentOptions())});
 }
 
+StatusOr<CommitResult> Client::CommitAtLeastOnce(
+    Transaction::ReadWriteOptions transaction_options, Mutations mutations,
+    Options opts) {
+  // Note: This implementation differs from `CommitAtLeastOnce({mutations})`
+  // for historical reasons.
+  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
+  return conn_->Commit({spanner_internal::MakeSingleUseCommitTransaction(
+                            std::move(transaction_options)),
+                        std::move(mutations),
+                        CommitOptions(internal::CurrentOptions())});
+}
+
+BatchedCommitResultStream Client::CommitAtLeastOnce(
+    std::vector<Mutations> mutation_groups, Options opts) {
+  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
+  return conn_->BatchWrite(
+      {std::move(mutation_groups), internal::CurrentOptions()});
+}
+
 Status Client::Rollback(Transaction transaction, Options opts) {
   internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
   return conn_->Rollback({std::move(transaction)});
 }
 
 StatusOr<PartitionedDmlResult> Client::ExecutePartitionedDml(
-    SqlStatement statement, QueryOptions const& opts) {
+    SqlStatement statement, Options opts) {
+  internal::OptionsSpan span(internal::MergeOptions(std::move(opts), opts_));
   return conn_->ExecutePartitionedDml(
-      {std::move(statement), OverlayQueryOptions(opts)});
-}
-
-// Returns a QueryOptions that has each attribute set according to
-// the hierarchy that options specified at the function call (i.e.,
-// `preferred`) are preferred, followed by options set at the Client
-// level (i.e., `opts_.query_options()`), followed by some environment
-// variables. If none are set, the attribute's optional will be unset
-// and nothing will be included in the proto sent to Spanner, in which
-// case the Database default will be used.
-QueryOptions Client::OverlayQueryOptions(QueryOptions const& preferred) {
-  // GetEnv() is not super fast, so we look it up once and cache it.
-  static auto const* const kOptimizerVersionEnvValue =
-      new auto(google::cloud::internal::GetEnv("SPANNER_OPTIMIZER_VERSION"));
-  static auto const* const kOptimizerStatisticsPackageEnvValue = new auto(
-      google::cloud::internal::GetEnv("SPANNER_OPTIMIZER_STATISTICS_PACKAGE"));
-
-  return spanner_internal::OverlayQueryOptions(
-      preferred, query_opts_, *kOptimizerVersionEnvValue,
-      *kOptimizerStatisticsPackageEnvValue);
+      {std::move(statement), QueryOptions(internal::CurrentOptions())});
 }
 
 std::shared_ptr<spanner::Connection> MakeConnection(spanner::Database const& db,
                                                     Options opts) {
   internal::CheckExpectedOptions<
-      CommonOptionList, GrpcOptionList, SessionPoolOptionList,
-      spanner_internal::SessionPoolClockOption, SpannerPolicyOptionList>(
-      opts, __func__);
+      CommonOptionList, GrpcOptionList, UnifiedCredentialsOptionList,
+      SessionPoolOptionList, spanner_internal::SessionPoolClockOption,
+      SpannerPolicyOptionList>(opts, __func__);
   opts = spanner_internal::DefaultOptions(std::move(opts));
 
   auto background = internal::MakeBackgroundThreadsFactory(opts)();
@@ -384,57 +405,5 @@ std::shared_ptr<Connection> MakeConnection(
 
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace spanner
-
-namespace spanner_internal {
-GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
-
-// Returns a QueryOptions that has each attribute set according to the
-// preferred/fallback/environment hierarchy.
-spanner::QueryOptions OverlayQueryOptions(
-    spanner::QueryOptions const& preferred,
-    spanner::QueryOptions const& fallback,
-    absl::optional<std::string> const& optimizer_version_env,
-    absl::optional<std::string> const& optimizer_statistics_package_env) {
-  spanner::QueryOptions opts;
-
-  // Choose the `optimizer_version` option.
-  if (preferred.optimizer_version().has_value()) {
-    opts.set_optimizer_version(preferred.optimizer_version());
-  } else if (fallback.optimizer_version().has_value()) {
-    opts.set_optimizer_version(fallback.optimizer_version());
-  } else if (optimizer_version_env.has_value()) {
-    opts.set_optimizer_version(*optimizer_version_env);
-  }
-
-  // Choose the `optimizer_statistics_package` option.
-  if (preferred.optimizer_statistics_package().has_value()) {
-    opts.set_optimizer_statistics_package(
-        preferred.optimizer_statistics_package());
-  } else if (fallback.optimizer_statistics_package().has_value()) {
-    opts.set_optimizer_statistics_package(
-        fallback.optimizer_statistics_package());
-  } else if (optimizer_statistics_package_env.has_value()) {
-    opts.set_optimizer_statistics_package(*optimizer_statistics_package_env);
-  }
-
-  // Choose the `request_priority` option.
-  if (preferred.request_priority().has_value()) {
-    opts.set_request_priority(preferred.request_priority());
-  } else if (fallback.request_priority().has_value()) {
-    opts.set_request_priority(fallback.request_priority());
-  }
-
-  // Choose the `request_tag` option.
-  if (preferred.request_tag().has_value()) {
-    opts.set_request_tag(preferred.request_tag());
-  } else if (fallback.request_tag().has_value()) {
-    opts.set_request_tag(fallback.request_tag());
-  }
-
-  return opts;
-}
-
-GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
-}  // namespace spanner_internal
 }  // namespace cloud
 }  // namespace google
