@@ -31,6 +31,8 @@ namespace spanner {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
+namespace gsai = ::google::spanner::admin::instance;
+
 using ::google::cloud::testing_util::IsProtoEqual;
 using ::google::cloud::testing_util::StatusIs;
 using ::google::protobuf::TextFormat;
@@ -38,9 +40,6 @@ using ::testing::AtLeast;
 using ::testing::ElementsAre;
 using ::testing::Return;
 
-namespace gcsa = ::google::spanner::admin::instance::v1;
-namespace giam = ::google::iam::v1;
-
 // Create a `Connection` suitable for use in tests that continue retrying
 // until the retry policy is exhausted - attempting that with the default
 // policies would take too long (30 minutes).
@@ -72,23 +71,25 @@ TEST(InstanceAdminConnectionTest, GetInstanceSuccess) {
     display_name: "test display name"
     node_count: 7
     state: CREATING
+    create_time { seconds: 1657025805 nanos: 967908745 }
+    update_time { seconds: 1657025805 nanos: 967908745 }
   )pb";
-  gcsa::Instance expected_instance;
+  gsai::v1::Instance expected_instance;
   ASSERT_TRUE(TextFormat::ParseFromString(kInstanceText, &expected_instance));
 
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
   EXPECT_CALL(*mock, GetInstance)
       .WillOnce([&expected_name](grpc::ClientContext&,
-                                 gcsa::GetInstanceRequest const& request) {
+                                 gsai::v1::GetInstanceRequest const& request) {
         EXPECT_EQ(expected_name, request.name());
         return Status(StatusCode::kUnavailable, "try-again");
       })
-      .WillOnce(
-          [&expected_name, &expected_instance](
-              grpc::ClientContext&, gcsa::GetInstanceRequest const& request) {
-            EXPECT_EQ(expected_name, request.name());
-            return expected_instance;
-          });
+      .WillOnce([&expected_name, &expected_instance](
+                    grpc::ClientContext&,
+                    gsai::v1::GetInstanceRequest const& request) {
+        EXPECT_EQ(expected_name, request.name());
+        return expected_instance;
+      });
 
   auto conn = MakeLimitedRetryConnection(mock);
   auto actual = conn->GetInstance({expected_name});
@@ -121,9 +122,8 @@ TEST(InstanceAdminClientTest, CreateInstanceSuccess) {
       "projects/test-project/instances/test-instance";
 
   EXPECT_CALL(*mock, AsyncCreateInstance)
-      .WillOnce([&expected_name](CompletionQueue&,
-                                 std::unique_ptr<grpc::ClientContext>,
-                                 gcsa::CreateInstanceRequest const& r) {
+      .WillOnce([&expected_name](CompletionQueue&, auto,
+                                 gsai::v1::CreateInstanceRequest const& r) {
         EXPECT_EQ("test-instance", r.instance_id());
         EXPECT_EQ("projects/test-project", r.parent());
         auto const& instance = r.instance();
@@ -139,18 +139,18 @@ TEST(InstanceAdminClientTest, CreateInstanceSuccess) {
         return make_ready_future(make_status_or(op));
       });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&expected_name](
-                    CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
-                    google::longrunning::GetOperationRequest const& r) {
-        EXPECT_EQ("test-operation-name", r.name());
-        google::longrunning::Operation op;
-        op.set_name(r.name());
-        op.set_done(true);
-        gcsa::Instance instance;
-        instance.set_name(expected_name);
-        op.mutable_response()->PackFrom(instance);
-        return make_ready_future(make_status_or(op));
-      });
+      .WillOnce(
+          [&expected_name](CompletionQueue&, auto,
+                           google::longrunning::GetOperationRequest const& r) {
+            EXPECT_EQ("test-operation-name", r.name());
+            google::longrunning::Operation op;
+            op.set_name(r.name());
+            op.set_done(true);
+            gsai::v1::Instance instance;
+            instance.set_name(expected_name);
+            op.mutable_response()->PackFrom(instance);
+            return make_ready_future(make_status_or(op));
+          });
 
   auto conn = MakeLimitedRetryConnection(std::move(mock));
   Instance in("test-project", "test-instance");
@@ -170,11 +170,11 @@ TEST(InstanceAdminClientTest, CreateInstanceError) {
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
 
   EXPECT_CALL(*mock, AsyncCreateInstance)
-      .WillOnce([](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
-                   gcsa::CreateInstanceRequest const&) {
-        return make_ready_future(StatusOr<google::longrunning::Operation>(
-            Status(StatusCode::kPermissionDenied, "uh-oh")));
-      });
+      .WillOnce(
+          [](CompletionQueue&, auto, gsai::v1::CreateInstanceRequest const&) {
+            return make_ready_future(StatusOr<google::longrunning::Operation>(
+                Status(StatusCode::kPermissionDenied, "uh-oh")));
+          });
 
   auto conn = MakeLimitedRetryConnection(std::move(mock));
   Instance in("test-project", "test-instance");
@@ -193,14 +193,13 @@ TEST(InstanceAdminClientTest, UpdateInstanceSuccess) {
   std::string expected_name = "projects/test-project/instances/test-instance";
 
   EXPECT_CALL(*mock, AsyncUpdateInstance)
-      .WillOnce([](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
-                   gcsa::UpdateInstanceRequest const&) {
-        return make_ready_future(StatusOr<google::longrunning::Operation>(
-            Status(StatusCode::kUnavailable, "try-again")));
-      })
-      .WillOnce([&expected_name](CompletionQueue&,
-                                 std::unique_ptr<grpc::ClientContext>,
-                                 gcsa::UpdateInstanceRequest const& r) {
+      .WillOnce(
+          [](CompletionQueue&, auto, gsai::v1::UpdateInstanceRequest const&) {
+            return make_ready_future(StatusOr<google::longrunning::Operation>(
+                Status(StatusCode::kUnavailable, "try-again")));
+          })
+      .WillOnce([&expected_name](CompletionQueue&, auto,
+                                 gsai::v1::UpdateInstanceRequest const& r) {
         EXPECT_EQ(expected_name, r.instance().name());
         google::longrunning::Operation op;
         op.set_name("test-operation-name");
@@ -208,21 +207,21 @@ TEST(InstanceAdminClientTest, UpdateInstanceSuccess) {
         return make_ready_future(make_status_or(op));
       });
   EXPECT_CALL(*mock, AsyncGetOperation)
-      .WillOnce([&expected_name](
-                    CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
-                    google::longrunning::GetOperationRequest const& r) {
-        EXPECT_EQ("test-operation-name", r.name());
-        google::longrunning::Operation op;
-        op.set_name(r.name());
-        op.set_done(true);
-        gcsa::Instance instance;
-        instance.set_name(expected_name);
-        op.mutable_response()->PackFrom(instance);
-        return make_ready_future(make_status_or(op));
-      });
+      .WillOnce(
+          [&expected_name](CompletionQueue&, auto,
+                           google::longrunning::GetOperationRequest const& r) {
+            EXPECT_EQ("test-operation-name", r.name());
+            google::longrunning::Operation op;
+            op.set_name(r.name());
+            op.set_done(true);
+            gsai::v1::Instance instance;
+            instance.set_name(expected_name);
+            op.mutable_response()->PackFrom(instance);
+            return make_ready_future(make_status_or(op));
+          });
 
   auto conn = MakeLimitedRetryConnection(std::move(mock));
-  gcsa::UpdateInstanceRequest req;
+  gsai::v1::UpdateInstanceRequest req;
   req.mutable_instance()->set_name(expected_name);
   auto fut = conn->UpdateInstance({req});
   auto instance = fut.get();
@@ -235,14 +234,14 @@ TEST(InstanceAdminClientTest, UpdateInstancePermanentFailure) {
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
 
   EXPECT_CALL(*mock, AsyncUpdateInstance)
-      .WillOnce([](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
-                   gcsa::UpdateInstanceRequest const&) {
-        return make_ready_future(StatusOr<google::longrunning::Operation>(
-            Status(StatusCode::kPermissionDenied, "uh-oh")));
-      });
+      .WillOnce(
+          [](CompletionQueue&, auto, gsai::v1::UpdateInstanceRequest const&) {
+            return make_ready_future(StatusOr<google::longrunning::Operation>(
+                Status(StatusCode::kPermissionDenied, "uh-oh")));
+          });
 
   auto conn = MakeLimitedRetryConnection(std::move(mock));
-  auto fut = conn->UpdateInstance({gcsa::UpdateInstanceRequest()});
+  auto fut = conn->UpdateInstance({gsai::v1::UpdateInstanceRequest()});
   auto instance = fut.get();
   EXPECT_THAT(instance, StatusIs(StatusCode::kPermissionDenied));
 }
@@ -252,13 +251,13 @@ TEST(InstanceAdminClientTest, UpdateInstanceTooManyTransients) {
 
   EXPECT_CALL(*mock, AsyncUpdateInstance)
       .Times(AtLeast(2))
-      .WillRepeatedly([](CompletionQueue&, std::unique_ptr<grpc::ClientContext>,
-                         gcsa::UpdateInstanceRequest const&) {
-        return make_ready_future(StatusOr<google::longrunning::Operation>(
-            Status(StatusCode::kUnavailable, "try-again")));
-      });
+      .WillRepeatedly(
+          [](CompletionQueue&, auto, gsai::v1::UpdateInstanceRequest const&) {
+            return make_ready_future(StatusOr<google::longrunning::Operation>(
+                Status(StatusCode::kUnavailable, "try-again")));
+          });
   auto conn = MakeLimitedRetryConnection(std::move(mock));
-  auto fut = conn->UpdateInstance({gcsa::UpdateInstanceRequest()});
+  auto fut = conn->UpdateInstance({gsai::v1::UpdateInstanceRequest()});
   auto instance = fut.get();
   EXPECT_THAT(instance, StatusIs(StatusCode::kUnavailable));
 }
@@ -269,20 +268,22 @@ TEST(InstanceAdminConnectionTest, DeleteInstanceSuccess) {
 
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
   EXPECT_CALL(*mock, DeleteInstance)
-      .WillOnce([&expected_name](grpc::ClientContext&,
-                                 gcsa::DeleteInstanceRequest const& request) {
-        EXPECT_EQ(expected_name, request.name());
-        return Status(StatusCode::kUnavailable, "try-again");
-      })
-      .WillOnce([&expected_name](grpc::ClientContext&,
-                                 gcsa::DeleteInstanceRequest const& request) {
-        EXPECT_EQ(expected_name, request.name());
-        return Status();
-      });
+      .WillOnce(
+          [&expected_name](grpc::ClientContext&,
+                           gsai::v1::DeleteInstanceRequest const& request) {
+            EXPECT_EQ(expected_name, request.name());
+            return Status(StatusCode::kUnavailable, "try-again");
+          })
+      .WillOnce(
+          [&expected_name](grpc::ClientContext&,
+                           gsai::v1::DeleteInstanceRequest const& request) {
+            EXPECT_EQ(expected_name, request.name());
+            return Status();
+          });
 
   auto conn = MakeLimitedRetryConnection(mock);
   auto status = conn->DeleteInstance({expected_name});
-  ASSERT_STATUS_OK(status);
+  EXPECT_STATUS_OK(status);
 }
 
 TEST(InstanceAdminConnectionTest, DeleteInstancePermanentFailure) {
@@ -318,7 +319,7 @@ TEST(InstanceAdminConnectionTest, GetInstanceConfigSuccess) {
     }
     leader_options: "location"
   )pb";
-  gcsa::InstanceConfig expected_instance_config;
+  gsai::v1::InstanceConfig expected_instance_config;
   ASSERT_TRUE(
       TextFormat::ParseFromString(kConfigText, &expected_instance_config));
 
@@ -326,13 +327,13 @@ TEST(InstanceAdminConnectionTest, GetInstanceConfigSuccess) {
   EXPECT_CALL(*mock, GetInstanceConfig)
       .WillOnce(
           [&expected_name](grpc::ClientContext&,
-                           gcsa::GetInstanceConfigRequest const& request) {
+                           gsai::v1::GetInstanceConfigRequest const& request) {
             EXPECT_EQ(expected_name, request.name());
             return Status(StatusCode::kUnavailable, "try-again");
           })
       .WillOnce([&expected_name, &expected_instance_config](
                     grpc::ClientContext&,
-                    gcsa::GetInstanceConfigRequest const& request) {
+                    gsai::v1::GetInstanceConfigRequest const& request) {
         EXPECT_EQ(expected_name, request.name());
         return expected_instance_config;
       });
@@ -397,7 +398,7 @@ TEST(InstanceAdminConnectionTest, ListInstanceConfigsSuccess) {
         leader_options: "location3"
       )pb",
   };
-  gcsa::InstanceConfig expected_instance_configs[3];
+  gsai::v1::InstanceConfig expected_instance_configs[3];
   ASSERT_TRUE(TextFormat::ParseFromString(kInstanceConfigText[0],
                                           &expected_instance_configs[0]));
   ASSERT_TRUE(TextFormat::ParseFromString(kInstanceConfigText[1],
@@ -409,35 +410,35 @@ TEST(InstanceAdminConnectionTest, ListInstanceConfigsSuccess) {
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
   EXPECT_CALL(*mock, ListInstanceConfigs)
       .WillOnce([&](grpc::ClientContext&,
-                    gcsa::ListInstanceConfigsRequest const& request) {
+                    gsai::v1::ListInstanceConfigsRequest const& request) {
         EXPECT_EQ(expected_parent, request.parent());
         EXPECT_TRUE(request.page_token().empty());
         return Status(StatusCode::kUnavailable, "try-again");
       })
       .WillOnce([&](grpc::ClientContext&,
-                    gcsa::ListInstanceConfigsRequest const& request) {
+                    gsai::v1::ListInstanceConfigsRequest const& request) {
         EXPECT_EQ(expected_parent, request.parent());
         EXPECT_TRUE(request.page_token().empty());
 
-        gcsa::ListInstanceConfigsResponse response;
+        gsai::v1::ListInstanceConfigsResponse response;
         response.set_next_page_token("p1");
         *response.add_instance_configs() = expected_instance_configs[0];
         *response.add_instance_configs() = expected_instance_configs[1];
         return response;
       })
       .WillOnce([&](grpc::ClientContext&,
-                    gcsa::ListInstanceConfigsRequest const& request) {
+                    gsai::v1::ListInstanceConfigsRequest const& request) {
         EXPECT_EQ(expected_parent, request.parent());
         EXPECT_EQ("p1", request.page_token());
 
-        gcsa::ListInstanceConfigsResponse response;
+        gsai::v1::ListInstanceConfigsResponse response;
         response.clear_next_page_token();
         *response.add_instance_configs() = expected_instance_configs[2];
         return response;
       });
 
   auto conn = MakeLimitedRetryConnection(mock);
-  std::vector<gcsa::InstanceConfig> actual_instance_configs;
+  std::vector<gsai::v1::InstanceConfig> actual_instance_configs;
   for (auto const& instance_config :
        conn->ListInstanceConfigs({"test-project"})) {
     ASSERT_STATUS_OK(instance_config);
@@ -479,36 +480,36 @@ TEST(InstanceAdminConnectionTest, ListInstancesSuccess) {
 
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
   EXPECT_CALL(*mock, ListInstances)
-      .WillOnce(
-          [&](grpc::ClientContext&, gcsa::ListInstancesRequest const& request) {
-            EXPECT_EQ(expected_parent, request.parent());
-            EXPECT_EQ("labels.test-key:test-value", request.filter());
-            EXPECT_TRUE(request.page_token().empty());
-            return Status(StatusCode::kUnavailable, "try-again");
-          })
-      .WillOnce(
-          [&](grpc::ClientContext&, gcsa::ListInstancesRequest const& request) {
-            EXPECT_EQ(expected_parent, request.parent());
-            EXPECT_EQ("labels.test-key:test-value", request.filter());
-            EXPECT_TRUE(request.page_token().empty());
+      .WillOnce([&](grpc::ClientContext&,
+                    gsai::v1::ListInstancesRequest const& request) {
+        EXPECT_EQ(expected_parent, request.parent());
+        EXPECT_EQ("labels.test-key:test-value", request.filter());
+        EXPECT_TRUE(request.page_token().empty());
+        return Status(StatusCode::kUnavailable, "try-again");
+      })
+      .WillOnce([&](grpc::ClientContext&,
+                    gsai::v1::ListInstancesRequest const& request) {
+        EXPECT_EQ(expected_parent, request.parent());
+        EXPECT_EQ("labels.test-key:test-value", request.filter());
+        EXPECT_TRUE(request.page_token().empty());
 
-            gcsa::ListInstancesResponse response;
-            response.set_next_page_token("p1");
-            response.add_instances()->set_name("i1");
-            response.add_instances()->set_name("i2");
-            return response;
-          })
-      .WillOnce(
-          [&](grpc::ClientContext&, gcsa::ListInstancesRequest const& request) {
-            EXPECT_EQ(expected_parent, request.parent());
-            EXPECT_EQ("labels.test-key:test-value", request.filter());
-            EXPECT_EQ("p1", request.page_token());
+        gsai::v1::ListInstancesResponse response;
+        response.set_next_page_token("p1");
+        response.add_instances()->set_name("i1");
+        response.add_instances()->set_name("i2");
+        return response;
+      })
+      .WillOnce([&](grpc::ClientContext&,
+                    gsai::v1::ListInstancesRequest const& request) {
+        EXPECT_EQ(expected_parent, request.parent());
+        EXPECT_EQ("labels.test-key:test-value", request.filter());
+        EXPECT_EQ("p1", request.page_token());
 
-            gcsa::ListInstancesResponse response;
-            response.clear_next_page_token();
-            response.add_instances()->set_name("i3");
-            return response;
-          });
+        gsai::v1::ListInstancesResponse response;
+        response.clear_next_page_token();
+        response.add_instances()->set_name("i3");
+        return response;
+      });
 
   auto conn = MakeLimitedRetryConnection(mock);
   std::vector<std::string> actual_names;
@@ -550,15 +551,17 @@ TEST(InstanceAdminConnectionTest, GetIamPolicySuccess) {
 
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
   EXPECT_CALL(*mock, GetIamPolicy)
-      .WillOnce([&expected_name](grpc::ClientContext&,
-                                 giam::GetIamPolicyRequest const& request) {
+      .WillOnce([&expected_name](
+                    grpc::ClientContext&,
+                    google::iam::v1::GetIamPolicyRequest const& request) {
         EXPECT_EQ(expected_name, request.resource());
         return Status(StatusCode::kUnavailable, "try-again");
       })
-      .WillOnce([&expected_name](grpc::ClientContext&,
-                                 giam::GetIamPolicyRequest const& request) {
+      .WillOnce([&expected_name](
+                    grpc::ClientContext&,
+                    google::iam::v1::GetIamPolicyRequest const& request) {
         EXPECT_EQ(expected_name, request.resource());
-        giam::Policy response;
+        google::iam::v1::Policy response;
         auto& binding = *response.add_bindings();
         binding.set_role("roles/spanner.databaseReader");
         binding.add_members("user:test-account@example.com");
@@ -606,25 +609,26 @@ TEST(InstanceAdminConnectionTest, SetIamPolicySuccess) {
       members: "user:test-user-2@example.com"
     }
   )pb";
-  giam::Policy expected_policy;
+  google::iam::v1::Policy expected_policy;
   ASSERT_TRUE(TextFormat::ParseFromString(kPolicyText, &expected_policy));
 
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
   EXPECT_CALL(*mock, SetIamPolicy)
-      .WillOnce([&expected_name](grpc::ClientContext&,
-                                 giam::SetIamPolicyRequest const& request) {
+      .WillOnce([&expected_name](
+                    grpc::ClientContext&,
+                    google::iam::v1::SetIamPolicyRequest const& request) {
         EXPECT_EQ(expected_name, request.resource());
         return Status(StatusCode::kUnavailable, "try-again");
       })
-      .WillOnce(
-          [&expected_name, &expected_policy](
-              grpc::ClientContext&, giam::SetIamPolicyRequest const& request) {
-            EXPECT_EQ(expected_name, request.resource());
-            EXPECT_THAT(request.policy(), IsProtoEqual(expected_policy));
-            giam::Policy response = expected_policy;
-            response.set_etag("response-etag");
-            return response;
-          });
+      .WillOnce([&expected_name, &expected_policy](
+                    grpc::ClientContext&,
+                    google::iam::v1::SetIamPolicyRequest const& request) {
+        EXPECT_EQ(expected_name, request.resource());
+        EXPECT_THAT(request.policy(), IsProtoEqual(expected_policy));
+        google::iam::v1::Policy response = expected_policy;
+        response.set_etag("response-etag");
+        return response;
+      });
 
   auto conn = MakeLimitedRetryConnection(mock);
   auto actual = conn->SetIamPolicy({expected_name, expected_policy});
@@ -675,20 +679,20 @@ TEST(InstanceAdminConnectionTest, TestIamPermissionsSuccess) {
 
   auto mock = std::make_shared<spanner_testing::MockInstanceAdminStub>();
   EXPECT_CALL(*mock, TestIamPermissions)
-      .WillOnce(
-          [&expected_name](grpc::ClientContext&,
-                           giam::TestIamPermissionsRequest const& request) {
-            EXPECT_EQ(expected_name, request.resource());
-            return Status(StatusCode::kUnavailable, "try-again");
-          })
-      .WillOnce(
-          [&expected_name](grpc::ClientContext&,
-                           giam::TestIamPermissionsRequest const& request) {
-            EXPECT_EQ(expected_name, request.resource());
-            giam::TestIamPermissionsResponse response;
-            response.add_permissions("test.permission2");
-            return response;
-          });
+      .WillOnce([&expected_name](
+                    grpc::ClientContext&,
+                    google::iam::v1::TestIamPermissionsRequest const& request) {
+        EXPECT_EQ(expected_name, request.resource());
+        return Status(StatusCode::kUnavailable, "try-again");
+      })
+      .WillOnce([&expected_name](
+                    grpc::ClientContext&,
+                    google::iam::v1::TestIamPermissionsRequest const& request) {
+        EXPECT_EQ(expected_name, request.resource());
+        google::iam::v1::TestIamPermissionsResponse response;
+        response.add_permissions("test.permission2");
+        return response;
+      });
 
   auto conn = MakeLimitedRetryConnection(mock);
   auto actual = conn->TestIamPermissions(
