@@ -26,6 +26,7 @@
 #include "google/cloud/internal/random.h"
 #include "google/cloud/testing_util/integration_test.h"
 #include "google/cloud/testing_util/status_matchers.h"
+#include "absl/strings/match.h"
 #include "absl/time/time.h"
 #include <gmock/gmock.h>
 #include <chrono>
@@ -57,10 +58,10 @@ std::string const& ProjectId() {
 
 bool RunSlowBackupTests() {
   static bool run_slow_backup_tests =
-      google::cloud::internal::GetEnv(
-          "GOOGLE_CLOUD_CPP_SPANNER_SLOW_INTEGRATION_TESTS")
-          .value_or("")
-          .find("backup") != std::string::npos;
+      absl::StrContains(google::cloud::internal::GetEnv(
+                            "GOOGLE_CLOUD_CPP_SPANNER_SLOW_INTEGRATION_TESTS")
+                            .value_or(""),
+                        "backup");
   return run_slow_backup_tests;
 }
 
@@ -82,7 +83,7 @@ class BackupExtraIntegrationTest
                                      std::chrono::minutes(1), 2.0)
                 .clone(),
             GenericPollingPolicy<>(
-                LimitedTimeRetryPolicy(std::chrono::hours(3)),
+                LimitedTimeRetryPolicy(std::chrono::minutes(90)),
                 ExponentialBackoffPolicy(std::chrono::seconds(1),
                                          std::chrono::minutes(1), 2.0))
                 .clone())) {}
@@ -100,7 +101,7 @@ TEST_F(BackupExtraIntegrationTest, BackupRestoreWithVersionTime) {
       generator_, ProjectId(),
       "(labels.restore-database-partition:legacy-extra OR"
       " labels.restore-database-partition:all)");
-  ASSERT_THAT(instance_id, IsOk()) << instance_id.status();
+  ASSERT_THAT(instance_id, IsOk());
   Instance in(ProjectId(), *instance_id);
   Database db(in, spanner_testing::RandomDatabaseName(generator_));
 
@@ -113,12 +114,11 @@ TEST_F(BackupExtraIntegrationTest, BackupRestoreWithVersionTime) {
       ") PRIMARY KEY (Name)"};
   auto database =
       database_admin_client_.CreateDatabase(db, extra_statements).get();
-  if (Emulator()) {
-    // TODO(#5479): Awaiting emulator support for version_retention_period.
+  if (Emulator()) {  // version_retention_period
     EXPECT_THAT(database, Not(IsOk()));
     return;
   }
-  ASSERT_THAT(database, IsOk()) << database.status();
+  ASSERT_THAT(database, IsOk());
   auto create_time =
       MakeTimestamp(database->create_time()).value().get<absl::Time>().value();
 
@@ -238,7 +238,7 @@ TEST_F(BackupExtraIntegrationTest, BackupRestoreWithVersionTime) {
 TEST_F(BackupExtraIntegrationTest, BackupWithExpiredVersionTime) {
   auto instance_id =
       spanner_testing::PickRandomInstance(generator_, ProjectId());
-  ASSERT_THAT(instance_id, IsOk()) << instance_id.status();
+  ASSERT_THAT(instance_id, IsOk());
   Instance in(ProjectId(), *instance_id);
   Database db(in, spanner_testing::RandomDatabaseName(generator_));
 
@@ -247,12 +247,11 @@ TEST_F(BackupExtraIntegrationTest, BackupWithExpiredVersionTime) {
                    "` SET OPTIONS (version_retention_period='1h')")};
   auto database =
       database_admin_client_.CreateDatabase(db, extra_statements).get();
-  if (Emulator()) {
-    // TODO(#5479): Awaiting emulator support for version_retention_period.
+  if (Emulator()) {  // version_retention_period
     EXPECT_THAT(database, Not(IsOk()));
     return;
   }
-  ASSERT_THAT(database, IsOk()) << database.status();
+  ASSERT_THAT(database, IsOk());
 
   auto create_time =
       MakeTimestamp(database->create_time()).value().get<absl::Time>().value();
@@ -276,7 +275,7 @@ TEST_F(BackupExtraIntegrationTest, BackupWithExpiredVersionTime) {
 TEST_F(BackupExtraIntegrationTest, BackupWithFutureVersionTime) {
   auto instance_id =
       spanner_testing::PickRandomInstance(generator_, ProjectId());
-  ASSERT_THAT(instance_id, IsOk()) << instance_id.status();
+  ASSERT_THAT(instance_id, IsOk());
   Instance in(ProjectId(), *instance_id);
   Database db(in, spanner_testing::RandomDatabaseName(generator_));
 
@@ -285,12 +284,11 @@ TEST_F(BackupExtraIntegrationTest, BackupWithFutureVersionTime) {
                    "` SET OPTIONS (version_retention_period='1h')")};
   auto database =
       database_admin_client_.CreateDatabase(db, extra_statements).get();
-  if (Emulator()) {
-    // TODO(#5479): Awaiting emulator support for version_retention_period.
+  if (Emulator()) {  // version_retention_period
     EXPECT_THAT(database, Not(IsOk()));
     return;
   }
-  ASSERT_THAT(database, IsOk()) << database.status();
+  ASSERT_THAT(database, IsOk());
 
   auto create_time =
       MakeTimestamp(database->create_time()).value().get<absl::Time>().value();
@@ -356,6 +354,19 @@ TEST_F(BackupExtraIntegrationTest, BackupRestoreWithCMEK) {
                     .CreateBackup(db, db.database_id(), expire_time,
                                   absl::nullopt, encryption_config)
                     .get();
+  {
+    // TODO(#8616): Remove this when we know how to deal with the issue.
+    auto matcher = testing_util::StatusIs(
+        StatusCode::kDeadlineExceeded,
+        testing::HasSubstr("terminated by polling policy"));
+    testing::StringMatchResultListener listener;
+    if (matcher.impl().MatchAndExplain(backup, &listener)) {
+      // The backup is still in progress (and may eventually complete),
+      // and we can't drop the database while it has pending backups, so
+      // we simply abandon them, to be cleaned up offline.
+      GTEST_SKIP();
+    }
+  }
   ASSERT_STATUS_OK(backup);
   EXPECT_TRUE(backup->has_encryption_info());
   if (backup->has_encryption_info()) {
