@@ -13,12 +13,14 @@
 // limitations under the License.
 
 #include "google/cloud/spanner/internal/partial_result_set_source.h"
+#include "google/cloud/spanner/mocks/row.h"
+#include "google/cloud/spanner/options.h"
 #include "google/cloud/spanner/row.h"
 #include "google/cloud/spanner/testing/mock_partial_result_set_reader.h"
 #include "google/cloud/spanner/value.h"
+#include "google/cloud/options.h"
 #include "google/cloud/testing_util/is_proto_equal.h"
 #include "google/cloud/testing_util/status_matchers.h"
-#include "absl/memory/memory.h"
 #include <google/protobuf/text_format.h>
 #include <gmock/gmock.h>
 #include <grpcpp/grpcpp.h>
@@ -32,15 +34,59 @@ namespace spanner_internal {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
-namespace spanner_proto = ::google::spanner::v1;
-
-using ::google::cloud::spanner::MakeTestRow;
 using ::google::cloud::spanner_testing::MockPartialResultSetReader;
 using ::google::cloud::testing_util::IsProtoEqual;
 using ::google::cloud::testing_util::StatusIs;
 using ::google::protobuf::TextFormat;
-using ::testing::HasSubstr;
-using ::testing::Return;
+using ::testing::_;
+using ::testing::UnitTest;
+
+std::string CurrentTestName() {
+  return UnitTest::GetInstance()->current_test_info()->name();
+}
+
+struct StringOption {
+  using Type = std::string;
+};
+
+// Create the `PartialResultSetSource` within an `OptionsSpan` that has its
+// `StringOption` set to the current test name, so that we might check that
+// all `PartialResultSetReader` calls happen within a matching span.
+StatusOr<std::unique_ptr<spanner::ResultSourceInterface>>
+CreatePartialResultSetSource(std::unique_ptr<PartialResultSetReader> reader,
+                             Options opts = {}) {
+  internal::OptionsSpan span(internal::MergeOptions(
+      std::move(opts.set<StringOption>(CurrentTestName())),
+      internal::CurrentOptions()));
+  return PartialResultSetSource::Create(std::move(reader));
+}
+
+// Returns a functor that expects the current `StringOption` to match the test
+// name.
+std::function<void()> VoidMock() {
+  return [] {
+    EXPECT_EQ(internal::CurrentOptions().get<StringOption>(),
+              CurrentTestName());
+  };
+}
+
+// Returns a functor that will return the argument after expecting that the
+// current `StringOption` matches the test name.
+template <typename T>
+std::function<T()> ResultMock(T const& result) {
+  return [result]() {
+    EXPECT_EQ(internal::CurrentOptions().get<StringOption>(),
+              CurrentTestName());
+    return result;
+  };
+}
+
+absl::optional<PartialResultSet> ReadResult(
+    google::spanner::v1::PartialResultSet response) {
+  return PartialResultSet{std::move(response), false};
+}
+
+absl::optional<PartialResultSet> ReadResult() { return {}; }
 
 MATCHER_P(IsValidAndEquals, expected,
           "Verifies that a StatusOr<Row> contains the given Row") {
@@ -49,14 +95,15 @@ MATCHER_P(IsValidAndEquals, expected,
 
 /// @test Verify the behavior when the initial `Read()` fails.
 TEST(PartialResultSetSourceTest, InitialReadFailure) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  Status finish_status(StatusCode::kInvalidArgument, "invalid");
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(finish_status));
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_)).WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish())
+      .WillOnce(ResultMock(Status(StatusCode::kInvalidArgument, "invalid")));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_THAT(reader, StatusIs(StatusCode::kInvalidArgument));
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  EXPECT_THAT(reader, StatusIs(StatusCode::kInvalidArgument, "invalid"));
 }
 
 /**
@@ -64,7 +111,6 @@ TEST(PartialResultSetSourceTest, InitialReadFailure) {
  * failed `Read()`.
  */
 TEST(PartialResultSetSourceTest, ReadSuccessThenFailure) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   auto constexpr kText = R"pb(
     metadata: {
       row_type: {
@@ -76,34 +122,41 @@ TEST(PartialResultSetSourceTest, ReadSuccessThenFailure) {
     }
     values: { string_value: "80" }
   )pb";
-  spanner_proto::PartialResultSet response;
+  google::spanner::v1::PartialResultSet response;
   ASSERT_TRUE(TextFormat::ParseFromString(kText, &response));
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response))
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  Status finish_status(StatusCode::kCancelled, "cancelled");
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(finish_status));
+
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response)))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish())
+      .WillOnce(ResultMock(Status(StatusCode::kCancelled, "cancelled")));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
   // The first call to NextRow() yields a row but the second gives an error.
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
-  EXPECT_THAT((*reader)->NextRow(),
-              IsValidAndEquals(MakeTestRow({{"AnInt", spanner::Value(80)}})));
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
+  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner_mocks::MakeRow(
+                                        {{"AnInt", spanner::Value(80)}})));
   auto row = (*reader)->NextRow();
-  EXPECT_THAT(row, StatusIs(StatusCode::kCancelled));
+  EXPECT_THAT(row, StatusIs(StatusCode::kCancelled, "cancelled"));
 }
 
 /// @test Verify the behavior when the first response does not contain metadata.
 TEST(PartialResultSetSourceTest, MissingMetadata) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
-  spanner_proto::PartialResultSet response;
-  EXPECT_CALL(*grpc_reader, Read()).WillOnce(Return(response));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
-  // The destructor should try to cancel the RPC to avoid deadlocks.
-  EXPECT_CALL(*grpc_reader, TryCancel()).Times(1);
+  google::spanner::v1::PartialResultSet response;
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_)).WillOnce(ResultMock(ReadResult(response)));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).WillOnce(VoidMock());
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
   EXPECT_THAT(reader, StatusIs(StatusCode::kInternal,
-                               "response contained no metadata"));
+                               "PartialResultSetSource response"
+                               " contained no metadata"));
 }
 
 /**
@@ -111,16 +164,21 @@ TEST(PartialResultSetSourceTest, MissingMetadata) {
  * type information.
  */
 TEST(PartialResultSetSourceTest, MissingRowTypeNoData) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
-  auto constexpr kText = R"pb(metadata: {})pb";
-  spanner_proto::PartialResultSet response;
+  auto constexpr kText = R"pb(
+    metadata: {}
+  )pb";
+  google::spanner::v1::PartialResultSet response;
   ASSERT_TRUE(TextFormat::ParseFromString(kText, &response));
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response))
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response)))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
   ASSERT_STATUS_OK(reader);
   EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner::Row{}));
 }
@@ -130,22 +188,26 @@ TEST(PartialResultSetSourceTest, MissingRowTypeNoData) {
  * row type information.
  */
 TEST(PartialResultSetSourceTest, MissingRowTypeWithData) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   auto constexpr kText = R"pb(
     metadata: {}
-    values: { string_value: "10" })pb";
-  spanner_proto::PartialResultSet response;
+    values: { string_value: "10" }
+  )pb";
+  google::spanner::v1::PartialResultSet response;
   ASSERT_TRUE(TextFormat::ParseFromString(kText, &response));
-  EXPECT_CALL(*grpc_reader, Read()).WillOnce(Return(response));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
-  // The destructor should try to cancel the RPC to avoid deadlocks.
-  EXPECT_CALL(*grpc_reader, TryCancel()).Times(1);
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response)))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
   ASSERT_STATUS_OK(reader);
-  StatusOr<spanner::Row> row = (*reader)->NextRow();
-  EXPECT_THAT(row, StatusIs(StatusCode::kInternal,
-                            HasSubstr("missing row type information")));
+  EXPECT_THAT((*reader)->NextRow(),
+              StatusIs(StatusCode::kInternal,
+                       "PartialResultSetSource metadata is missing row type"));
 }
 
 /**
@@ -154,7 +216,6 @@ TEST(PartialResultSetSourceTest, MissingRowTypeWithData) {
  * All of the data is returned in a single Read() from the gRPC reader.
  */
 TEST(PartialResultSetSourceTest, SingleResponse) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   auto constexpr kText = R"pb(
     metadata: {
       row_type: {
@@ -187,15 +248,19 @@ TEST(PartialResultSetSourceTest, SingleResponse) {
       }
     }
   )pb";
-  spanner_proto::PartialResultSet response;
+  google::spanner::v1::PartialResultSet response;
   ASSERT_TRUE(TextFormat::ParseFromString(kText, &response));
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response))
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response)))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
 
   // Verify the returned metadata is correct.
   auto constexpr kTextExpectedMetadata = R"pb(
@@ -210,7 +275,7 @@ TEST(PartialResultSetSourceTest, SingleResponse) {
       }
     }
   )pb";
-  spanner_proto::ResultSetMetadata expected_metadata;
+  google::spanner::v1::ResultSetMetadata expected_metadata;
   ASSERT_TRUE(
       TextFormat::ParseFromString(kTextExpectedMetadata, &expected_metadata));
   auto actual_metadata = (*reader)->Metadata();
@@ -218,7 +283,7 @@ TEST(PartialResultSetSourceTest, SingleResponse) {
   EXPECT_THAT(*actual_metadata, IsProtoEqual(expected_metadata));
 
   // Verify the returned rows are correct.
-  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(MakeTestRow({
+  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner_mocks::MakeRow({
                                         {"UserId", spanner::Value(10)},
                                         {"UserName", spanner::Value("user10")},
                                     })));
@@ -243,7 +308,7 @@ TEST(PartialResultSetSourceTest, SingleResponse) {
       }
     }
   )pb";
-  spanner_proto::ResultSetStats expected_stats;
+  google::spanner::v1::ResultSetStats expected_stats;
   ASSERT_TRUE(TextFormat::ParseFromString(kTextExpectedStats, &expected_stats));
   auto actual_stats = (*reader)->Stats();
   EXPECT_TRUE(actual_stats.has_value());
@@ -255,7 +320,6 @@ TEST(PartialResultSetSourceTest, SingleResponse) {
  * reader returns data across multiple Read() calls.
  */
 TEST(PartialResultSetSourceTest, MultipleResponses) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   std::array<char const*, 5> text{{
       R"pb(
         metadata: {
@@ -283,7 +347,7 @@ TEST(PartialResultSetSourceTest, MultipleResponses) {
         values: { string_value: "99" }
       )pb",
       R"pb(
-        values: { string_value: "99user99" }
+        values: { string_value: "user99" }
         stats: {
           query_stats: {
             fields: {
@@ -302,47 +366,60 @@ TEST(PartialResultSetSourceTest, MultipleResponses) {
         }
       )pb",
   }};
-  std::array<spanner_proto::PartialResultSet, text.size()> response;
+  std::array<google::spanner::v1::PartialResultSet, text.size()> response;
   for (std::size_t i = 0; i != text.size(); ++i) {
     SCOPED_TRACE("Converting text to proto [" + std::to_string(i) + "]");
     ASSERT_TRUE(TextFormat::ParseFromString(text[i], &response[i]));
   }
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response[0]))
-      .WillOnce(Return(response[1]))
-      .WillOnce(Return(response[2]))
-      .WillOnce(Return(response[3]))
-      .WillOnce(Return(response[4]))
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  // Choose values for the `StreamingResumabilityBufferSizeOption` that,
+  // given `text`, yield complete rows after varying numbers of calls to
+  // `ReadFromStream()`.
+  for (std::size_t buffer_size : {0, 153, 161, 321, 385, 448}) {
+    auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+    EXPECT_CALL(*grpc_reader, Read(_))
+        .WillOnce(ResultMock(ReadResult(response[0])))
+        .WillOnce(ResultMock(ReadResult(response[1])))
+        .WillOnce(ResultMock(ReadResult(response[2])))
+        .WillOnce(ResultMock(ReadResult(response[3])))
+        .WillOnce(ResultMock(ReadResult(response[4])))
+        .WillOnce(ResultMock(ReadResult()));
+    EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+    EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
 
-  // Verify the returned rows are correct.
-  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(MakeTestRow({
-                                        {"UserId", spanner::Value(10)},
-                                        {"UserName", spanner::Value("user10")},
-                                    })));
-  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(MakeTestRow({
-                                        {"UserId", spanner::Value(22)},
-                                        {"UserName", spanner::Value("user22")},
-                                    })));
-  EXPECT_THAT((*reader)->NextRow(),
-              IsValidAndEquals(MakeTestRow({
-                  {"UserId", spanner::Value(99)},
-                  {"UserName", spanner::Value("99user99")},
-              })));
+    internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+    auto reader = CreatePartialResultSetSource(
+        std::move(grpc_reader),
+        Options{}.set<spanner::StreamingResumabilityBufferSizeOption>(
+            buffer_size));
+    ASSERT_STATUS_OK(reader);
 
-  // At end of stream, we get an 'ok' response with an empty row.
-  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner::Row{}));
+    // Verify the returned rows are correct.
+    EXPECT_THAT((*reader)->NextRow(),
+                IsValidAndEquals(spanner_mocks::MakeRow({
+                    {"UserId", spanner::Value(10)},
+                    {"UserName", spanner::Value("user10")},
+                })));
+    EXPECT_THAT((*reader)->NextRow(),
+                IsValidAndEquals(spanner_mocks::MakeRow({
+                    {"UserId", spanner::Value(22)},
+                    {"UserName", spanner::Value("user22")},
+                })));
+    EXPECT_THAT((*reader)->NextRow(),
+                IsValidAndEquals(spanner_mocks::MakeRow({
+                    {"UserId", spanner::Value(99)},
+                    {"UserName", spanner::Value("user99")},
+                })));
+
+    // At end of stream, we get an 'ok' response with an empty row.
+    EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner::Row{}));
+  }
 }
 
 /**
  * @test Verify the behavior when a response with no values is received.
  */
 TEST(PartialResultSetSourceTest, ResponseWithNoValues) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   std::array<char const*, 3> text{{
       R"pb(
         metadata: {
@@ -354,40 +431,44 @@ TEST(PartialResultSetSourceTest, ResponseWithNoValues) {
           }
         }
       )pb",
-      "",
+      R"pb(
+      )pb",
       R"pb(
         values: { string_value: "22" }
       )pb",
   }};
-  std::array<spanner_proto::PartialResultSet, text.size()> response;
+  std::array<google::spanner::v1::PartialResultSet, text.size()> response;
   for (std::size_t i = 0; i != text.size(); ++i) {
     SCOPED_TRACE("Converting text to proto [" + std::to_string(i) + "]");
     ASSERT_TRUE(TextFormat::ParseFromString(text[i], &response[i]));
   }
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response[0]))
-      .WillOnce(Return(response[1]))
-      .WillOnce(Return(response[2]))
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response[0])))
+      .WillOnce(ResultMock(ReadResult(response[1])))
+      .WillOnce(ResultMock(ReadResult(response[2])))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
 
   // Verify the returned row is correct.
-  EXPECT_THAT((*reader)->NextRow(),
-              IsValidAndEquals(MakeTestRow({{"UserId", spanner::Value(22)}})));
+  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner_mocks::MakeRow(
+                                        {{"UserId", spanner::Value(22)}})));
 
   // At end of stream, we get an 'ok' response with an empty row.
   EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner::Row{}));
 }
 
 /**
- * @Test Verify reassembling chunked values works correctly, including a mixture
- * of chunked and reassembled chunked values.
+ * @test Verify reassembling chunked values works correctly, including a mixture
+ * of chunked and un-chunked values.
  */
 TEST(PartialResultSetSourceTest, ChunkedStringValueWellFormed) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   std::array<char const*, 5> text{{
       R"pb(
         metadata: {
@@ -420,31 +501,34 @@ TEST(PartialResultSetSourceTest, ChunkedStringValueWellFormed) {
         values: { string_value: "still not_chunked" }
       )pb",
   }};
-  std::array<spanner_proto::PartialResultSet, text.size()> response;
+  std::array<google::spanner::v1::PartialResultSet, text.size()> response;
   for (std::size_t i = 0; i != text.size(); ++i) {
     SCOPED_TRACE("Converting text to proto [" + std::to_string(i) + "]");
     ASSERT_TRUE(TextFormat::ParseFromString(text[i], &response[i]));
   }
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response[0]))
-      .WillOnce(Return(response[1]))
-      .WillOnce(Return(response[2]))
-      .WillOnce(Return(response[3]))
-      .WillOnce(Return(response[4]))
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response[0])))
+      .WillOnce(ResultMock(ReadResult(response[1])))
+      .WillOnce(ResultMock(ReadResult(response[2])))
+      .WillOnce(ResultMock(ReadResult(response[3])))
+      .WillOnce(ResultMock(ReadResult(response[4])))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
 
   // Verify the returned values are correct.
   for (auto const& value :
        {"not_chunked", "first_chunksecond_chunkthird_chunk",
         "second group first_chunk second group second_chunk",
         "also not_chunked", "still not_chunked"}) {
-    EXPECT_THAT(
-        (*reader)->NextRow(),
-        IsValidAndEquals(MakeTestRow({{"Prose", spanner::Value(value)}})));
+    EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner_mocks::MakeRow(
+                                          {{"Prose", spanner::Value(value)}})));
   }
 
   // At end of stream, we get an 'ok' response with an empty row.
@@ -456,7 +540,6 @@ TEST(PartialResultSetSourceTest, ChunkedStringValueWellFormed) {
  * values in the response.
  */
 TEST(PartialResultSetSourceTest, ChunkedValueSetNoValue) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   std::array<char const*, 2> text{{
       R"pb(
         metadata: {
@@ -468,28 +551,30 @@ TEST(PartialResultSetSourceTest, ChunkedValueSetNoValue) {
           }
         }
       )pb",
-      R"pb(chunked_value: true)pb",
+      R"pb(
+        chunked_value: true
+      )pb",
   }};
-  std::array<spanner_proto::PartialResultSet, text.size()> response;
+  std::array<google::spanner::v1::PartialResultSet, text.size()> response;
   for (std::size_t i = 0; i != text.size(); ++i) {
     SCOPED_TRACE("Converting text to proto [" + std::to_string(i) + "]");
     ASSERT_TRUE(TextFormat::ParseFromString(text[i], &response[i]));
   }
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response[0]))
-      .WillOnce(Return(response[1]));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
-  // The destructor should try to cancel the RPC to avoid deadlocks.
-  EXPECT_CALL(*grpc_reader, TryCancel()).Times(1);
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response[0])))
+      .WillOnce(ResultMock(ReadResult(response[1])))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
 
-  // Trying to read the next row should fail.
-  auto row = (*reader)->NextRow();
-  EXPECT_THAT(row, StatusIs(StatusCode::kInternal,
-                            "PartialResultSet had chunked_value set true but "
-                            "contained no values"));
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
+
+  // We ignore `chunked_value` when there is no value to be chunked.
+  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner::Row{}));
 }
 
 /**
@@ -497,7 +582,6 @@ TEST(PartialResultSetSourceTest, ChunkedValueSetNoValue) {
  * with `chunked_value` set.
  */
 TEST(PartialResultSetSourceTest, ChunkedValueSetNoFollowingValue) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   std::array<char const*, 2> text{{
       R"pb(
         metadata: {
@@ -511,35 +595,38 @@ TEST(PartialResultSetSourceTest, ChunkedValueSetNoFollowingValue) {
         values: { string_value: "incomplete" }
         chunked_value: true
       )pb",
-      R"pb()pb",
+      R"pb(
+      )pb",
   }};
-  std::array<spanner_proto::PartialResultSet, text.size()> response;
+  std::array<google::spanner::v1::PartialResultSet, text.size()> response;
   for (std::size_t i = 0; i != text.size(); ++i) {
     SCOPED_TRACE("Converting text to proto [" + std::to_string(i) + "]");
     ASSERT_TRUE(TextFormat::ParseFromString(text[i], &response[i]));
   }
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response[0]))
-      .WillOnce(Return(response[1]));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
-  // The destructor should try to cancel the RPC to avoid deadlocks.
-  EXPECT_CALL(*grpc_reader, TryCancel()).Times(1);
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response[0])))
+      .WillOnce(ResultMock(ReadResult(response[1])))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
 
   // Trying to read the next row should fail.
-  auto row = (*reader)->NextRow();
-  EXPECT_THAT(row, StatusIs(StatusCode::kInternal,
-                            "PartialResultSet contained no values to merge "
-                            "with prior chunked_value"));
+  EXPECT_THAT((*reader)->NextRow(),
+              StatusIs(StatusCode::kInternal,
+                       "PartialResultSetSource stream ended at a point"
+                       " that is not on a row boundary"));
 }
 
 /**
  * @test Verify the behavior when `chunked_value` is set in the final response.
  */
 TEST(PartialResultSetSourceTest, ChunkedValueSetAtEndOfStream) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   std::array<char const*, 2> text{{
       R"pb(
         metadata: {
@@ -556,24 +643,29 @@ TEST(PartialResultSetSourceTest, ChunkedValueSetAtEndOfStream) {
         chunked_value: true
       )pb",
   }};
-  std::array<spanner_proto::PartialResultSet, text.size()> response;
+  std::array<google::spanner::v1::PartialResultSet, text.size()> response;
   for (std::size_t i = 0; i != text.size(); ++i) {
     SCOPED_TRACE("Converting text to proto [" + std::to_string(i) + "]");
     ASSERT_TRUE(TextFormat::ParseFromString(text[i], &response[i]));
   }
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response[0]))
-      .WillOnce(Return(response[1]))
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response[0])))
+      .WillOnce(ResultMock(ReadResult(response[1])))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
 
   // Trying to read the next row should fail.
-  auto row = (*reader)->NextRow();
-  EXPECT_THAT(row, StatusIs(StatusCode::kInternal,
-                            "incomplete chunked_value at end of stream"));
+  EXPECT_THAT((*reader)->NextRow(),
+              StatusIs(StatusCode::kInternal,
+                       "PartialResultSetSource stream ended at a point"
+                       " that is not on a row boundary"));
 }
 
 /**
@@ -581,7 +673,6 @@ TEST(PartialResultSetSourceTest, ChunkedValueSetAtEndOfStream) {
  * chunked (float64/number).
  */
 TEST(PartialResultSetSourceTest, ChunkedValueMergeFailure) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   std::array<char const*, 3> text{{
       R"pb(
         metadata: {
@@ -601,32 +692,33 @@ TEST(PartialResultSetSourceTest, ChunkedValueMergeFailure) {
         values: { number_value: 99 }
       )pb",
   }};
-  std::array<spanner_proto::PartialResultSet, text.size()> response;
+  std::array<google::spanner::v1::PartialResultSet, text.size()> response;
   for (std::size_t i = 0; i != text.size(); ++i) {
     SCOPED_TRACE("Converting text to proto [" + std::to_string(i) + "]");
     ASSERT_TRUE(TextFormat::ParseFromString(text[i], &response[i]));
   }
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response[0]))
-      .WillOnce(Return(response[1]))
-      .WillOnce(Return(response[2]));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
-  // The destructor should try to cancel the RPC to avoid deadlocks.
-  EXPECT_CALL(*grpc_reader, TryCancel()).Times(1);
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response[0])))
+      .WillOnce(ResultMock(ReadResult(response[1])))
+      .WillOnce(ResultMock(ReadResult(response[2])));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).WillOnce(VoidMock());
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
 
   // Trying to read the next row should fail.
-  auto row = (*reader)->NextRow();
-  EXPECT_THAT(row, StatusIs(StatusCode::kInvalidArgument, "invalid type"));
+  EXPECT_THAT((*reader)->NextRow(),
+              StatusIs(StatusCode::kInvalidArgument, "invalid type"));
 }
 
 /**
  * @test Verify the behavior when we get an incomplete Row.
  */
 TEST(PartialResultSetSourceTest, ErrorOnIncompleteRow) {
-  auto grpc_reader = absl::make_unique<MockPartialResultSetReader>();
   std::array<char const*, 5> text{{
       R"pb(
         metadata: {
@@ -672,36 +764,42 @@ TEST(PartialResultSetSourceTest, ErrorOnIncompleteRow) {
         }
       )pb",
   }};
-  std::array<spanner_proto::PartialResultSet, text.size()> response;
+  std::array<google::spanner::v1::PartialResultSet, text.size()> response;
   for (std::size_t i = 0; i != text.size(); ++i) {
     SCOPED_TRACE("Converting text to proto [" + std::to_string(i) + "]");
     ASSERT_TRUE(TextFormat::ParseFromString(text[i], &response[i]));
   }
-  EXPECT_CALL(*grpc_reader, Read())
-      .WillOnce(Return(response[0]))
-      .WillOnce(Return(response[1]))
-      .WillOnce(Return(response[2]))
-      .WillOnce(Return(response[3]))
-      .WillOnce(Return(response[4]))
-      .WillOnce(Return(absl::optional<spanner_proto::PartialResultSet>{}));
-  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(Return(Status()));
 
-  auto reader = PartialResultSetSource::Create(std::move(grpc_reader));
-  EXPECT_STATUS_OK(reader.status());
+  auto grpc_reader = std::make_unique<MockPartialResultSetReader>();
+  EXPECT_CALL(*grpc_reader, Read(_))
+      .WillOnce(ResultMock(ReadResult(response[0])))
+      .WillOnce(ResultMock(ReadResult(response[1])))
+      .WillOnce(ResultMock(ReadResult(response[2])))
+      .WillOnce(ResultMock(ReadResult(response[3])))
+      .WillOnce(ResultMock(ReadResult(response[4])))
+      .WillOnce(ResultMock(ReadResult()));
+  EXPECT_CALL(*grpc_reader, Finish()).WillOnce(ResultMock(Status()));
+  EXPECT_CALL(*grpc_reader, TryCancel()).Times(0);
+
+  internal::OptionsSpan overlay(Options{}.set<StringOption>("uh-oh"));
+  auto reader = CreatePartialResultSetSource(std::move(grpc_reader));
+  ASSERT_STATUS_OK(reader);
 
   // Verify the first two rows are correct.
-  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(MakeTestRow({
+  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner_mocks::MakeRow({
                                         {"UserId", spanner::Value(10)},
                                         {"UserName", spanner::Value("user10")},
                                     })));
-  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(MakeTestRow({
+  EXPECT_THAT((*reader)->NextRow(), IsValidAndEquals(spanner_mocks::MakeRow({
                                         {"UserId", spanner::Value(22)},
                                         {"UserName", spanner::Value("user22")},
                                     })));
 
-  auto row = (*reader)->NextRow();
-  EXPECT_THAT(row,
-              StatusIs(StatusCode::kInternal, HasSubstr("incomplete row")));
+  // Trying to read the next row should fail.
+  EXPECT_THAT((*reader)->NextRow(),
+              StatusIs(StatusCode::kInternal,
+                       "PartialResultSetSource stream ended at a point"
+                       " that is not on a row boundary"));
 }
 
 }  // namespace
