@@ -13,15 +13,16 @@
 // limitations under the License.
 
 #include "google/cloud/spanner/internal/session_pool.h"
-#include "google/cloud/spanner/internal/connection_impl.h"
+#include "google/cloud/spanner/internal/route_to_leader.h"
 #include "google/cloud/spanner/internal/session.h"
+#include "google/cloud/spanner/internal/status_utils.h"
 #include "google/cloud/spanner/options.h"
 #include "google/cloud/completion_queue.h"
 #include "google/cloud/internal/async_retry_loop.h"
 #include "google/cloud/internal/retry_loop.h"
 #include "google/cloud/log.h"
+#include "google/cloud/options.h"
 #include "google/cloud/status.h"
-#include "absl/memory/memory.h"
 #include <grpcpp/grpcpp.h>
 #include <algorithm>
 #include <chrono>
@@ -35,8 +36,6 @@ namespace cloud {
 namespace spanner_internal {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
-namespace spanner_proto = ::google::spanner::v1;
-
 using ::google::cloud::Idempotency;
 
 std::shared_ptr<SessionPool> MakeSessionPool(
@@ -77,30 +76,30 @@ SessionPool::SessionPool(spanner::Database db,
 }
 
 void SessionPool::Initialize() {
+  internal::OptionsSpan span(opts_);
   auto const min_sessions = opts_.get<spanner::SessionPoolMinSessionsOption>();
   if (min_sessions > 0) {
     std::unique_lock<std::mutex> lk(mu_);
-    (void)Grow(lk, min_sessions, WaitForSessionAllocation::kWait);
+    Grow(lk, min_sessions, WaitForSessionAllocation::kWait);
   }
   ScheduleBackgroundWork(std::chrono::seconds(5));
 }
 
 SessionPool::~SessionPool() {
-  // All references to this object are via `shared_ptr`; since we're in the
+  // All references to this object are via `shared_ptr`. Since we're in the
   // destructor that implies there can be no concurrent accesses to any member
   // variables, including `current_timer_`.
   //
   // Note that it *is* possible the timer lambda in `ScheduleBackgroundWork`
-  // is executing concurrently. However, since we are in the destructor we know
-  // that the lambda must not have yet successfully finished a call to `lock()`
-  // on the `weak_ptr` to `this` it holds. Any subsequent or in-progress calls
-  // must return `nullptr`, and the lambda will not do any work nor reschedule
-  // the timer.
+  // or the response handler in `RefreshExpiringSessions` are executing
+  // concurrently. However, since we are in the destructor we know that
+  // they must not have successfully finished a call to `lock()` on the
+  // `weak_ptr` to `this` they hold. Any in-progress or subsequent `lock()`
+  // will now return `nullptr`, in which case no work is done.
   current_timer_.cancel();
 }
 
 void SessionPool::ScheduleBackgroundWork(std::chrono::seconds relative_time) {
-  // See the comment in the destructor about the thread safety of this method.
   std::weak_ptr<SessionPool> pool = shared_from_this();
   current_timer_ =
       cq_.MakeRelativeTimer(relative_time)
@@ -154,20 +153,43 @@ void SessionPool::RefreshExpiringSessions() {
       }
     }
   }
+  std::weak_ptr<SessionPool> pool = shared_from_this();
   for (auto& refresh : sessions_to_refresh) {
-    AsyncRefreshSession(cq_, refresh.first, std::move(refresh.second))
-        .then([](future<StatusOr<spanner_proto::ResultSet>> result) {
-          // We simply discard the response as handling IsSessionNotFound()
-          // by removing the session from the pool is problematic (and would
-          // not eliminate the possibility of IsSessionNotFound() elsewhere).
-          // The last-use time has already been updated to throttle attempts.
-          // TODO(#4026): Re-evaluate these decisions.
-          (void)result.get();
-        });
+    auto handler =
+        [pool, session_name = refresh.second](
+            future<StatusOr<google::spanner::v1::ResultSet>> result) {
+          auto response = result.get();
+          if (!response && IsSessionNotFound(response.status())) {
+            if (auto shared_pool = pool.lock()) {
+              // The pool still exists, but the bad session may no
+              // longer be in the pool because someone else has already
+              // tried to use it, discovered that it is bad, and so did
+              // not return it (or they are in process of doing all that).
+              // But if it is still in the pool, we remove it now.
+              shared_pool->Erase(session_name);
+            }
+          }
+        };
+    AsyncRefreshSession(cq_, std::move(refresh.first),
+                        std::move(refresh.second))
+        .then(std::move(handler));
   }
 }
 
-/**
+void SessionPool::Erase(std::string const& session_name) {
+  std::unique_ptr<Session> target;
+  std::unique_lock<std::mutex> lk(mu_);
+  for (auto& session : sessions_) {
+    if (session->session_name() == session_name) {
+      target = std::move(session);  // deferred deletion
+      session = std::move(sessions_.back());
+      sessions_.pop_back();
+      break;
+    }
+  }
+}
+
+/*
  * Grow the session pool by creating up to `sessions_to_create` sessions and
  * adding them to the pool.  Note that `lk` may be released and reacquired in
  * this method.
@@ -179,7 +201,7 @@ Status SessionPool::Grow(std::unique_lock<std::mutex>& lk,
                          int sessions_to_create,
                          WaitForSessionAllocation wait) {
   auto create_counts = ComputeCreateCounts(sessions_to_create);
-  if (!create_counts.ok()) {
+  if (!create_counts.ok() || create_counts->empty()) {
     return create_counts.status();
   }
   create_calls_in_progress_ += static_cast<int>(create_counts->size());
@@ -245,18 +267,20 @@ Status SessionPool::CreateSessions(
     std::vector<CreateCount> const& create_counts,
     WaitForSessionAllocation wait) {
   Status return_status;
+  auto const& labels = opts_.get<spanner::SessionPoolLabelsOption>();
+  auto const& role = opts_.get<spanner::SessionCreatorRoleOption>();
   for (auto const& op : create_counts) {
-    auto const& labels = opts_.get<spanner::SessionPoolLabelsOption>();
     switch (wait) {
       case WaitForSessionAllocation::kWait: {
-        auto status = CreateSessionsSync(op.channel, labels, op.session_count);
+        auto status =
+            CreateSessionsSync(op.channel, labels, role, op.session_count);
         if (!status.ok()) {
           return_status = status;
         }
         break;
       }
       case WaitForSessionAllocation::kNoWait:
-        CreateSessionsAsync(op.channel, labels, op.session_count);
+        CreateSessionsAsync(op.channel, labels, role, op.session_count);
         break;
     }
   }
@@ -264,6 +288,10 @@ Status SessionPool::CreateSessions(
 }
 
 StatusOr<SessionHolder> SessionPool::Allocate(bool dissociate_from_pool) {
+  // We choose to ignore the internal::CurrentOptions() here as it is
+  // non-deterministic when RPCs to create sessions are actually made.
+  // It is clearer if we just stick with the construction-time Options.
+  internal::OptionsSpan span(opts_);
   std::unique_lock<std::mutex> lk(mu_);
   for (;;) {
     if (!sessions_.empty()) {
@@ -357,18 +385,25 @@ void SessionPool::Release(std::unique_ptr<Session> session) {
 // Creates `num_sessions` on `channel` and adds them to the pool.
 Status SessionPool::CreateSessionsSync(
     std::shared_ptr<Channel> const& channel,
-    std::map<std::string, std::string> const& labels, int num_sessions) {
-  spanner_proto::BatchCreateSessionsRequest request;
+    std::map<std::string, std::string> const& labels, std::string const& role,
+    int num_sessions) {
+  google::spanner::v1::BatchCreateSessionsRequest request;
   request.set_database(db_.FullName());
-  request.mutable_session_template()->mutable_labels()->insert(labels.begin(),
-                                                               labels.end());
+  if (!labels.empty()) {
+    request.mutable_session_template()->mutable_labels()->insert(labels.begin(),
+                                                                 labels.end());
+  }
+  if (!role.empty()) {
+    request.mutable_session_template()->set_creator_role(role);
+  }
   request.set_session_count(std::int32_t{num_sessions});
   auto const& stub = channel->stub;
   auto response = RetryLoop(
       retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
       google::cloud::Idempotency::kIdempotent,
       [&stub](grpc::ClientContext& context,
-              spanner_proto::BatchCreateSessionsRequest const& request) {
+              google::spanner::v1::BatchCreateSessionsRequest const& request) {
+        RouteToLeader(context);  // always for BatchCreateSessions()
         return stub->BatchCreateSessions(context, request);
       },
       request, __func__);
@@ -377,17 +412,19 @@ Status SessionPool::CreateSessionsSync(
 
 void SessionPool::CreateSessionsAsync(
     std::shared_ptr<Channel> const& channel,
-    std::map<std::string, std::string> const& labels, int num_sessions) {
+    std::map<std::string, std::string> const& labels, std::string const& role,
+    int num_sessions) {
   std::weak_ptr<SessionPool> pool = shared_from_this();
-  AsyncBatchCreateSessions(cq_, channel->stub, labels, num_sessions)
-      .then([pool, channel](
-                future<StatusOr<spanner_proto::BatchCreateSessionsResponse>>
-                    result) {
-        if (auto shared_pool = pool.lock()) {
-          shared_pool->HandleBatchCreateSessionsDone(channel,
-                                                     std::move(result).get());
-        }
-      });
+  AsyncBatchCreateSessions(cq_, channel->stub, labels, role, num_sessions)
+      .then(
+          [pool, channel](
+              future<StatusOr<google::spanner::v1::BatchCreateSessionsResponse>>
+                  result) {
+            if (auto shared_pool = pool.lock()) {
+              shared_pool->HandleBatchCreateSessionsDone(
+                  channel, std::move(result).get());
+            }
+          });
 }
 
 SessionHolder SessionPool::MakeSessionHolder(std::unique_ptr<Session> session,
@@ -406,20 +443,27 @@ SessionHolder SessionPool::MakeSessionHolder(std::unique_ptr<Session> session,
   });
 }
 
-future<StatusOr<spanner_proto::BatchCreateSessionsResponse>>
+future<StatusOr<google::spanner::v1::BatchCreateSessionsResponse>>
 SessionPool::AsyncBatchCreateSessions(
     CompletionQueue& cq, std::shared_ptr<SpannerStub> const& stub,
-    std::map<std::string, std::string> const& labels, int num_sessions) {
-  spanner_proto::BatchCreateSessionsRequest request;
+    std::map<std::string, std::string> const& labels, std::string const& role,
+    int num_sessions) {
+  google::spanner::v1::BatchCreateSessionsRequest request;
   request.set_database(db_.FullName());
-  request.mutable_session_template()->mutable_labels()->insert(labels.begin(),
-                                                               labels.end());
+  if (!labels.empty()) {
+    request.mutable_session_template()->mutable_labels()->insert(labels.begin(),
+                                                                 labels.end());
+  }
+  if (!role.empty()) {
+    request.mutable_session_template()->set_creator_role(role);
+  }
   request.set_session_count(std::int32_t{num_sessions});
   return google::cloud::internal::AsyncRetryLoop(
       retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
       Idempotency::kIdempotent, cq,
-      [stub](CompletionQueue& cq, std::unique_ptr<grpc::ClientContext> context,
-             spanner_proto::BatchCreateSessionsRequest const& request) {
+      [stub](CompletionQueue& cq, std::shared_ptr<grpc::ClientContext> context,
+             google::spanner::v1::BatchCreateSessionsRequest const& request) {
+        RouteToLeader(*context);  // always for BatchCreateSessions()
         return stub->AsyncBatchCreateSessions(cq, std::move(context), request);
       },
       std::move(request), __func__);
@@ -428,33 +472,35 @@ SessionPool::AsyncBatchCreateSessions(
 future<Status> SessionPool::AsyncDeleteSession(
     CompletionQueue& cq, std::shared_ptr<SpannerStub> const& stub,
     std::string session_name) {
-  spanner_proto::DeleteSessionRequest request;
+  google::spanner::v1::DeleteSessionRequest request;
   request.set_name(std::move(session_name));
   return google::cloud::internal::AsyncRetryLoop(
       retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
       Idempotency::kIdempotent, cq,
-      [stub](CompletionQueue& cq, std::unique_ptr<grpc::ClientContext> context,
-             spanner_proto::DeleteSessionRequest const& request) {
+      [stub](CompletionQueue& cq, std::shared_ptr<grpc::ClientContext> context,
+             google::spanner::v1::DeleteSessionRequest const& request) {
         return stub->AsyncDeleteSession(cq, std::move(context), request);
       },
       std::move(request), __func__);
 }
 
 /// Refresh the session `session_name` by executing a `SELECT 1` query on it.
-future<StatusOr<spanner_proto::ResultSet>> SessionPool::AsyncRefreshSession(
-    CompletionQueue& cq, std::shared_ptr<SpannerStub> const& stub,
-    std::string session_name) {
-  spanner_proto::ExecuteSqlRequest request;
+future<StatusOr<google::spanner::v1::ResultSet>>
+SessionPool::AsyncRefreshSession(CompletionQueue& cq,
+                                 std::shared_ptr<SpannerStub> const& stub,
+                                 std::string session_name) {
+  google::spanner::v1::ExecuteSqlRequest request;
   request.set_session(std::move(session_name));
   // Single-use transaction with strong concurrency.
   request.set_sql("SELECT 1;");
   request.mutable_request_options()->set_priority(
-      spanner_proto::RequestOptions::PRIORITY_LOW);
+      google::spanner::v1::RequestOptions::PRIORITY_LOW);
   return google::cloud::internal::AsyncRetryLoop(
       retry_policy_prototype_->clone(), backoff_policy_prototype_->clone(),
       Idempotency::kIdempotent, cq,
-      [stub](CompletionQueue& cq, std::unique_ptr<grpc::ClientContext> context,
-             spanner_proto::ExecuteSqlRequest const& request) {
+      [stub](CompletionQueue& cq, std::shared_ptr<grpc::ClientContext> context,
+             google::spanner::v1::ExecuteSqlRequest const& request) {
+        // Read-only transaction, so no route-to-leader.
         return stub->AsyncExecuteSql(cq, std::move(context), request);
       },
       std::move(request), __func__);
@@ -462,7 +508,7 @@ future<StatusOr<spanner_proto::ResultSet>> SessionPool::AsyncRefreshSession(
 
 Status SessionPool::HandleBatchCreateSessionsDone(
     std::shared_ptr<Channel> const& channel,
-    StatusOr<spanner_proto::BatchCreateSessionsResponse> response) {
+    StatusOr<google::spanner::v1::BatchCreateSessionsResponse> response) {
   std::unique_lock<std::mutex> lk(mu_);
   --create_calls_in_progress_;
   if (!response.ok()) {
@@ -474,7 +520,7 @@ Status SessionPool::HandleBatchCreateSessionsDone(
   total_sessions_ += sessions_created;
   sessions_.reserve(sessions_.size() + sessions_created);
   for (auto& session : *response->mutable_session()) {
-    sessions_.push_back(absl::make_unique<Session>(
+    sessions_.push_back(std::make_unique<Session>(
         std::move(*session.mutable_name()), channel, clock_));
   }
   // Shuffle the pool so we distribute returned sessions across channels.
