@@ -13,22 +13,26 @@
 // limitations under the License.
 
 #include "google/cloud/spanner/internal/session_pool.h"
-#include "google/cloud/spanner/internal/clock.h"
 #include "google/cloud/spanner/internal/defaults.h"
 #include "google/cloud/spanner/internal/session.h"
 #include "google/cloud/spanner/options.h"
-#include "google/cloud/spanner/testing/fake_clock.h"
 #include "google/cloud/spanner/testing/mock_spanner_stub.h"
+#include "google/cloud/spanner/testing/status_utils.h"
+#include "google/cloud/spanner/timestamp.h"
 #include "google/cloud/internal/background_threads_impl.h"
+#include "google/cloud/internal/clock.h"
 #include "google/cloud/status.h"
+#include "google/cloud/testing_util/fake_clock.h"
 #include "google/cloud/testing_util/fake_completion_queue_impl.h"
 #include "google/cloud/testing_util/mock_async_response_reader.h"
 #include "google/cloud/testing_util/status_matchers.h"
-#include "absl/memory/memory.h"
+#include "google/cloud/testing_util/validate_metadata.h"
+#include "absl/time/clock.h"
 #include <google/protobuf/text_format.h>
 #include <gmock/gmock.h>
 #include <grpcpp/grpcpp.h>
 #include <chrono>
+#include <map>
 #include <memory>
 #include <string>
 #include <thread>
@@ -40,23 +44,39 @@ namespace spanner_internal {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
-using ::google::cloud::spanner_testing::FakeSteadyClock;
 using ::google::cloud::testing_util::FakeCompletionQueueImpl;
+using ::google::cloud::testing_util::FakeSteadyClock;
 using ::google::cloud::testing_util::StatusIs;
 using ::google::protobuf::TextFormat;
 using ::testing::_;
+using ::testing::AllOf;
+using ::testing::AnyOf;
 using ::testing::ByMove;
+using ::testing::Contains;
 using ::testing::HasSubstr;
+using ::testing::Not;
+using ::testing::Pair;
 using ::testing::Return;
 using ::testing::StrictMock;
 using ::testing::UnorderedElementsAre;
 
-namespace spanner_proto = ::google::spanner::v1;
+auto constexpr kRouteToLeader = "x-goog-spanner-route-to-leader";
 
-// Matches a BatchCreateSessionsRequest with the specified `session_count`.
-MATCHER_P(SessionCountIs, session_count,
-          "BatchCreateSessionsRequest has expected session_count") {
-  return arg.session_count() == session_count;
+class SessionPoolTest : public testing::Test {
+ protected:
+  std::multimap<std::string, std::string> GetMetadata(
+      grpc::ClientContext& context) {
+    return validate_metadata_fixture_.GetMetadata(context);
+  }
+
+ private:
+  testing_util::ValidateMetadataFixture validate_metadata_fixture_;
+};
+
+// Matches a BatchCreateSessionsRequest with the specified `database_name`.
+MATCHER_P(DatabaseIs, database_name,
+          "BatchCreateSessionsRequest has expected database name") {
+  return arg.database() == database_name;
 }
 
 // Matches a BatchCreateSessionsRequest with the specified `labels`.
@@ -65,12 +85,33 @@ MATCHER_P(LabelsAre, labels, "BatchCreateSessionsRequest has expected labels") {
   return labels_type(arg_labels.begin(), arg_labels.end()) == labels;
 }
 
+// Matches a BatchCreateSessionsRequest with the specified `role`.
+MATCHER_P(CreatorRoleIs, role,
+          "BatchCreateSessionsRequest has expected creator role") {
+  return arg.session_template().creator_role() == role;
+}
+
+// Matches a BatchCreateSessionsRequest with the specified `session_count`.
+MATCHER_P(SessionCountIs, session_count,
+          "BatchCreateSessionsRequest has expected session count") {
+  return arg.session_count() == session_count;
+}
+
+google::protobuf::Timestamp Now() {
+  auto now = spanner::MakeTimestamp(absl::Now()).value();
+  return now.get<google::protobuf::Timestamp>().value();
+}
+
 // Create a response with the given `sessions`
-spanner_proto::BatchCreateSessionsResponse MakeSessionsResponse(
-    std::vector<std::string> sessions) {
-  spanner_proto::BatchCreateSessionsResponse response;
+google::spanner::v1::BatchCreateSessionsResponse MakeSessionsResponse(
+    std::vector<std::string> sessions, std::string role = "") {
+  google::spanner::v1::BatchCreateSessionsResponse response;
   for (auto& session : sessions) {
-    response.add_session()->set_name(std::move(session));
+    auto* s = response.add_session();
+    s->set_name(std::move(session));
+    *s->mutable_create_time() = Now();
+    *s->mutable_approximate_last_use_time() = Now();
+    if (!role.empty()) s->set_creator_role(role);
   }
   return response;
 }
@@ -89,47 +130,69 @@ std::shared_ptr<SessionPool> MakeTestSessionPool(
                          std::move(opts));
 }
 
-TEST(SessionPool, Allocate) {
+TEST_F(SessionPoolTest, AllocateRouteToLeader) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
-  EXPECT_CALL(*mock, BatchCreateSessions)
-      .WillOnce(
-          [&db](grpc::ClientContext&,
-                spanner_proto::BatchCreateSessionsRequest const& request) {
-            EXPECT_EQ(db.FullName(), request.database());
-            EXPECT_EQ(1, request.session_count());
-            return MakeSessionsResponse({"session1"});
-          });
+  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
+                                                  SessionCountIs(42))))
+      .WillOnce([this](grpc::ClientContext& context,
+                       google::spanner::v1::BatchCreateSessionsRequest const&) {
+        EXPECT_THAT(GetMetadata(context),
+                    Contains(Pair(kRouteToLeader, "true")));
+        return MakeSessionsResponse({"session1"});
+      });
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock}, threads.cq());
+  auto pool =
+      MakeTestSessionPool(db, {mock}, threads.cq(),
+                          Options{}
+                              .set<spanner::RouteToLeaderOption>(true)
+                              .set<spanner::SessionPoolMinSessionsOption>(42));
   auto session = pool->Allocate();
   ASSERT_STATUS_OK(session);
   EXPECT_EQ((*session)->session_name(), "session1");
   EXPECT_EQ(pool->GetStub(**session), mock);
 }
 
-TEST(SessionPool, ReleaseBadSession) {
+TEST_F(SessionPoolTest, AllocateNoRouteToLeader) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
-  EXPECT_CALL(*mock, BatchCreateSessions)
-      .WillOnce(
-          [&db](grpc::ClientContext&,
-                spanner_proto::BatchCreateSessionsRequest const& request) {
-            EXPECT_EQ(db.FullName(), request.database());
-            EXPECT_EQ(1, request.session_count());
-            return MakeSessionsResponse({"session1"});
-          })
-      .WillOnce(
-          [&db](grpc::ClientContext&,
-                spanner_proto::BatchCreateSessionsRequest const& request) {
-            EXPECT_EQ(db.FullName(), request.database());
-            EXPECT_EQ(1, request.session_count());
-            return MakeSessionsResponse({"session2"});
-          });
+  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
+                                                  SessionCountIs(42))))
+      .WillOnce([this](grpc::ClientContext& context,
+                       google::spanner::v1::BatchCreateSessionsRequest const&) {
+        EXPECT_THAT(GetMetadata(context),
+                    AnyOf(Contains(Pair(kRouteToLeader, "false")),
+                          Not(Contains(Pair(kRouteToLeader, _)))));
+        return MakeSessionsResponse({"session1"});
+      });
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock}, threads.cq());
+  auto pool =
+      MakeTestSessionPool(db, {mock}, threads.cq(),
+                          Options{}
+                              .set<spanner::RouteToLeaderOption>(false)
+                              .set<spanner::SessionPoolMinSessionsOption>(42));
+  auto session = pool->Allocate();
+  ASSERT_STATUS_OK(session);
+  EXPECT_EQ((*session)->session_name(), "session1");
+  EXPECT_EQ(pool->GetStub(**session), mock);
+}
+
+TEST_F(SessionPoolTest, ReleaseBadSession) {
+  auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
+  auto db = spanner::Database("project", "instance", "database");
+  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
+                                                  SessionCountIs(1))))
+      .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}))));
+  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
+                                                  SessionCountIs(2))))
+      .WillOnce(Return(ByMove(MakeSessionsResponse({"session2"}))));
+
+  google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
+  auto pool = MakeTestSessionPool(
+      db, {mock}, threads.cq(),
+      Options{}.set<spanner::SessionPoolMinSessionsOption>(1));
   {
     auto session = pool->Allocate();
     ASSERT_STATUS_OK(session);
@@ -148,10 +211,11 @@ TEST(SessionPool, ReleaseBadSession) {
   }
 }
 
-TEST(SessionPool, CreateError) {
+TEST_F(SessionPoolTest, CreateError) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
   EXPECT_CALL(*mock, BatchCreateSessions)
+      .WillOnce(Return(ByMove(Status(StatusCode::kInternal, "init failure"))))
       .WillOnce(Return(ByMove(Status(StatusCode::kInternal, "some failure"))));
 
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
@@ -161,7 +225,7 @@ TEST(SessionPool, CreateError) {
               StatusIs(StatusCode::kInternal, HasSubstr("some failure")));
 }
 
-TEST(SessionPool, ReuseSession) {
+TEST_F(SessionPoolTest, ReuseSession) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
   EXPECT_CALL(*mock, BatchCreateSessions)
@@ -179,7 +243,7 @@ TEST(SessionPool, ReuseSession) {
   EXPECT_EQ((*session2)->session_name(), "session1");
 }
 
-TEST(SessionPool, Lifo) {
+TEST_F(SessionPoolTest, Lifo) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
   EXPECT_CALL(*mock, BatchCreateSessions)
@@ -210,36 +274,35 @@ TEST(SessionPool, Lifo) {
   EXPECT_EQ((*session4)->session_name(), "session1");
 }
 
-TEST(SessionPool, MinSessionsEagerAllocation) {
+TEST_F(SessionPoolTest, MinSessionsEagerAllocation) {
   int const min_sessions = 3;
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
-  EXPECT_CALL(*mock, BatchCreateSessions)
+  EXPECT_CALL(*mock, BatchCreateSessions(_, SessionCountIs(min_sessions)))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s3", "s2", "s1"}))));
 
-  Options opts;
-  opts.set<spanner::SessionPoolMinSessionsOption>(min_sessions);
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock}, threads.cq(), std::move(opts));
+  auto pool = MakeTestSessionPool(
+      db, {mock}, threads.cq(),
+      Options{}.set<spanner::SessionPoolMinSessionsOption>(min_sessions));
   auto session = pool->Allocate();
 }
 
-TEST(SessionPool, MinSessionsMultipleAllocations) {
+TEST_F(SessionPoolTest, MinSessionsMultipleAllocations) {
   int const min_sessions = 3;
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
   // The constructor will make this call.
   EXPECT_CALL(*mock, BatchCreateSessions(_, SessionCountIs(min_sessions)))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s3", "s2", "s1"}))));
-
-  Options opts;
-  opts.set<spanner::SessionPoolMinSessionsOption>(min_sessions);
-  google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock}, threads.cq(), std::move(opts));
-
   // When we run out of sessions it will make this call.
   EXPECT_CALL(*mock, BatchCreateSessions(_, SessionCountIs(min_sessions + 1)))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s7", "s6", "s5", "s4"}))));
+
+  google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
+  auto pool = MakeTestSessionPool(
+      db, {mock}, threads.cq(),
+      Options{}.set<spanner::SessionPoolMinSessionsOption>(min_sessions));
   std::vector<SessionHolder> sessions;
   std::vector<std::string> session_names;
   for (int i = 1; i <= 7; ++i) {
@@ -252,7 +315,7 @@ TEST(SessionPool, MinSessionsMultipleAllocations) {
               UnorderedElementsAre("s1", "s2", "s3", "s4", "s5", "s6", "s7"));
 }
 
-TEST(SessionPool, MaxSessionsFailOnExhaustion) {
+TEST_F(SessionPoolTest, MaxSessionsFailOnExhaustion) {
   int const max_sessions_per_channel = 3;
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
@@ -261,13 +324,14 @@ TEST(SessionPool, MaxSessionsFailOnExhaustion) {
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s2"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s3"}))));
 
-  Options opts;
-  opts.set<spanner::SessionPoolMaxSessionsPerChannelOption>(
-      max_sessions_per_channel);
-  opts.set<spanner::SessionPoolActionOnExhaustionOption>(
-      spanner::ActionOnExhaustion::kFail);
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock}, threads.cq(), std::move(opts));
+  auto pool = MakeTestSessionPool(
+      db, {mock}, threads.cq(),
+      Options{}
+          .set<spanner::SessionPoolMaxSessionsPerChannelOption>(
+              max_sessions_per_channel)
+          .set<spanner::SessionPoolActionOnExhaustionOption>(
+              spanner::ActionOnExhaustion::kFail));
   std::vector<SessionHolder> sessions;
   std::vector<std::string> session_names;
   for (int i = 1; i <= 3; ++i) {
@@ -282,20 +346,21 @@ TEST(SessionPool, MaxSessionsFailOnExhaustion) {
                                 "session pool exhausted"));
 }
 
-TEST(SessionPool, MaxSessionsBlockUntilRelease) {
+TEST_F(SessionPoolTest, MaxSessionsBlockUntilRelease) {
   int const max_sessions_per_channel = 1;
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s1"}))));
 
-  Options opts;
-  opts.set<spanner::SessionPoolMaxSessionsPerChannelOption>(
-      max_sessions_per_channel);
-  opts.set<spanner::SessionPoolActionOnExhaustionOption>(
-      spanner::ActionOnExhaustion::kBlock);
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock}, threads.cq(), std::move(opts));
+  auto pool = MakeTestSessionPool(
+      db, {mock}, threads.cq(),
+      Options{}
+          .set<spanner::SessionPoolMaxSessionsPerChannelOption>(
+              max_sessions_per_channel)
+          .set<spanner::SessionPoolActionOnExhaustionOption>(
+              spanner::ActionOnExhaustion::kBlock));
   auto session = pool->Allocate();
   ASSERT_STATUS_OK(session);
   EXPECT_EQ((*session)->session_name(), "s1");
@@ -311,7 +376,7 @@ TEST(SessionPool, MaxSessionsBlockUntilRelease) {
   t.join();
 }
 
-TEST(SessionPool, Labels) {
+TEST_F(SessionPoolTest, Labels) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
   std::map<std::string, std::string> labels = {
@@ -319,16 +384,33 @@ TEST(SessionPool, Labels) {
   EXPECT_CALL(*mock, BatchCreateSessions(_, LabelsAre(labels)))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}))));
 
-  Options opts;
-  opts.set<spanner::SessionPoolLabelsOption>(std::move(labels));
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock}, threads.cq(), std::move(opts));
+  auto pool = MakeTestSessionPool(
+      db, {mock}, threads.cq(),
+      Options{}.set<spanner::SessionPoolLabelsOption>(std::move(labels)));
   auto session = pool->Allocate();
   ASSERT_STATUS_OK(session);
   EXPECT_EQ((*session)->session_name(), "session1");
 }
 
-TEST(SessionPool, MultipleChannels) {
+TEST_F(SessionPoolTest, CreatorRole) {
+  auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
+  auto db = spanner::Database("project", "instance", "database");
+  std::string const role = "public";
+  EXPECT_CALL(*mock, BatchCreateSessions(_, AllOf(DatabaseIs(db.FullName()),
+                                                  CreatorRoleIs(role))))
+      .WillOnce(Return(ByMove(MakeSessionsResponse({"session1"}, role))));
+
+  google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
+  auto pool = MakeTestSessionPool(
+      db, {mock}, threads.cq(),
+      Options{}.set<spanner::SessionCreatorRoleOption>(role));
+  auto session = pool->Allocate();
+  ASSERT_STATUS_OK(session);
+  EXPECT_EQ((*session)->session_name(), "session1");
+}
+
+TEST_F(SessionPoolTest, MultipleChannels) {
   auto mock1 = std::make_shared<spanner_testing::MockSpannerStub>();
   auto mock2 = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
@@ -355,7 +437,7 @@ TEST(SessionPool, MultipleChannels) {
                                                   "c2s1", "c2s2", "c2s3"));
 }
 
-TEST(SessionPool, MultipleChannelsPreAllocation) {
+TEST_F(SessionPoolTest, MultipleChannelsPreAllocation) {
   auto mock1 = std::make_shared<spanner_testing::MockSpannerStub>();
   auto mock2 = std::make_shared<spanner_testing::MockSpannerStub>();
   auto mock3 = std::make_shared<spanner_testing::MockSpannerStub>();
@@ -367,15 +449,15 @@ TEST(SessionPool, MultipleChannelsPreAllocation) {
   EXPECT_CALL(*mock3, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"c3s1", "c3s2", "c3s3"}))));
 
+  google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
   // note that min_sessions will effectively be reduced to 9
   // (max_sessions_per_channel * num_channels).
-  Options opts;
-  opts.set<spanner::SessionPoolMaxSessionsPerChannelOption>(3);
-  opts.set<spanner::SessionPoolActionOnExhaustionOption>(
-      spanner::ActionOnExhaustion::kFail);
-  google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock1, mock2, mock3}, threads.cq(),
-                                  std::move(opts));
+  auto pool = MakeTestSessionPool(
+      db, {mock1, mock2, mock3}, threads.cq(),
+      Options{}
+          .set<spanner::SessionPoolMaxSessionsPerChannelOption>(3)
+          .set<spanner::SessionPoolActionOnExhaustionOption>(
+              spanner::ActionOnExhaustion::kFail));
   std::vector<SessionHolder> sessions;
   std::vector<std::string> session_names;
   for (int i = 1; i <= 9; ++i) {
@@ -392,23 +474,25 @@ TEST(SessionPool, MultipleChannelsPreAllocation) {
                                 "session pool exhausted"));
 }
 
-TEST(SessionPool, GetStubForStublessSession) {
+TEST_F(SessionPoolTest, GetStubForStublessSession) {
   auto mock = std::make_shared<spanner_testing::MockSpannerStub>();
   auto db = spanner::Database("project", "instance", "database");
   google::cloud::internal::AutomaticallyCreatedBackgroundThreads threads;
-  auto pool = MakeTestSessionPool(db, {mock}, threads.cq());
+  auto pool = MakeTestSessionPool(
+      db, {mock}, threads.cq(),
+      Options{}.set<spanner::SessionPoolMinSessionsOption>(0));
   // ensure we get a stub even if we didn't allocate from the pool.
   auto session = MakeDissociatedSessionHolder("session_id");
   EXPECT_EQ(pool->GetStub(*session), mock);
 }
 
-TEST(SessionPool, SessionRefresh) {
+TEST_F(SessionPoolTest, SessionRefresh) {
   auto mock = std::make_shared<StrictMock<spanner_testing::MockSpannerStub>>();
   EXPECT_CALL(*mock, BatchCreateSessions)
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s1"}))))
       .WillOnce(Return(ByMove(MakeSessionsResponse({"s2"}))));
 
-  spanner_proto::ResultSet result;
+  google::spanner::v1::ResultSet result;
   auto constexpr kResultSetText = R"pb(
     metadata: {
       row_type: { fields: { type: { code: INT64 } } }
@@ -419,21 +503,22 @@ TEST(SessionPool, SessionRefresh) {
   ASSERT_TRUE(TextFormat::ParseFromString(kResultSetText, &result));
 
   EXPECT_CALL(*mock, AsyncExecuteSql)
-      .WillOnce([&result](CompletionQueue&,
-                          std::unique_ptr<grpc::ClientContext>,
-                          spanner_proto::ExecuteSqlRequest const& request) {
-        EXPECT_EQ("s2", request.session());
-        return make_ready_future(make_status_or(std::move(result)));
-      });
+      .WillOnce(
+          [&result](CompletionQueue&, auto,
+                    google::spanner::v1::ExecuteSqlRequest const& request) {
+            EXPECT_EQ("s2", request.session());
+            return make_ready_future(make_status_or(std::move(result)));
+          });
 
   auto db = spanner::Database("project", "instance", "database");
   auto impl = std::make_shared<FakeCompletionQueueImpl>();
+  auto keep_alive_interval = std::chrono::seconds(1);
   auto clock = std::make_shared<FakeSteadyClock>();
-  auto opts = Options{}
-                  .set<spanner::SessionPoolKeepAliveIntervalOption>(
-                      std::chrono::seconds(1))
-                  .set<SessionPoolClockOption>(clock);
-  auto pool = MakeTestSessionPool(db, {mock}, CompletionQueue(impl), opts);
+  auto pool = MakeTestSessionPool(
+      db, {mock}, CompletionQueue(impl),
+      Options{}
+          .set<spanner::SessionPoolKeepAliveIntervalOption>(keep_alive_interval)
+          .set<SessionPoolClockOption>(clock));
 
   // Allocate and release two session, "s1" and "s2". This will satisfy the
   // BatchCreateSessions() expectations.
@@ -447,14 +532,78 @@ TEST(SessionPool, SessionRefresh) {
       EXPECT_EQ("s2", (*s2)->session_name());
     }
     // Wait for "s2" to need refreshing before releasing "s1".
-    clock->AdvanceTime(opts.get<spanner::SessionPoolKeepAliveIntervalOption>() *
-                       2);
+    clock->AdvanceTime(keep_alive_interval * 2);
   }
 
   // Simulate completion of pending operations, which will result in
   // a call to RefreshExpiringSessions(). This should refresh "s2" and
   // satisfy the AsyncExecuteSql() expectation.
   impl->SimulateCompletion(true);
+
+  // We should still be able to allocate sessions "s1" and "s2".
+  auto s1 = pool->Allocate();
+  ASSERT_STATUS_OK(s1);
+  EXPECT_EQ("s1", (*s1)->session_name());
+  auto s2 = pool->Allocate();
+  ASSERT_STATUS_OK(s2);
+  EXPECT_EQ("s2", (*s2)->session_name());
+}
+
+TEST_F(SessionPoolTest, SessionRefreshNotFound) {
+  auto mock = std::make_shared<StrictMock<spanner_testing::MockSpannerStub>>();
+  EXPECT_CALL(*mock, BatchCreateSessions)
+      .WillOnce(Return(ByMove(MakeSessionsResponse({"s1"}))))
+      .WillOnce(Return(ByMove(MakeSessionsResponse({"s2"}))))
+      .WillOnce(Return(ByMove(MakeSessionsResponse({"s3"}))));
+
+  EXPECT_CALL(*mock, AsyncExecuteSql)
+      .WillOnce([](CompletionQueue&, auto,
+                   google::spanner::v1::ExecuteSqlRequest const& request) {
+        EXPECT_EQ("s2", request.session());
+        // The "SELECT 1" refresh returns "Session not found".
+        return make_ready_future(StatusOr<google::spanner::v1::ResultSet>(
+            spanner_testing::SessionNotFoundError(request.session())));
+      });
+
+  auto db = spanner::Database("project", "instance", "database");
+  auto impl = std::make_shared<FakeCompletionQueueImpl>();
+  auto keep_alive_interval = std::chrono::seconds(1);
+  auto clock = std::make_shared<FakeSteadyClock>();
+  auto pool = MakeTestSessionPool(
+      db, {mock}, CompletionQueue(impl),
+      Options{}
+          .set<spanner::SessionPoolKeepAliveIntervalOption>(keep_alive_interval)
+          .set<SessionPoolClockOption>(clock));
+
+  // Allocate and release two session, "s1" and "s2". This will satisfy the
+  // the first two BatchCreateSessions() expectations.
+  {
+    auto s1 = pool->Allocate();
+    ASSERT_STATUS_OK(s1);
+    EXPECT_EQ("s1", (*s1)->session_name());
+    {
+      auto s2 = pool->Allocate();
+      ASSERT_STATUS_OK(s2);
+      EXPECT_EQ("s2", (*s2)->session_name());
+    }
+    // Wait for "s2" to need refreshing before releasing "s1".
+    clock->AdvanceTime(keep_alive_interval * 2);
+  }
+
+  // Simulate completion of pending operations, which will result in
+  // a call to RefreshExpiringSessions(). This should refresh "s2" and
+  // satisfy the AsyncExecuteSql() expectation, which fails the call.
+  impl->SimulateCompletion(true);
+
+  // We should still be able to allocate session "s1".
+  auto s1 = pool->Allocate();
+  ASSERT_STATUS_OK(s1);
+  EXPECT_EQ("s1", (*s1)->session_name());
+  // However "s2" will be gone now, so a new allocation will produce
+  // "s3", satisfying the final BatchCreateSessions() expectation.
+  auto s3 = pool->Allocate();
+  ASSERT_STATUS_OK(s3);
+  EXPECT_EQ("s3", (*s3)->session_name());
 }
 
 }  // namespace
