@@ -24,8 +24,11 @@ namespace cloud {
 namespace spanner_testing {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
-/// Create a random instance name given a PRNG generator.
-std::string RandomInstanceName(google::cloud::internal::DefaultPRNG& generator);
+/// Returns a random instance ID given a PRNG generator.
+std::string RandomInstanceName(internal::DefaultPRNG&);
+
+/// Returns a random instance config ID given a PRNG generator.
+std::string RandomInstanceConfigName(internal::DefaultPRNG&);
 
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace spanner_testing
