@@ -34,13 +34,35 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
 using ::google::cloud::testing_util::IsOk;
+using ::google::cloud::testing_util::IsOkAndHolds;
 using ::google::cloud::testing_util::IsProtoEqual;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::HasSubstr;
 using ::testing::Not;
 
-absl::Time MakeTime(std::time_t sec, int nanos) {
-  return absl::FromTimeT(sec) + absl::Nanoseconds(nanos);
+absl::Time MakeTime(std::int64_t sec, int nanos) {
+  return absl::FromUnixSeconds(sec) + absl::Nanoseconds(nanos);
+}
+
+std::vector<Timestamp> TestTimes() {
+  std::vector<Timestamp> times;
+  for (auto s : {
+           std::int64_t{-9223372035LL},  // near the limit of 64-bit/ns clock
+           std::int64_t{-2147483649LL},  // below min 32-bit value
+           std::int64_t{-2147483648LL},  // min 32-bit value
+           std::int64_t{-1},             // just before Unix epoch
+           std::int64_t{0},              // Unix epoch
+           std::int64_t{1},              // just after Unix epoch
+           std::int64_t{1561147549LL},   // contemporary
+           std::int64_t{2147483647LL},   // max 32-bit value
+           std::int64_t{2147483648LL},   // above max 32-bit value
+           std::int64_t{9223372036LL},   // near the limit of 64-bit/ns clock
+       }) {
+    for (auto nanos : {-1, 0, 1}) {
+      times.push_back(MakeTimestamp(MakeTime(s, nanos)).value());
+    }
+  }
+  return times;
 }
 
 template <typename T>
@@ -50,7 +72,7 @@ void TestBasicSemantics(T init) {
 
   Value const v{init};
 
-  EXPECT_STATUS_OK(v.get<T>());
+  ASSERT_STATUS_OK(v.get<T>());
   EXPECT_EQ(init, *v.get<T>());
 
   Value copy = v;
@@ -62,7 +84,7 @@ void TestBasicSemantics(T init) {
   Value const null = MakeNullValue<T>();
 
   EXPECT_THAT(null.get<T>(), Not(IsOk()));
-  EXPECT_STATUS_OK(null.get<absl::optional<T>>());
+  ASSERT_STATUS_OK(null.get<absl::optional<T>>());
   EXPECT_EQ(absl::optional<T>{}, *null.get<absl::optional<T>>());
 
   Value copy_null = null;
@@ -81,9 +103,9 @@ void TestBasicSemantics(T init) {
             google::protobuf::NullValue::NULL_VALUE);
 
   Value const not_null{absl::optional<T>(init)};
-  EXPECT_STATUS_OK(not_null.get<T>());
+  ASSERT_STATUS_OK(not_null.get<T>());
   EXPECT_EQ(init, *not_null.get<T>());
-  EXPECT_STATUS_OK(not_null.get<absl::optional<T>>());
+  ASSERT_STATUS_OK(not_null.get<absl::optional<T>>());
   EXPECT_EQ(init, **not_null.get<absl::optional<T>>());
 }
 
@@ -150,6 +172,16 @@ TEST(Value, BasicSemantics) {
     TestBasicSemantics(v);
   }
 
+  for (auto const& x : std::vector<JsonB>{JsonB(), JsonB(R"("Hello world!")"),
+                                          JsonB("42"), JsonB("true")}) {
+    SCOPED_TRACE("Testing: JSONB " + std::string(x));
+    TestBasicSemantics(x);
+    TestBasicSemantics(std::vector<JsonB>(5, x));
+    std::vector<absl::optional<JsonB>> v(5, x);
+    v.resize(10);
+    TestBasicSemantics(v);
+  }
+
   for (auto const& x : {
            MakeNumeric(-0.9e29).value(),
            MakeNumeric(-1).value(),
@@ -167,27 +199,15 @@ TEST(Value, BasicSemantics) {
     TestBasicSemantics(v);
   }
 
-  for (time_t t : {
-           -9223372035LL,   // near the limit of 64-bit/ns system_clock
-           -2147483649LL,   // below min 32-bit int
-           -2147483648LL,   // min 32-bit int
-           -1LL, 0LL, 1LL,  // around the unix epoch
-           1561147549LL,    // contemporary
-           2147483647LL,    // max 32-bit int
-           2147483648LL,    // above max 32-bit int
-           9223372036LL     // near the limit of 64-bit/ns system_clock
-       }) {
-    for (auto nanos : {-1, 0, 1}) {
-      auto ts = MakeTimestamp(MakeTime(t, nanos)).value();
-      SCOPED_TRACE("Testing: google::cloud::spanner::Timestamp " +
-                   spanner_internal::TimestampToRFC3339(ts));
-      TestBasicSemantics(ts);
-      std::vector<Timestamp> v(5, ts);
-      TestBasicSemantics(v);
-      std::vector<absl::optional<Timestamp>> ov(5, ts);
-      ov.resize(10);
-      TestBasicSemantics(ov);
-    }
+  for (auto ts : TestTimes()) {
+    SCOPED_TRACE("Testing: google::cloud::spanner::Timestamp " +
+                 spanner_internal::TimestampToRFC3339(ts));
+    TestBasicSemantics(ts);
+    std::vector<Timestamp> v(5, ts);
+    TestBasicSemantics(v);
+    std::vector<absl::optional<Timestamp>> ov(5, ts);
+    ov.resize(10);
+    TestBasicSemantics(ov);
   }
 
   for (auto x : {
@@ -224,7 +244,10 @@ TEST(Value, Equality) {
       {Value("foo"), Value("bar")},
       {Value(Bytes("foo")), Value(Bytes("bar"))},
       {Value(Json("42")), Value(Json("true"))},
+      {Value(JsonB("42")), Value(JsonB("true"))},
       {Value(MakeNumeric(0).value()), Value(MakeNumeric(1).value())},
+      {Value(MakePgNumeric(0).value()), Value(MakePgNumeric(1).value())},
+      {Value(PgOid(200)), Value(PgOid(999))},
       {Value(absl::CivilDay(1970, 1, 1)), Value(absl::CivilDay(2020, 3, 15))},
       {Value(std::vector<double>{1.2, 3.4}),
        Value(std::vector<double>{4.5, 6.7})},
@@ -256,16 +279,16 @@ TEST(Value, RvalueGetString) {
   Value v(data);
 
   auto s = v.get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(data, *s);
 
   s = std::move(v).get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(data, *s);
 
   // NOLINTNEXTLINE(bugprone-use-after-move)
   s = v.get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ("", *s);
 }
 
@@ -281,16 +304,16 @@ TEST(Value, RvalueGetOptionalString) {
   Value v(data);
 
   auto s = v.get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(*data, **s);
 
   s = std::move(v).get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(*data, **s);
 
   // NOLINTNEXTLINE(bugprone-use-after-move)
   s = v.get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ("", **s);
 }
 
@@ -306,16 +329,16 @@ TEST(Value, RvalueGetVectorString) {
   Value v(data);
 
   auto s = v.get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(data, *s);
 
   s = std::move(v).get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(data, *s);
 
   // NOLINTNEXTLINE(bugprone-use-after-move)
   s = v.get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(Type(data.size(), ""), *s);
 }
 
@@ -332,16 +355,16 @@ TEST(Value, RvalueGetStructString) {
   Value v(data);
 
   auto s = v.get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(data, *s);
 
   s = std::move(v).get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(data, *s);
 
   // NOLINTNEXTLINE(bugprone-use-after-move)
   s = v.get<Type>();
-  EXPECT_STATUS_OK(s);
+  ASSERT_STATUS_OK(s);
   EXPECT_EQ(Type({"name", ""}, ""), *s);
 }
 
@@ -350,10 +373,11 @@ TEST(Value, DoubleNaN) {
   Value v{nan};
   EXPECT_TRUE(std::isnan(*v.get<double>()));
 
-  // Since IEEE 754 defines that nan is not equal to itself, then a Value with
-  // NaN should not be equal to itself.
+  // Unlike IEEE 754, which defines that NaN is not equal to itself,
+  // Spanner NaN values are considered equal (for easy sorting), so
+  // spanner::Value behaves the same way.
   EXPECT_NE(nan, nan);
-  EXPECT_NE(v, v);
+  EXPECT_EQ(v, v);
 }
 
 TEST(Value, BytesDecodingError) {
@@ -389,6 +413,14 @@ TEST(Value, JsonRelationalOperators) {
   EXPECT_NE(j1, j2);
 }
 
+TEST(Value, JsonBRelationalOperators) {
+  JsonB jb1("42");
+  JsonB jb2("true");
+
+  EXPECT_EQ(jb1, jb1);
+  EXPECT_NE(jb1, jb2);
+}
+
 TEST(Value, ConstructionFromLiterals) {
   Value v_int64(42);
   EXPECT_EQ(42, *v_int64.get<std::int64_t>());
@@ -418,7 +450,6 @@ TEST(Value, MixingTypes) {
   Value a(A{});
   EXPECT_STATUS_OK(a.get<A>());
   EXPECT_THAT(a.get<B>(), Not(IsOk()));
-  EXPECT_THAT(a.get<B>(), Not(IsOk()));
 
   Value null_a = MakeNullValue<A>();
   EXPECT_THAT(null_a.get<A>(), Not(IsOk()));
@@ -429,7 +460,6 @@ TEST(Value, MixingTypes) {
   Value b(B{});
   EXPECT_STATUS_OK(b.get<B>());
   EXPECT_THAT(b.get<A>(), Not(IsOk()));
-  EXPECT_THAT(b.get<A>(), Not(IsOk()));
 
   EXPECT_NE(b, a);
   EXPECT_NE(b, null_a);
@@ -450,14 +480,14 @@ TEST(Value, SpannerArray) {
   ArrayInt64 const empty = {};
   Value const ve(empty);
   EXPECT_EQ(ve, ve);
-  EXPECT_STATUS_OK(ve.get<ArrayInt64>());
+  ASSERT_STATUS_OK(ve.get<ArrayInt64>());
   EXPECT_THAT(ve.get<ArrayDouble>(), Not(IsOk()));
   EXPECT_EQ(empty, *ve.get<ArrayInt64>());
 
   ArrayInt64 const ai = {1, 2, 3};
   Value const vi(ai);
   EXPECT_EQ(vi, vi);
-  EXPECT_STATUS_OK(vi.get<ArrayInt64>());
+  ASSERT_STATUS_OK(vi.get<ArrayInt64>());
   EXPECT_THAT(vi.get<ArrayDouble>(), Not(IsOk()));
   EXPECT_EQ(ai, *vi.get<ArrayInt64>());
 
@@ -466,7 +496,7 @@ TEST(Value, SpannerArray) {
   EXPECT_EQ(vd, vd);
   EXPECT_NE(vi, vd);
   EXPECT_THAT(vd.get<ArrayInt64>(), Not(IsOk()));
-  EXPECT_STATUS_OK(vd.get<ArrayDouble>());
+  ASSERT_STATUS_OK(vd.get<ArrayDouble>());
   EXPECT_EQ(ad, *vd.get<ArrayDouble>());
 
   Value const null_vi = MakeNullValue<ArrayInt64>();
@@ -497,30 +527,29 @@ TEST(Value, SpannerStruct) {
   auto tup1 = make_tuple(false, int64_t{123});
   using T1 = decltype(tup1);
   Value v1(tup1);
-  EXPECT_STATUS_OK(v1.get<T1>());
+  ASSERT_STATUS_OK(v1.get<T1>());
   EXPECT_EQ(tup1, *v1.get<T1>());
   EXPECT_EQ(v1, v1);
 
   // Verify we can extract tuple elements even if they're wrapped in a pair.
   auto const pair0 = v1.get<tuple<pair<string, bool>, int64_t>>();
-  EXPECT_STATUS_OK(pair0);
+  ASSERT_STATUS_OK(pair0);
   EXPECT_EQ(std::get<0>(tup1), std::get<0>(*pair0).second);
   EXPECT_EQ(std::get<1>(tup1), std::get<1>(*pair0));
   auto const pair1 = v1.get<tuple<bool, pair<string, int64_t>>>();
-  EXPECT_STATUS_OK(pair1);
+  ASSERT_STATUS_OK(pair1);
   EXPECT_EQ(std::get<0>(tup1), std::get<0>(*pair1));
   EXPECT_EQ(std::get<1>(tup1), std::get<1>(*pair1).second);
   auto const pair01 =
       v1.get<tuple<pair<string, bool>, pair<string, int64_t>>>();
-  EXPECT_STATUS_OK(pair01);
+  ASSERT_STATUS_OK(pair01);
   EXPECT_EQ(std::get<0>(tup1), std::get<0>(*pair01).second);
   EXPECT_EQ(std::get<1>(tup1), std::get<1>(*pair01).second);
 
   auto tup2 = make_tuple(false, make_pair(string("f2"), int64_t{123}));
   using T2 = decltype(tup2);
   Value v2(tup2);
-  EXPECT_STATUS_OK(v2.get<T2>());
-  EXPECT_EQ(tup2, *v2.get<T2>());
+  EXPECT_THAT(v2.get<T2>(), IsOkAndHolds(tup2));
   EXPECT_EQ(v2, v2);
   EXPECT_NE(v2, v1);
 
@@ -531,8 +560,7 @@ TEST(Value, SpannerStruct) {
   auto tup3 = make_tuple(false, make_pair(string("Other"), int64_t{123}));
   using T3 = decltype(tup3);
   Value v3(tup3);
-  EXPECT_STATUS_OK(v3.get<T3>());
-  EXPECT_EQ(tup3, *v3.get<T3>());
+  EXPECT_THAT(v3.get<T3>(), IsOkAndHolds(tup3));
   EXPECT_EQ(v3, v3);
   EXPECT_NE(v3, v2);
   EXPECT_NE(v3, v1);
@@ -563,8 +591,7 @@ TEST(Value, SpannerStruct) {
   EXPECT_THAT(v4.get<T2>(), Not(IsOk()));
   EXPECT_THAT(v4.get<T1>(), Not(IsOk()));
 
-  EXPECT_STATUS_OK(v4.get<T4>());
-  EXPECT_EQ(array_struct, *v4.get<T4>());
+  EXPECT_THAT(v4.get<T4>(), IsOkAndHolds(array_struct));
 
   auto empty = tuple<>{};
   using T5 = decltype(empty);
@@ -574,8 +601,7 @@ TEST(Value, SpannerStruct) {
   EXPECT_EQ(v5, v5);
   EXPECT_NE(v5, v4);
 
-  EXPECT_STATUS_OK(v5.get<T5>());
-  EXPECT_EQ(empty, *v5.get<T5>());
+  EXPECT_THAT(v5.get<T5>(), IsOkAndHolds(empty));
 
   auto deeply_nested = tuple<tuple<std::vector<absl::optional<bool>>>>{};
   using T6 = decltype(deeply_nested);
@@ -585,8 +611,7 @@ TEST(Value, SpannerStruct) {
   EXPECT_EQ(v6, v6);
   EXPECT_NE(v6, v5);
 
-  EXPECT_STATUS_OK(v6.get<T6>());
-  EXPECT_EQ(deeply_nested, *v6.get<T6>());
+  EXPECT_THAT(v6.get<T6>(), IsOkAndHolds(deeply_nested));
 }
 
 TEST(Value, SpannerStructWithNull) {
@@ -655,9 +680,9 @@ TEST(Value, ProtoConversionFloat64) {
   auto const nan = std::nan("NaN");
   v = Value(nan);
   p = spanner_internal::ToProto(v);
-  // Note: NaN is defined to be not equal to everything, including itself, so
-  // we instead ensure that it is not equal with EXPECT_NE.
-  EXPECT_NE(v, spanner_internal::FromProto(p.first, p.second));
+  // Note: Unlike IEEE 754, Spanner NaN values are considered equal
+  // (for easy sorting), so spanner::Value behaves the same way.
+  EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
   EXPECT_EQ(google::spanner::v1::TypeCode::FLOAT64, p.first.code());
   EXPECT_EQ("NaN", p.second.string_value());
 }
@@ -691,6 +716,22 @@ TEST(Value, ProtoConversionJson) {
     auto const p = spanner_internal::ToProto(v);
     EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
     EXPECT_EQ(google::spanner::v1::TypeCode::JSON, p.first.code());
+    EXPECT_EQ(google::spanner::v1::TypeAnnotationCode::
+                  TYPE_ANNOTATION_CODE_UNSPECIFIED,
+              p.first.type_annotation());
+    EXPECT_EQ(std::string(x), p.second.string_value());
+  }
+}
+
+TEST(Value, ProtoConversionJsonB) {
+  for (auto const& x : std::vector<JsonB>{JsonB(), JsonB(R"("Hello world!")"),
+                                          JsonB("42"), JsonB("true")}) {
+    Value const v(x);
+    auto const p = spanner_internal::ToProto(v);
+    EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+    EXPECT_EQ(google::spanner::v1::TypeCode::JSON, p.first.code());
+    EXPECT_EQ(google::spanner::v1::TypeAnnotationCode::PG_JSONB,
+              p.first.type_annotation());
     EXPECT_EQ(std::string(x), p.second.string_value());
   }
 }
@@ -713,26 +754,31 @@ TEST(Value, ProtoConversionNumeric) {
   }
 }
 
-TEST(Value, ProtoConversionTimestamp) {
-  for (time_t t : {
-           -9223372035LL,   // near the limit of 64-bit/ns system_clock
-           -2147483649LL,   // below min 32-bit int
-           -2147483648LL,   // min 32-bit int
-           -1LL, 0LL, 1LL,  // around the unix epoch
-           1561147549LL,    // contemporary
-           2147483647LL,    // max 32-bit int
-           2147483648LL,    // above max 32-bit int
-           9223372036LL     // near the limit of 64-bit/ns system_clock
+TEST(Value, ProtoConversionPgOid) {
+  for (auto const& x : std::vector<PgOid>{
+           PgOid(0),
+           PgOid(200),
+           PgOid(999),
        }) {
-    for (auto nanos : {-1, 0, 1}) {
-      auto ts = MakeTimestamp(MakeTime(t, nanos)).value();
-      Value const v(ts);
-      auto const p = spanner_internal::ToProto(v);
-      EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
-      EXPECT_EQ(google::spanner::v1::TypeCode::TIMESTAMP, p.first.code());
-      EXPECT_EQ(spanner_internal::TimestampToRFC3339(ts),
-                p.second.string_value());
-    }
+    Value const v(x);
+    auto const p = spanner_internal::ToProto(v);
+    EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+    EXPECT_EQ(google::spanner::v1::TypeCode::INT64, p.first.code());
+    EXPECT_EQ(google::spanner::v1::TypeAnnotationCode::PG_OID,
+              p.first.type_annotation());
+    EXPECT_EQ(std::to_string(static_cast<std::uint64_t>(x)),
+              p.second.string_value());
+  }
+}
+
+TEST(Value, ProtoConversionTimestamp) {
+  for (auto ts : TestTimes()) {
+    Value const v(ts);
+    auto const p = spanner_internal::ToProto(v);
+    EXPECT_EQ(v, spanner_internal::FromProto(p.first, p.second));
+    EXPECT_EQ(google::spanner::v1::TypeCode::TIMESTAMP, p.first.code());
+    EXPECT_EQ(spanner_internal::TimestampToRFC3339(ts),
+              p.second.string_value());
   }
 }
 
@@ -908,6 +954,21 @@ TEST(Value, GetBadJson) {
   EXPECT_THAT(v.get<Json>(), Not(IsOk()));
 }
 
+TEST(Value, GetBadJsonB) {
+  Value v(JsonB("true"));
+  ClearProtoKind(v);
+  EXPECT_THAT(v.get<JsonB>(), Not(IsOk()));
+
+  SetProtoKind(v, google::protobuf::NULL_VALUE);
+  EXPECT_THAT(v.get<JsonB>(), Not(IsOk()));
+
+  SetProtoKind(v, true);
+  EXPECT_THAT(v.get<JsonB>(), Not(IsOk()));
+
+  SetProtoKind(v, 0.0);
+  EXPECT_THAT(v.get<JsonB>(), Not(IsOk()));
+}
+
 TEST(Value, GetBadNumeric) {
   Value v(MakeNumeric(0).value());
   ClearProtoKind(v);
@@ -932,6 +993,21 @@ TEST(Value, GetBadNumeric) {
   EXPECT_THAT(v.get<std::int64_t>(), Not(IsOk()));
 }
 
+TEST(Value, GetBadPgOid) {
+  Value v(PgOid(1));
+  ClearProtoKind(v);
+  EXPECT_THAT(v.get<std::string>(), Not(IsOk()));
+
+  SetProtoKind(v, google::protobuf::NULL_VALUE);
+  EXPECT_THAT(v.get<PgOid>(), Not(IsOk()));
+
+  SetProtoKind(v, true);
+  EXPECT_THAT(v.get<PgOid>(), Not(IsOk()));
+
+  SetProtoKind(v, 0.0);
+  EXPECT_THAT(v.get<PgOid>(), Not(IsOk()));
+}
+
 TEST(Value, GetBadInt) {
   Value v(42);
   ClearProtoKind(v);
@@ -1071,8 +1147,7 @@ TEST(Value, CommitTimestamp) {
   EXPECT_THAT(tv.second, IsProtoEqual(pv));
 
   auto good = v.get<CommitTimestamp>();
-  EXPECT_STATUS_OK(good);
-  EXPECT_EQ(CommitTimestamp{}, *good);
+  EXPECT_THAT(good, IsOkAndHolds(CommitTimestamp{}));
 
   auto bad = v.get<Timestamp>();
   EXPECT_THAT(bad, Not(IsOk()));
@@ -1094,7 +1169,6 @@ TEST(Value, OutputStream) {
   };
 
   auto const inf = std::numeric_limits<double>::infinity();
-  auto const nan = std::nan("NaN");
 
   struct TestCase {
     Value value;
@@ -1113,14 +1187,17 @@ TEST(Value, OutputStream) {
       {Value(42.0), "42.00", float4},
       {Value(inf), "inf", normal},
       {Value(-inf), "-inf", normal},
-      {Value(nan), "nan", normal},
       {Value(""), "", normal},
       {Value("foo"), "foo", normal},
       {Value("NULL"), "NULL", normal},
       {Value(Bytes(std::string("DEADBEEF"))), R"(B"DEADBEEF")", normal},
       {Value(Json()), "null", normal},
       {Value(Json("true")), "true", normal},
+      {Value(JsonB()), "null", normal},
+      {Value(JsonB("true")), "true", normal},
       {Value(MakeNumeric(1234567890).value()), "1234567890", normal},
+      {Value(MakePgNumeric(1234567890).value()), "1234567890", normal},
+      {Value(PgOid(1234567890)), "1234567890", normal},
       {Value(absl::CivilDay()), "1970-01-01", normal},
       {Value(Timestamp()), "1970-01-01T00:00:00Z", normal},
 
@@ -1144,7 +1221,9 @@ TEST(Value, OutputStream) {
       {MakeNullValue<std::string>(), "NULL", normal},
       {MakeNullValue<Bytes>(), "NULL", normal},
       {MakeNullValue<Json>(), "NULL", normal},
+      {MakeNullValue<JsonB>(), "NULL", normal},
       {MakeNullValue<Numeric>(), "NULL", normal},
+      {MakeNullValue<PgOid>(), "NULL", normal},
       {MakeNullValue<absl::CivilDay>(), "NULL", normal},
       {MakeNullValue<Timestamp>(), "NULL", normal},
 
@@ -1158,10 +1237,13 @@ TEST(Value, OutputStream) {
       {Value(std::vector<std::string>{"a", "b"}), R"(["a", "b"])", normal},
       {Value(std::vector<Bytes>{2}), R"([B"", B""])", normal},
       {Value(std::vector<Json>{2}), R"([null, null])", normal},
+      {Value(std::vector<JsonB>{2}), R"([null, null])", normal},
       {Value(std::vector<Numeric>{2}), "[0, 0]", normal},
       {Value(std::vector<absl::CivilDay>{2}), "[1970-01-01, 1970-01-01]",
        normal},
       {Value(std::vector<Timestamp>{1}), "[1970-01-01T00:00:00Z]", normal},
+
+      // Tests arrays with null elements
       {Value(std::vector<absl::optional<double>>{1, {}, 2}), "[1, NULL, 2]",
        normal},
 
@@ -1172,6 +1254,7 @@ TEST(Value, OutputStream) {
       {MakeNullValue<std::vector<std::string>>(), "NULL", normal},
       {MakeNullValue<std::vector<Bytes>>(), "NULL", normal},
       {MakeNullValue<std::vector<Json>>(), "NULL", normal},
+      {MakeNullValue<std::vector<JsonB>>(), "NULL", normal},
       {MakeNullValue<std::vector<Numeric>>(), "NULL", normal},
       {MakeNullValue<std::vector<absl::CivilDay>>(), "NULL", normal},
       {MakeNullValue<std::vector<Timestamp>>(), "NULL", normal},
@@ -1216,6 +1299,7 @@ TEST(Value, OutputStream) {
       {MakeNullValue<std::tuple<double, Bytes, Timestamp>>(), "NULL", normal},
       {MakeNullValue<std::tuple<Numeric, absl::CivilDay>>(), "NULL", normal},
       {MakeNullValue<std::tuple<Json, std::vector<bool>>>(), "NULL", normal},
+      {MakeNullValue<std::tuple<JsonB, std::vector<bool>>>(), "NULL", normal},
   };
 
   for (auto const& tc : test_case) {
@@ -1223,6 +1307,12 @@ TEST(Value, OutputStream) {
     tc.manip(ss) << tc.value;
     EXPECT_EQ(ss.str(), tc.expected);
   }
+
+  // `double std::nan("")` is a special case because the output conversion
+  // is implementation defined. So, we just look for a "nan" substring.
+  std::stringstream ss;
+  ss << Value(std::nan(""));
+  EXPECT_THAT(ss.str(), HasSubstr("nan"));
 }
 
 // Ensures that the following expressions produce the same output.
@@ -1271,11 +1361,28 @@ TEST(Value, OutputStreamMatchesT) {
   StreamMatchesValueStream(Json("42"));
   StreamMatchesValueStream(Json("true"));
 
+  // JSONB
+  StreamMatchesValueStream(JsonB());
+  StreamMatchesValueStream(JsonB(R"("Hello world!")"));
+  StreamMatchesValueStream(JsonB("42"));
+  StreamMatchesValueStream(JsonB("true"));
+
   // Numeric
   StreamMatchesValueStream(MakeNumeric("999").value());
   StreamMatchesValueStream(MakeNumeric(3.14159).value());
   StreamMatchesValueStream(MakeNumeric(42).value());
 
+  // PgNumeric
+  StreamMatchesValueStream(MakePgNumeric("999").value());
+  StreamMatchesValueStream(MakePgNumeric(3.14159).value());
+  StreamMatchesValueStream(MakePgNumeric(42).value());
+  StreamMatchesValueStream(MakePgNumeric("NaN").value());
+
+  // PgOid
+  StreamMatchesValueStream(PgOid(999));
+  StreamMatchesValueStream(PgOid(42));
+  StreamMatchesValueStream(PgOid(0));
+
   // Date
   StreamMatchesValueStream(absl::CivilDay(1, 1, 1));
   StreamMatchesValueStream(absl::CivilDay());
