@@ -17,6 +17,7 @@
 set -euo pipefail
 
 source "$(dirname "$0")/../../../../ci/lib/init.sh"
+source module /ci/cloudbuild/builds/lib/ctest.sh
 source module /ci/etc/integration-tests-config.sh
 source module /ci/lib/run_gcs_httpbin_emulator_utils.sh
 
@@ -33,16 +34,15 @@ readonly BINARY_DIR
 shift
 ctest_args=("$@")
 
+if ctest::has_no_tests "${BINARY_DIR}" "^storage_" "${ctest_args[@]}"; then
+  exit 0
+fi
+
 cd "${BINARY_DIR}"
 start_emulator
 
-# Create the test buckets in the emulator:
-printf '{"name": "%s"}' "${GOOGLE_CLOUD_CPP_STORAGE_TEST_BUCKET_NAME}" |
-  curl -X POST -H "Content-Type: application/json" --data-binary @- \
-    "${CLOUD_STORAGE_EMULATOR_ENDPOINT}/storage/v1/b?project=${GOOGLE_CLOUD_PROJECT}"
-printf '{"name": "%s"}' "${GOOGLE_CLOUD_CPP_STORAGE_TEST_DESTINATION_BUCKET_NAME}" |
-  curl -s -X POST -H "Content-Type: application/json" --data-binary @- \
-    "${CLOUD_STORAGE_EMULATOR_ENDPOINT}/storage/v1/b?project=${GOOGLE_CLOUD_PROJECT}"
+# Create any GCS resources needed to run the tests
+create_testbench_resources
 
 ctest -R "^storage_" "${ctest_args[@]}"
 exit_status=$?
