@@ -13,66 +13,64 @@
 // limitations under the License.
 
 #include "google/cloud/storage/client.h"
-#include "google/cloud/storage/internal/curl_client.h"
-#include "google/cloud/storage/internal/curl_handle.h"
+#include "google/cloud/storage/internal/connection_factory.h"
 #include "google/cloud/storage/internal/openssl_util.h"
 #include "google/cloud/storage/oauth2/service_account_credentials.h"
-#include "google/cloud/internal/algorithm.h"
+#include "google/cloud/internal/curl_handle.h"
+#include "google/cloud/internal/curl_options.h"
 #include "google/cloud/internal/filesystem.h"
+#include "google/cloud/internal/make_status.h"
+#include "google/cloud/internal/opentelemetry.h"
 #include "google/cloud/log.h"
-#include "absl/memory/memory.h"
-#include <openssl/md5.h>
 #include <fstream>
+#include <memory>
 #include <thread>
+#include <utility>
 
 namespace google {
 namespace cloud {
 namespace storage {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 
+using ::google::cloud::rest_internal::CurlHandle;
+
 static_assert(std::is_copy_constructible<storage::Client>::value,
               "storage::Client must be constructible");
 static_assert(std::is_copy_assignable<storage::Client>::value,
               "storage::Client must be assignable");
 
+// This is the constructor used by most applications. We apply the default
+// options, and then call the constructor that creates a connection.
 Client::Client(Options opts)
-    : Client(Client::InternalOnlyNoDecorations{},
-             Client::CreateDefaultInternalClient(
-                 internal::DefaultOptionsWithCredentials(std::move(opts)))) {}
+    : Client(InternalOnly{},
+             internal::DefaultOptionsWithCredentials(std::move(opts))) {}
 
-std::shared_ptr<internal::RawClient> Client::CreateDefaultInternalClient(
-    Options const& opts, std::shared_ptr<internal::RawClient> client) {
-  using ::google::cloud::internal::Contains;
-  auto const& tracing_components = opts.get<TracingComponentsOption>();
-  auto const enable_logging = Contains(tracing_components, "raw-client") ||
-                              Contains(tracing_components, "rpc");
-  if (enable_logging) {
-    client = std::make_shared<internal::LoggingClient>(std::move(client));
-  }
-  return std::make_shared<internal::RetryClient>(std::move(client), opts);
-}
+/// Apply all decorators to @p connection, based on @p opts.
+Client::Client(InternalOnly, Options const& opts,
+               std::shared_ptr<internal::StorageConnection> connection)
+    : Client(InternalOnlyNoDecorations{}, storage_internal::DecorateConnection(
+                                              opts, std::move(connection))) {}
 
-std::shared_ptr<internal::RawClient> Client::CreateDefaultInternalClient(
-    Options const& opts) {
-  return CreateDefaultInternalClient(opts, internal::CurlClient::Create(opts));
-}
+/// Create a connection from @p opts, applying all decorators if needed.
+Client::Client(InternalOnly, Options const& opts)
+    : Client(InternalOnlyNoDecorations{},
+             storage_internal::MakeStorageConnection(opts)) {}
 
 StatusOr<Client> Client::CreateDefaultClient() { return Client(Options{}); }
 
 ObjectReadStream Client::ReadObjectImpl(
     internal::ReadObjectRangeRequest const& request) {
-  auto source = raw_client_->ReadObject(request);
+  auto source = connection_->ReadObject(request);
   if (!source) {
     ObjectReadStream error_stream(
-        absl::make_unique<internal::ObjectReadStreambuf>(
+        std::make_unique<internal::ObjectReadStreambuf>(
             request, std::move(source).status()));
     error_stream.setstate(std::ios::badbit | std::ios::eofbit);
     return error_stream;
   }
   auto stream =
-      ObjectReadStream(absl::make_unique<internal::ObjectReadStreambuf>(
-          request, *std::move(source),
-          request.GetOption<ReadFromOffset>().value_or(0)));
+      ObjectReadStream(std::make_unique<internal::ObjectReadStreambuf>(
+          request, *std::move(source)));
   (void)stream.peek();
 #if !GOOGLE_CLOUD_CPP_HAVE_EXCEPTIONS
   // Without exceptions the streambuf cannot report errors, so we have to
@@ -86,24 +84,22 @@ ObjectReadStream Client::ReadObjectImpl(
 
 ObjectWriteStream Client::WriteObjectImpl(
     internal::ResumableUploadRequest const& request) {
-  auto session = raw_client_->CreateResumableSession(request);
-  if (!session) {
-    auto error = absl::make_unique<internal::ResumableUploadSessionError>(
-        std::move(session).status());
-
+  auto response = internal::CreateOrResume(*connection_, request);
+  if (!response) {
     ObjectWriteStream error_stream(
-        absl::make_unique<internal::ObjectWriteStreambuf>(
-            std::move(error), 0, internal::CreateNullHashFunction(),
-            internal::HashValues{}, internal::CreateNullHashValidator(),
-            AutoFinalizeConfig::kDisabled));
+        std::make_unique<internal::ObjectWriteStreambuf>(
+            std::move(response).status()));
     error_stream.setstate(std::ios::badbit | std::ios::eofbit);
     error_stream.Close();
     return error_stream;
   }
+  auto const& current = google::cloud::internal::CurrentOptions();
   auto const buffer_size = request.GetOption<UploadBufferSize>().value_or(
-      raw_client_->client_options().upload_buffer_size());
-  return ObjectWriteStream(absl::make_unique<internal::ObjectWriteStreambuf>(
-      *std::move(session), buffer_size, internal::CreateHashFunction(request),
+      current.get<UploadBufferSizeOption>());
+  return ObjectWriteStream(std::make_unique<internal::ObjectWriteStreambuf>(
+      connection_, request, std::move(response->upload_id),
+      response->committed_size, std::move(response->metadata), buffer_size,
+      internal::CreateHashFunction(request),
       internal::HashValues{
           request.GetOption<Crc32cChecksumValue>().value_or(""),
           request.GetOption<MD5HashValue>().value_or(""),
@@ -113,14 +109,14 @@ ObjectWriteStream Client::WriteObjectImpl(
           AutoFinalizeConfig::kEnabled)));
 }
 
-bool Client::UseSimpleUpload(std::string const& file_name,
-                             std::size_t& size) const {
+bool Client::UseSimpleUpload(std::string const& file_name, std::size_t& size) {
   auto status = google::cloud::internal::status(file_name);
   if (!is_regular(status)) {
     return false;
   }
   auto const fs = google::cloud::internal::file_size(file_name);
-  if (fs <= raw_client_->client_options().maximum_simple_upload_size()) {
+  auto const& current = google::cloud::internal::CurrentOptions();
+  if (fs <= current.get<MaximumSimpleUploadSizeOption>()) {
     size = static_cast<std::size_t>(fs);
     return true;
   }
@@ -152,6 +148,8 @@ StatusOr<ObjectMetadata> Client::UploadFileSimple(
 
   std::string payload(static_cast<std::size_t>(upload_size), char{});
   is.seekg(upload_offset, std::ios::beg);
+  // We need to use `&payload[0]` until C++17
+  // NOLINTNEXTLINE(readability-container-data-pointer)
   is.read(&payload[0], payload.size());
   if (static_cast<std::size_t>(is.gcount()) < payload.size()) {
     std::ostringstream os;
@@ -161,9 +159,9 @@ StatusOr<ObjectMetadata> Client::UploadFileSimple(
     return Status(StatusCode::kInternal, std::move(os).str());
   }
   is.close();
-  request.set_contents(std::move(payload));
+  request.set_payload(payload);
 
-  return raw_client_->InsertObjectMedia(request);
+  return connection_->InsertObjectMedia(request);
 }
 
 StatusOr<ObjectMetadata> Client::UploadFileResumable(
@@ -216,83 +214,82 @@ integrity checks using the DisableMD5Hash() and DisableCrc32cChecksum() options.
   return UploadStreamResumable(source, request);
 }
 
-// NOLINTNEXTLINE(readability-make-member-function-const)
 StatusOr<ObjectMetadata> Client::UploadStreamResumable(
-    std::istream& source, internal::ResumableUploadRequest const& request) {
-  StatusOr<std::unique_ptr<internal::ResumableUploadSession>> session_status =
-      raw_client_->CreateResumableSession(request);
-  if (!session_status) {
-    return std::move(session_status).status();
-  }
+    std::istream& source,
+    internal::ResumableUploadRequest const& request) const {
+  auto response = internal::CreateOrResume(*connection_, request);
+  if (!response) return std::move(response).status();
+
+  if (response->metadata.has_value()) return *std::move(response->metadata);
 
-  auto session = std::move(*session_status);
   // How many bytes of the local file are uploaded to the GCS server.
-  auto server_size = session->next_expected_byte();
+  auto upload_id = std::move(response->upload_id);
+  auto committed_size = response->committed_size;
   auto upload_limit = request.GetOption<UploadLimit>().value_or(
       (std::numeric_limits<std::uint64_t>::max)());
-  // If `server_size == upload_limit`, we will upload an empty string and
+  // If `committed_size == upload_limit`, we will upload an empty string and
   // finalize the upload.
-  if (server_size > upload_limit) {
+  if (committed_size > upload_limit) {
     return Status(StatusCode::kOutOfRange,
                   "UploadLimit (" + std::to_string(upload_limit) +
                       ") is not bigger than the uploaded size (" +
-                      std::to_string(server_size) + ") on GCS server");
+                      std::to_string(committed_size) + ") on GCS server");
   }
-  source.seekg(server_size, std::ios::cur);
+  source.seekg(committed_size, std::ios::cur);
 
   // GCS requires chunks to be a multiple of 256KiB.
+  auto const& current = google::cloud::internal::CurrentOptions();
   auto chunk_size = internal::UploadChunkRequest::RoundUpToQuantum(
-      raw_client_->client_options().upload_buffer_size());
+      current.get<UploadBufferSizeOption>());
 
-  StatusOr<internal::ResumableUploadResponse> upload_response(
-      internal::ResumableUploadResponse{});
   // We iterate while `source` is good, the upload size does not reach the
   // `UploadLimit` and the retry policy has not been exhausted.
   bool reach_upload_limit = false;
   internal::ConstBufferSequence buffers(1);
   std::vector<char> buffer(chunk_size);
-  while (!source.eof() && upload_response &&
-         !upload_response->payload.has_value() && !reach_upload_limit) {
+  std::shared_ptr<internal::HashFunction> hash_function =
+      internal::CreateHashFunction(request);
+  while (!source.eof() && !reach_upload_limit) {
     // Read a chunk of data from the source file.
-    if (upload_limit - server_size <= chunk_size) {
+    if (upload_limit - committed_size <= chunk_size) {
       // We don't want the `source_size` to exceed `upload_limit`.
-      chunk_size = static_cast<std::size_t>(upload_limit - server_size);
+      chunk_size = static_cast<std::size_t>(upload_limit - committed_size);
       reach_upload_limit = true;
     }
     source.read(buffer.data(), buffer.size());
     auto gcount = static_cast<std::size_t>(source.gcount());
-    bool final_chunk = (gcount < buffer.size()) || reach_upload_limit;
-    auto source_size = session->next_expected_byte() + gcount;
-    auto expected = source_size;
+    auto expected = committed_size + gcount;
     buffers[0] = internal::ConstBuffer{buffer.data(), gcount};
-    if (final_chunk) {
-      upload_response = session->UploadFinalChunk(buffers, source_size, {});
-    } else {
-      upload_response = session->UploadChunk(buffers);
-    }
-    if (!upload_response) {
-      return std::move(upload_response).status();
-    }
-    if (session->next_expected_byte() != expected) {
+    auto upload_request = [&] {
+      bool final_chunk = (gcount < buffer.size()) || reach_upload_limit;
+      if (!final_chunk) {
+        return internal::UploadChunkRequest(upload_id, committed_size, buffers,
+                                            hash_function);
+      }
+      return internal::UploadChunkRequest(upload_id, committed_size, buffers,
+                                          hash_function,
+                                          internal::HashValues{});
+    }();
+    request.ForEachOption(internal::CopyCommonOptions(upload_request));
+    auto upload = connection_->UploadChunk(upload_request);
+    if (!upload) return std::move(upload).status();
+    if (upload->payload.has_value()) return *std::move(upload->payload);
+    auto const actual_committed_size = upload->committed_size.value_or(0);
+    if (actual_committed_size != expected) {
       // Defensive programming: unless there is a bug, this should be dead code.
       return Status(
           StatusCode::kInternal,
-          "Unexpected last committed byte expected=" +
-              std::to_string(expected) +
-              " got=" + std::to_string(session->next_expected_byte()) +
+          "Mismatch in committed size expected=" + std::to_string(expected) +
+              " got=" + std::to_string(actual_committed_size) +
               ". This is a bug, please report it at "
               "https://github.com/googleapis/google-cloud-cpp/issues/new");
     }
 
-    // We only update `server_size` when uploading is successful.
-    server_size = expected;
+    // We only update `committed_size` when uploading is successful.
+    committed_size = expected;
   }
-
-  if (!upload_response) {
-    return std::move(upload_response).status();
-  }
-
-  return *std::move(upload_response->payload);
+  return Status(StatusCode::kInternal,
+                "Upload did not complete but source is exhausted");
 }
 
 Status Client::DownloadFileImpl(internal::ReadObjectRangeRequest const& request,
@@ -306,7 +303,7 @@ Status Client::DownloadFileImpl(internal::ReadObjectRangeRequest const& request,
   };
 
   auto stream = ReadObjectImpl(request);
-  if (!stream.status().ok()) {
+  if (stream.bad()) {
     return report_error(__func__, "cannot open download source object",
                         stream.status());
   }
@@ -319,37 +316,36 @@ Status Client::DownloadFileImpl(internal::ReadObjectRangeRequest const& request,
         Status(StatusCode::kInvalidArgument, "ofstream::open()"));
   }
 
-  std::string buffer;
-  buffer.resize(raw_client_->client_options().download_buffer_size(), '\0');
+  auto const& current = google::cloud::internal::CurrentOptions();
+  auto const size = current.get<DownloadBufferSizeOption>();
+  std::unique_ptr<char[]> buffer(new char[size]);
   do {
-    stream.read(&buffer[0], buffer.size());
-    os.write(buffer.data(), stream.gcount());
+    stream.read(buffer.get(), size);
+    os.write(buffer.get(), stream.gcount());
   } while (os.good() && stream.good());
   os.close();
   if (!os.good()) {
     return report_error(__func__, "cannot close download destination file",
                         Status(StatusCode::kUnknown, "ofstream::close()"));
   }
-  if (!stream.status().ok()) {
+  if (stream.bad()) {
     return report_error(__func__, "error reading download source object",
                         stream.status());
   }
   return Status();
 }
 
-// NOLINTNEXTLINE(readability-make-member-function-const)
-std::string Client::SigningEmail(SigningAccount const& signing_account) {
+std::string Client::SigningEmail(SigningAccount const& signing_account) const {
   if (signing_account.has_value()) {
     return signing_account.value();
   }
-  return raw_client_->client_options().credentials()->AccountEmail();
+  return connection_->client_options().credentials()->AccountEmail();
 }
 
 StatusOr<Client::SignBlobResponseRaw> Client::SignBlobImpl(
     SigningAccount const& signing_account, std::string const& string_to_sign) {
-  auto credentials = raw_client_->client_options().credentials();
+  auto credentials = connection_->client_options().credentials();
 
-  std::string signing_account_email = SigningEmail(signing_account);
   // First try to sign locally.
   auto signed_blob = credentials->SignBlob(signing_account, string_to_sign);
   if (signed_blob) {
@@ -359,9 +355,20 @@ StatusOr<Client::SignBlobResponseRaw> Client::SignBlobImpl(
   // If signing locally fails that may be because the credentials do not
   // support signing, or because the signing account is different than the
   // credentials account. In either case, try to sign using the API.
+  // In this case, however, we want to validate the signing account, because
+  // otherwise the errors are almost impossible to troubleshoot.
+  auto signing_email = SigningEmail(signing_account);
+  if (signing_email.empty()) {
+    return google::cloud::internal::InvalidArgumentError(
+        "signing account cannot be empty."
+        " The client library was unable to fetch a valid signing email from"
+        " the configured credentials, and the application did not provide"
+        " a value in the `google::cloud::storage::SigningAccount` option.",
+        GCP_ERROR_INFO());
+  }
   internal::SignBlobRequest sign_request(
-      signing_account_email, internal::Base64Encode(string_to_sign), {});
-  auto response = raw_client_->SignBlob(sign_request);
+      std::move(signing_email), internal::Base64Encode(string_to_sign), {});
+  auto response = connection_->SignBlob(sign_request);
   if (!response) return response.status();
   auto decoded = internal::Base64Decode(response->signed_blob);
   if (!decoded) return std::move(decoded).status();
@@ -372,11 +379,9 @@ StatusOr<std::string> Client::SignUrlV2(
     internal::V2SignUrlRequest const& request) {
   SigningAccount const& signing_account = request.signing_account();
   auto signed_blob = SignBlobImpl(signing_account, request.StringToSign());
-  if (!signed_blob) {
-    return signed_blob.status();
-  }
+  if (!signed_blob) return std::move(signed_blob).status();
 
-  internal::CurlHandle curl;
+  CurlHandle curl;
   auto encoded = internal::Base64Encode(signed_blob->signed_blob);
   std::string signature = curl.MakeEscapedString(encoded).get();
 
@@ -407,8 +412,9 @@ StatusOr<std::string> Client::SignUrlV4(internal::V4SignUrlRequest request) {
     return signed_blob.status();
   }
 
-  std::string signature = internal::HexEncode(signed_blob->signed_blob);
-  internal::CurlHandle curl;
+  std::string signature =
+      google::cloud::internal::HexEncode(signed_blob->signed_blob);
+  CurlHandle curl;
   std::ostringstream os;
   os << request.HostnameWithBucket();
   for (auto& part : request.ObjectNameParts()) {
@@ -451,7 +457,8 @@ StatusOr<PolicyDocumentV4Result> Client::SignPolicyDocumentV4(
   if (!signed_blob) {
     return signed_blob.status();
   }
-  std::string signature = internal::HexEncode(signed_blob->signed_blob);
+  std::string signature =
+      google::cloud::internal::HexEncode(signed_blob->signed_blob);
   auto required_fields = request.RequiredFormFields();
   required_fields["x-goog-signature"] = signature;
   required_fields["policy"] = base64_policy;
@@ -473,9 +480,16 @@ std::string CreateRandomPrefixName(std::string const& prefix) {
 
 namespace internal {
 
+Client ClientImplDetails::CreateWithDecorations(
+    Options const& opts, std::shared_ptr<StorageConnection> connection) {
+  return Client(
+      Client::InternalOnlyNoDecorations{},
+      storage_internal::DecorateConnection(opts, std::move(connection)));
+}
+
 ScopedDeleter::ScopedDeleter(
-    std::function<Status(std::string, std::int64_t)> delete_fun)
-    : enabled_(true), delete_fun_(std::move(delete_fun)) {}
+    std::function<Status(std::string, std::int64_t)> df)
+    : delete_fun_(std::move(df)) {}
 
 ScopedDeleter::~ScopedDeleter() {
   if (enabled_) {
