@@ -31,9 +31,7 @@ void ListObjects(google::cloud::storage::Client client,
   namespace gcs = ::google::cloud::storage;
   [](gcs::Client client, std::string const& bucket_name) {
     for (auto&& object_metadata : client.ListObjects(bucket_name)) {
-      if (!object_metadata) {
-        throw std::runtime_error(object_metadata.status().message());
-      }
+      if (!object_metadata) throw std::move(object_metadata).status();
 
       std::cout << "bucket_name=" << object_metadata->bucket()
                 << ", object_name=" << object_metadata->name() << "\n";
@@ -51,9 +49,7 @@ void ListObjectsWithPrefix(google::cloud::storage::Client client,
      std::string const& bucket_prefix) {
     for (auto&& object_metadata :
          client.ListObjects(bucket_name, gcs::Prefix(bucket_prefix))) {
-      if (!object_metadata) {
-        throw std::runtime_error(object_metadata.status().message());
-      }
+      if (!object_metadata) throw std::move(object_metadata).status();
 
       std::cout << "bucket_name=" << object_metadata->bucket()
                 << ", object_name=" << object_metadata->name() << "\n";
@@ -70,9 +66,7 @@ void ListVersionedObjects(google::cloud::storage::Client client,
   [](gcs::Client client, std::string const& bucket_name) {
     for (auto&& object_metadata :
          client.ListObjects(bucket_name, gcs::Versions{true})) {
-      if (!object_metadata) {
-        throw std::runtime_error(object_metadata.status().message());
-      }
+      if (!object_metadata) throw std::move(object_metadata).status();
 
       std::cout << "bucket_name=" << object_metadata->bucket()
                 << ", object_name=" << object_metadata->name()
@@ -91,7 +85,7 @@ void ListObjectsAndPrefixes(google::cloud::storage::Client client,
      std::string const& bucket_prefix) {
     for (auto&& item : client.ListObjectsAndPrefixes(
              bucket_name, gcs::Prefix(bucket_prefix), gcs::Delimiter("/"))) {
-      if (!item) throw std::runtime_error(item.status().message());
+      if (!item) throw std::move(item).status();
       auto result = *std::move(item);
       if (absl::holds_alternative<gcs::ObjectMetadata>(result)) {
         std::cout << "object_name="
@@ -114,10 +108,7 @@ void InsertObject(google::cloud::storage::Client client,
      std::string const& object_name, std::string const& contents) {
     StatusOr<gcs::ObjectMetadata> object_metadata =
         client.InsertObject(bucket_name, object_name, std::move(contents));
-
-    if (!object_metadata) {
-      throw std::runtime_error(object_metadata.status().message());
-    }
+    if (!object_metadata) throw std::move(object_metadata).status();
 
     std::cout << "The object " << object_metadata->name()
               << " was created in bucket " << object_metadata->bucket()
@@ -143,10 +134,7 @@ void InsertObjectStrictIdempotency(google::cloud::storage::Client,
     StatusOr<gcs::ObjectMetadata> object_metadata =
         client.InsertObject(bucket_name, object_name, std::move(contents),
                             gcs::IfGenerationMatch(0));
-
-    if (!object_metadata) {
-      throw std::runtime_error(object_metadata.status().message());
-    }
+    if (!object_metadata) throw std::move(object_metadata).status();
 
     std::cout << "The object " << object_metadata->name()
               << " was created in bucket " << object_metadata->bucket()
@@ -173,10 +161,7 @@ void InsertObjectModifiedRetry(google::cloud::storage::Client,
     StatusOr<gcs::ObjectMetadata> object_metadata =
         client.InsertObject(bucket_name, object_name, std::move(contents),
                             gcs::IfGenerationMatch(0));
-
-    if (!object_metadata) {
-      throw std::runtime_error(object_metadata.status().message());
-    }
+    if (!object_metadata) throw std::move(object_metadata).status();
 
     std::cout << "The object " << object_metadata->name()
               << " was created in bucket " << object_metadata->bucket()
@@ -201,10 +186,7 @@ void InsertObjectMultipart(google::cloud::storage::Client client,
         bucket_name, object_name, std::move(contents),
         gcs::WithObjectMetadata(
             gcs::ObjectMetadata().set_content_type(content_type)));
-
-    if (!object_metadata) {
-      throw std::runtime_error(object_metadata.status().message());
-    }
+    if (!object_metadata) throw std::move(object_metadata).status();
 
     std::cout << "The object " << object_metadata->name()
               << " was created in bucket " << object_metadata->bucket()
@@ -228,10 +210,7 @@ void CopyObject(google::cloud::storage::Client client,
     StatusOr<gcs::ObjectMetadata> new_copy_meta =
         client.CopyObject(source_bucket_name, source_object_name,
                           destination_bucket_name, destination_object_name);
-
-    if (!new_copy_meta) {
-      throw std::runtime_error(new_copy_meta.status().message());
-    }
+    if (!new_copy_meta) throw std::move(new_copy_meta).status();
 
     std::cout << "Successfully copied " << source_object_name << " in bucket "
               << source_bucket_name << " to bucket " << new_copy_meta->bucket()
@@ -252,10 +231,7 @@ void GetObjectMetadata(google::cloud::storage::Client client,
      std::string const& object_name) {
     StatusOr<gcs::ObjectMetadata> object_metadata =
         client.GetObjectMetadata(bucket_name, object_name);
-
-    if (!object_metadata) {
-      throw std::runtime_error(object_metadata.status().message());
-    }
+    if (!object_metadata) throw std::move(object_metadata).status();
 
     std::cout << "The metadata for object " << object_metadata->name()
               << " in bucket " << object_metadata->bucket() << " is "
@@ -279,6 +255,7 @@ void ReadObject(google::cloud::storage::Client client,
     while (std::getline(stream, line, '\n')) {
       ++count;
     }
+    if (stream.bad()) throw google::cloud::Status(stream.status());
 
     std::cout << "The object has " << count << " lines\n";
   }
@@ -302,6 +279,7 @@ void ReadObjectRange(google::cloud::storage::Client client,
       std::cout << line << "\n";
       ++count;
     }
+    if (stream.bad()) throw google::cloud::Status(stream.status());
 
     std::cout << "The requested range has " << count << " lines\n";
   }
@@ -319,6 +297,7 @@ void ReadObjectIntoMemory(google::cloud::storage::Client client,
     gcs::ObjectReadStream stream = client.ReadObject(bucket_name, object_name);
     std::string buffer{std::istream_iterator<char>(stream),
                        std::istream_iterator<char>()};
+    if (stream.bad()) throw google::cloud::Status(stream.status());
 
     std::cout << "The object has " << buffer.size() << " characters\n";
   }
@@ -326,6 +305,22 @@ void ReadObjectIntoMemory(google::cloud::storage::Client client,
   (std::move(client), argv.at(0), argv.at(1));
 }
 
+void ReadObjectGzip(google::cloud::storage::Client client,
+                    std::vector<std::string> const& argv) {
+  //! [read object gzip]
+  namespace gcs = ::google::cloud::storage;
+  [](gcs::Client client, std::string const& bucket_name,
+     std::string const& object_name) {
+    auto is =
+        client.ReadObject(bucket_name, object_name, gcs::AcceptEncodingGzip());
+    auto const contents = std::string{std::istream_iterator<char>(is), {}};
+    if (is.bad()) throw google::cloud::Status(is.status());
+    std::cout << "The object has " << contents.size() << " characters\n";
+  }
+  //! [read object gzip]
+  (std::move(client), argv.at(0), argv.at(1));
+}
+
 void DeleteObject(google::cloud::storage::Client client,
                   std::vector<std::string> const& argv) {
   //! [delete object] [START storage_delete_file]
@@ -363,7 +358,7 @@ void WriteObject(google::cloud::storage::Client client,
     stream.Close();
 
     StatusOr<gcs::ObjectMetadata> metadata = std::move(stream).metadata();
-    if (!metadata) throw std::runtime_error(metadata.status().message());
+    if (!metadata) throw std::move(metadata).status();
     std::cout << "Successfully wrote to object " << metadata->name()
               << " its size is: " << metadata->size()
               << "\nFull metadata: " << *metadata << "\n";
@@ -380,16 +375,25 @@ void WriteObjectFromMemory(google::cloud::storage::Client client,
   [](gcs::Client client, std::string const& bucket_name,
      std::string const& object_name) {
     std::string const text = "Lorem ipsum dolor sit amet";
+    // For small uploads where the data is contiguous in memory use
+    // `InsertObject()`. For more specific size recommendations see
+    //     https://cloud.google.com/storage/docs/uploads-downloads#size
+    auto metadata = client.InsertObject(bucket_name, object_name, text);
+    if (!metadata) throw std::move(metadata).status();
+    std::cout << "Successfully wrote to object " << metadata->name()
+              << " its size is: " << metadata->size() << "\n";
+
+    // For larger uploads, or uploads where the data is not contiguous in
+    // memory, use `WriteObject()`. Consider using `std::ostream::write()` for
+    // best performance.
     std::vector<std::string> v(100, text);
     gcs::ObjectWriteStream stream =
         client.WriteObject(bucket_name, object_name);
-
     std::copy(v.begin(), v.end(), std::ostream_iterator<std::string>(stream));
-
     stream.Close();
 
-    StatusOr<gcs::ObjectMetadata> metadata = std::move(stream).metadata();
-    if (!metadata) throw std::runtime_error(metadata.status().message());
+    metadata = std::move(stream).metadata();
+    if (!metadata) throw std::move(metadata).status();
     std::cout << "Successfully wrote to object " << metadata->name()
               << " its size is: " << metadata->size()
               << "\nFull metadata: " << *metadata << "\n";
@@ -408,10 +412,7 @@ void UpdateObjectMetadata(google::cloud::storage::Client client,
      std::string const& value) {
     StatusOr<gcs::ObjectMetadata> object_metadata =
         client.GetObjectMetadata(bucket_name, object_name);
-
-    if (!object_metadata) {
-      throw std::runtime_error(object_metadata.status().message());
-    }
+    if (!object_metadata) throw std::move(object_metadata).status();
 
     gcs::ObjectMetadata desired = *object_metadata;
     desired.mutable_metadata().emplace(key, value);
@@ -420,7 +421,7 @@ void UpdateObjectMetadata(google::cloud::storage::Client client,
         client.UpdateObject(bucket_name, object_name, desired,
                             gcs::Generation(object_metadata->generation()));
 
-    if (!updated) throw std::runtime_error(updated.status().message());
+    if (!updated) throw std::move(updated).status();
     std::cout << "Object updated. The full metadata after the update is: "
               << *updated << "\n";
   }
@@ -438,14 +439,14 @@ void PatchObjectDeleteMetadata(google::cloud::storage::Client client,
     StatusOr<gcs::ObjectMetadata> original =
         client.GetObjectMetadata(bucket_name, object_name);
 
-    if (!original) throw std::runtime_error(original.status().message());
+    if (!original) throw std::move(original).status();
     gcs::ObjectMetadata desired = *original;
     desired.mutable_metadata().erase(key);
 
     StatusOr<gcs::ObjectMetadata> updated =
         client.PatchObject(bucket_name, object_name, *original, desired);
 
-    if (!updated) throw std::runtime_error(updated.status().message());
+    if (!updated) throw std::move(updated).status();
     std::cout << "Object updated. The full metadata after the update is: "
               << *updated << "\n";
   }
@@ -464,7 +465,7 @@ void PatchObjectContentType(google::cloud::storage::Client client,
         bucket_name, object_name,
         gcs::ObjectMetadataPatchBuilder().SetContentType(content_type));
 
-    if (!updated) throw std::runtime_error(updated.status().message());
+    if (!updated) throw std::move(updated).status();
     std::cout << "Object updated. The full metadata after the update is: "
               << *updated << "\n";
   }
@@ -479,6 +480,7 @@ void ComposeObject(google::cloud::storage::Client client,
   auto destination_object_name = *it++;
   std::vector<google::cloud::storage::ComposeSourceObject> compose_objects;
   do {
+    // NOLINTNEXTLINE(modernize-use-emplace) - brace initialization
     compose_objects.push_back({*it++, {}, {}});
   } while (it != argv.cend());
 
@@ -490,10 +492,7 @@ void ComposeObject(google::cloud::storage::Client client,
      std::vector<gcs::ComposeSourceObject> const& compose_objects) {
     StatusOr<gcs::ObjectMetadata> composed_object = client.ComposeObject(
         bucket_name, compose_objects, destination_object_name);
-
-    if (!composed_object) {
-      throw std::runtime_error(composed_object.status().message());
-    }
+    if (!composed_object) throw std::move(composed_object).status();
 
     std::cout << "Composed new object " << composed_object->name()
               << " in bucket " << composed_object->bucket()
@@ -511,6 +510,7 @@ void ComposeObjectFromMany(google::cloud::storage::Client client,
   auto destination_object_name = *it++;
   std::vector<google::cloud::storage::ComposeSourceObject> compose_objects;
   do {
+    // NOLINTNEXTLINE(modernize-use-emplace) - brace initialization
     compose_objects.push_back({*it++, {}, {}});
   } while (it != argv.cend());
 
@@ -524,13 +524,10 @@ void ComposeObjectFromMany(google::cloud::storage::Client client,
     StatusOr<gcs::ObjectMetadata> composed_object =
         ComposeMany(client, bucket_name, compose_objects, prefix,
                     destination_object_name, false);
-
-    if (!composed_object) {
-      // If this is an effect of some transient unavailability, stray temporary
-      // might be left over. You can use `DeleteByPrefix()` with `prefix` as
-      // argument to delete them.
-      throw std::runtime_error(composed_object.status().message());
-    }
+    // If this is an effect of some transient unavailability, stray temporary
+    // might be left over. You can use `DeleteByPrefix()` with `prefix` as
+    // argument to delete them.
+    if (!composed_object) throw std::move(composed_object).status();
 
     std::cout << "Composed new object " << composed_object->name()
               << " in bucket " << composed_object->bucket()
@@ -554,10 +551,7 @@ void ChangeObjectStorageClass(google::cloud::storage::Client client,
             bucket_name, object_name, bucket_name, object_name,
             gcs::WithObjectMetadata(
                 gcs::ObjectMetadata().set_storage_class(storage_class)));
-
-    if (!object_metadata) {
-      throw std::runtime_error(object_metadata.status().message());
-    }
+    if (!object_metadata) throw std::move(object_metadata).status();
 
     std::cout << "Changed storage class of object " << object_metadata->name()
               << " in bucket " << object_metadata->bucket() << " to "
@@ -576,13 +570,13 @@ void ChangeObjectCustomTime(google::cloud::storage::Client client,
   [](gcs::Client client, std::string const& bucket_name,
      std::string const& object_name) {
     auto original = client.GetObjectMetadata(bucket_name, object_name);
-    if (!original) throw std::runtime_error(original.status().message());
+    if (!original) throw std::move(original).status();
 
     auto const tp = std::chrono::system_clock::now() - std::chrono::hours(48);
     auto updated =
         client.PatchObject(bucket_name, object_name,
                            gcs::ObjectMetadataPatchBuilder{}.SetCustomTime(tp));
-    if (!updated) throw std::runtime_error(updated.status().message());
+    if (!updated) throw std::move(updated).status();
 
     std::cout << "The custom time for object " << updated->name()
               << " in bucket " << updated->bucket() << " was successfully set. "
@@ -600,9 +594,9 @@ google::cloud::storage::Client StorageRetries(std::vector<std::string> const&) {
   // Retries only idempotent operations.
   options.set<gcs::IdempotencyPolicyOption>(
       gcs::StrictIdempotencyPolicy().clone());
-  // On error, it backs off for 1 second, then 3 seconds, then 9 seconds, etc.
-  // The backoff time never grows larger than 1 minute. The strategy introduces
-  // jitter around the backoff delay.
+  // On error, it backs off for a random delay between [1, 3] seconds, then [3,
+  // 9] seconds, then [9, 27] seconds, etc. The backoff time never grows larger
+  // than 1 minute.
   options.set<gcs::BackoffPolicyOption>(
       gcs::ExponentialBackoffPolicy(
           /*initial_delay=*/std::chrono::seconds(1),
@@ -632,7 +626,8 @@ void RunAll(std::vector<std::string> const& argv) {
   std::cout << "\nCreating bucket to run the example (" << bucket_name << ")"
             << std::endl;
   (void)client
-      .CreateBucketForProject(bucket_name, project_id, gcs::BucketMetadata{})
+      .CreateBucketForProject(bucket_name, project_id, gcs::BucketMetadata{},
+                              examples::CreateBucketOptions())
       .value();
   // In GCS a single project cannot create or delete buckets more often than
   // once every two seconds. We will pause until that time before deleting the
@@ -691,6 +686,9 @@ void RunAll(std::vector<std::string> const& argv) {
   std::cout << "\nRunning ReadObjectRange() example" << std::endl;
   ReadObjectRange(client, {bucket_name, object_name, "1000", "2000"});
 
+  std::cout << "\nRunning ReadObjectGzip() example" << std::endl;
+  ReadObjectGzip(client, {bucket_name, object_name});
+
   std::cout << "\nRunning UpdateObjectMetadata() example" << std::endl;
   UpdateObjectMetadata(client,
                        {bucket_name, object_name, "test-label", "test-value"});
@@ -793,6 +791,7 @@ int main(int argc, char* argv[]) {
       make_entry("read-object", {"<object-name>"}, ReadObject),
       make_entry("read-object-range", {"<object-name>", "<start>", "<end>"},
                  ReadObjectRange),
+      make_entry("read-object-gzip", {"<object-name>"}, ReadObjectGzip),
       make_entry("read-object-into-memory", {"<object-name>"},
                  ReadObjectIntoMemory),
       make_entry("delete-object", {"<object-name>"}, DeleteObject),
