@@ -26,8 +26,7 @@ StatusOr<std::string> MakeJWTAssertionNoThrow(std::string const& header,
                                               std::string const& pem_contents) {
   auto const body =
       UrlsafeBase64Encode(header) + '.' + UrlsafeBase64Encode(payload);
-  auto pem_signature = internal::SignStringWithPem(
-      body, pem_contents, storage::oauth2::JwtSigningAlgorithms::RS256);
+  auto pem_signature = internal::SignStringWithPem(body, pem_contents);
   if (!pem_signature) return std::move(pem_signature).status();
   return body + '.' + UrlsafeBase64Encode(*pem_signature);
 }
