@@ -16,8 +16,10 @@
 #include "google/cloud/storage/internal/compute_engine_util.h"
 #include "google/cloud/storage/oauth2/credential_constants.h"
 #include "google/cloud/storage/testing/mock_http_request.h"
-#include "google/cloud/internal/setenv.h"
 #include "google/cloud/testing_util/mock_fake_clock.h"
+#include "google/cloud/testing_util/mock_http_payload.h"
+#include "google/cloud/testing_util/mock_rest_client.h"
+#include "google/cloud/testing_util/mock_rest_response.h"
 #include "google/cloud/testing_util/status_matchers.h"
 #include <gmock/gmock.h>
 #include <nlohmann/json.hpp>
@@ -29,6 +31,23 @@ namespace cloud {
 namespace storage {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace oauth2 {
+
+// Define a helper to test the specialization.
+struct ComputeEngineCredentialsTester {
+  static StatusOr<std::string> Header(
+      ComputeEngineCredentials<>& tested,
+      std::chrono::system_clock::time_point tp) {
+    return tested.AuthorizationHeaderForTesting(tp);
+  }
+
+  static ComputeEngineCredentials<> MakeComputeEngineCredentials(
+      std::string service_account_email,
+      oauth2_internal::HttpClientFactory factory) {
+    return ComputeEngineCredentials<>(std::move(service_account_email),
+                                      std::move(factory));
+  }
+};
+
 namespace {
 
 using ::google::cloud::storage::internal::GceMetadataHostname;
@@ -37,9 +56,15 @@ using ::google::cloud::storage::testing::MockHttpRequest;
 using ::google::cloud::storage::testing::MockHttpRequestBuilder;
 using ::google::cloud::testing_util::FakeClock;
 using ::google::cloud::testing_util::IsOk;
+using ::google::cloud::testing_util::IsOkAndHolds;
+using ::google::cloud::testing_util::MakeMockHttpPayloadSuccess;
+using ::google::cloud::testing_util::MockRestClient;
+using ::google::cloud::testing_util::MockRestResponse;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::_;
+using ::testing::ByMove;
 using ::testing::HasSubstr;
+using ::testing::IsEmpty;
 using ::testing::Not;
 using ::testing::Return;
 using ::testing::StrEq;
@@ -177,27 +202,41 @@ TEST_F(ComputeEngineCredentialsTest, ParseComputeEngineRefreshResponse) {
 /// @test Parsing a metadata server response with missing fields results in
 /// failure.
 TEST_F(ComputeEngineCredentialsTest, ParseMetadataServerResponseInvalid) {
-  std::string email = "foo@bar.baz";
-  std::string svc_acct_info_resp = R"""({})""";
-  std::string svc_acct_info_resp2 = R"""({
-      "scopes": ["scope1","scope2"]
-  })""";
+  std::string svc_acct_info_resp1 = R"""({})""";
+  std::string svc_acct_info_resp2 = R"""({"scopes": ["scope1","scope2"]})""";
+  std::string svc_acct_info_resp3 = R"""({"email": "foo@bar"})""";
+  std::string svc_acct_info_resp4 =
+      R"""({"email": "foo@bar", "scopes": "scope1\nscope2\n"})""";
 
   auto status =
-      ParseMetadataServerResponse(HttpResponse{200, svc_acct_info_resp, {}});
-  EXPECT_THAT(status,
-              StatusIs(Not(StatusCode::kOk),
-                       HasSubstr("Could not find all required fields")));
+      ParseMetadataServerResponse(HttpResponse{200, svc_acct_info_resp1, {}});
+  EXPECT_THAT(status, IsOk());
+  EXPECT_THAT(status->email, IsEmpty());
+  EXPECT_THAT(status->scopes, IsEmpty());
 
   status =
       ParseMetadataServerResponse(HttpResponse{400, svc_acct_info_resp2, {}});
-  EXPECT_THAT(status,
-              StatusIs(Not(StatusCode::kOk),
-                       HasSubstr("Could not find all required fields")));
+  EXPECT_THAT(status, IsOk());
+  EXPECT_THAT(status->email, IsEmpty());
+  EXPECT_THAT(status->scopes, UnorderedElementsAre("scope1", "scope2"));
 
-  EXPECT_THAT(ParseMetadataServerResponse(HttpResponse{
-                  400, R"js("valid-json-but-not-an-object")js", {}}),
-              StatusIs(Not(StatusCode::kOk)));
+  status =
+      ParseMetadataServerResponse(HttpResponse{400, svc_acct_info_resp3, {}});
+  EXPECT_THAT(status, IsOk());
+  EXPECT_THAT(status->email, "foo@bar");
+  EXPECT_THAT(status->scopes, IsEmpty());
+
+  status =
+      ParseMetadataServerResponse(HttpResponse{400, svc_acct_info_resp4, {}});
+  EXPECT_THAT(status, IsOk());
+  EXPECT_THAT(status->email, "foo@bar");
+  EXPECT_THAT(status->scopes, UnorderedElementsAre("scope1", "scope2"));
+
+  status = ParseMetadataServerResponse(
+      HttpResponse{400, R"js("valid-json-but-not-an-object")js", {}});
+  EXPECT_THAT(status, IsOk());
+  EXPECT_THAT(status->email, IsEmpty());
+  EXPECT_THAT(status->scopes, IsEmpty());
 }
 
 /// @test Parsing a metadata server response yields a ServiceAccountMetadata.
@@ -220,26 +259,18 @@ TEST_F(ComputeEngineCredentialsTest, ParseMetadataServerResponse) {
 /// @test Mock a failed refresh response during RetrieveServiceAccountInfo.
 TEST_F(ComputeEngineCredentialsTest, FailedRetrieveServiceAccountInfo) {
   std::string alias = "default";
-  std::string email = "foo@bar.baz";
   std::string hostname = GceMetadataHostname();
 
-  // Missing fields.
-  std::string svc_acct_info_resp = R"""({
-      "scopes": ["scope1","scope2"]
-  })""";
-
   auto first_mock_req_impl = std::make_shared<MockHttpRequest::Impl>();
   EXPECT_CALL(*first_mock_req_impl, MakeRequest)
       // Fail the first call to DoMetadataServerGetRequest immediately.
       .WillOnce(Return(Status(StatusCode::kAborted, "Fake Curl error")))
       // Likewise, except with a >= 300 status code.
-      .WillOnce(Return(HttpResponse{400, "", {}}))
-      // Parse with an invalid metadata response.
-      .WillOnce(Return(HttpResponse{1, svc_acct_info_resp, {}}));
+      .WillOnce(Return(HttpResponse{400, "", {}}));
 
   auto mock_req_builder = MockHttpRequestBuilder::mock_;
   EXPECT_CALL(*mock_req_builder, BuildRequest())
-      .Times(3)
+      .Times(2)
       .WillRepeatedly([first_mock_req_impl] {
         MockHttpRequest mock_request;
         mock_request.mock = first_mock_req_impl;
@@ -247,17 +278,17 @@ TEST_F(ComputeEngineCredentialsTest, FailedRetrieveServiceAccountInfo) {
       });
 
   EXPECT_CALL(*mock_req_builder, AddHeader(StrEq("metadata-flavor: Google")))
-      .Times(3);
+      .Times(2);
   EXPECT_CALL(*mock_req_builder,
               AddQueryParameter(StrEq("recursive"), StrEq("true")))
-      .Times(3);
+      .Times(2);
   EXPECT_CALL(
       *mock_req_builder,
       Constructor(
           StrEq(std::string("http://") + hostname +
                 "/computeMetadata/v1/instance/service-accounts/" + alias + "/"),
           _, _))
-      .Times(3);
+      .Times(2);
 
   ComputeEngineCredentials<MockHttpRequestBuilder> credentials(alias);
   // Response 1
@@ -266,11 +297,6 @@ TEST_F(ComputeEngineCredentialsTest, FailedRetrieveServiceAccountInfo) {
   // Response 2
   status = credentials.AuthorizationHeader();
   EXPECT_THAT(status, Not(IsOk()));
-  // Response 3
-  status = credentials.AuthorizationHeader();
-  EXPECT_THAT(status,
-              StatusIs(Not(StatusCode::kOk),
-                       HasSubstr("Could not find all required fields")));
 }
 
 /// @test Mock a failed refresh response.
@@ -407,6 +433,68 @@ TEST_F(ComputeEngineCredentialsTest, AccountEmail) {
   EXPECT_EQ(credentials.service_account_email(), refreshed_email);
 }
 
+TEST_F(ComputeEngineCredentialsTest, Caching) {
+  // We need to mock the Compute Engine Metadata Service or this would be an
+  // integration test that only runs on GCE.
+  auto make_mock_client = [](std::string const& payload) {
+    auto response = std::make_unique<MockRestResponse>();
+    EXPECT_CALL(*response, StatusCode)
+        .WillRepeatedly(
+            Return(google::cloud::rest_internal::HttpStatusCode::kOk));
+    EXPECT_CALL(std::move(*response), ExtractPayload)
+        .WillOnce(Return(ByMove(MakeMockHttpPayloadSuccess(payload))));
+    auto mock = std::make_unique<MockRestClient>();
+    EXPECT_CALL(*mock, Get)
+        .WillOnce(Return(ByMove(std::unique_ptr<rest_internal::RestResponse>(
+            std::move(response)))));
+    return std::unique_ptr<rest_internal::RestClient>(std::move(mock));
+  };
+
+  // We expect a call to the metadata service to get the service account
+  // metadata. Then one call to get the token (which is cached), and finally a
+  // call the refresh the token.
+  auto constexpr kPayload0 =
+      R"""({"email": "test-only-email", "scopes": ["s1", "s2]})""";
+  auto constexpr kPayload1 = R"""({
+            "access_token": "test-token-1",
+            "expires_in": 3600,
+            "token_type": "tokentype"
+        })""";
+  auto constexpr kPayload2 = R"""({
+            "access_token": "test-token-2",
+            "expires_in": 3600,
+            "token_type": "tokentype"
+        })""";
+
+  using MockHttpClientFactory =
+      ::testing::MockFunction<std::unique_ptr<rest_internal::RestClient>(
+          Options const&)>;
+  MockHttpClientFactory mock_factory;
+  EXPECT_CALL(mock_factory, Call)
+      .WillOnce(Return(ByMove(make_mock_client(kPayload0))))
+      .WillOnce(Return(ByMove(make_mock_client(kPayload1))))
+      .WillOnce(Return(ByMove(make_mock_client(kPayload2))));
+
+  auto tested = ComputeEngineCredentialsTester::MakeComputeEngineCredentials(
+      "test-only", mock_factory.AsStdFunction());
+  auto const tp = std::chrono::system_clock::now();
+  auto initial = ComputeEngineCredentialsTester::Header(tested, tp);
+  ASSERT_STATUS_OK(initial);
+
+  auto cached = ComputeEngineCredentialsTester::Header(
+      tested, tp + std::chrono::seconds(30));
+  EXPECT_THAT(cached, IsOkAndHolds(*initial));
+
+  cached = ComputeEngineCredentialsTester::Header(
+      tested, tp + std::chrono::seconds(300));
+  EXPECT_THAT(cached, IsOkAndHolds(*initial));
+
+  auto uncached = ComputeEngineCredentialsTester::Header(
+      tested, tp + std::chrono::hours(2));
+  ASSERT_STATUS_OK(uncached);
+  EXPECT_NE(*initial, *uncached);
+}
+
 }  // namespace
 }  // namespace oauth2
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
