@@ -14,6 +14,8 @@
 
 #include "google/cloud/storage/oauth2/refreshing_credentials_wrapper.h"
 #include "google/cloud/storage/oauth2/credential_constants.h"
+#include "google/cloud/internal/oauth2_refreshing_credentials_wrapper.h"
+#include "absl/strings/str_split.h"
 
 namespace google {
 namespace cloud {
@@ -21,15 +23,25 @@ namespace storage {
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace oauth2 {
 
+RefreshingCredentialsWrapper::RefreshingCredentialsWrapper()
+    : impl_(std::make_unique<oauth2_internal::RefreshingCredentialsWrapper>()) {
+}
+
+std::pair<std::string, std::string> RefreshingCredentialsWrapper::SplitToken(
+    std::string const& token) {
+  std::pair<std::string, std::string> split_token = absl::StrSplit(token, ": ");
+  return split_token;
+}
+
 bool RefreshingCredentialsWrapper::IsExpired(
     std::chrono::system_clock::time_point now) const {
-  return now > (temporary_token_.expiration_time -
-                GoogleOAuthAccessTokenExpirationSlack());
+  return now >
+         (impl_->token_.expiration - GoogleOAuthAccessTokenExpirationSlack());
 }
 
 bool RefreshingCredentialsWrapper::IsValid(
     std::chrono::system_clock::time_point now) const {
-  return !temporary_token_.token.empty() && !IsExpired(now);
+  return !impl_->token_.token.empty() && !IsExpired(now);
 }
 
 }  // namespace oauth2
