@@ -19,11 +19,20 @@ namespace cloud {
 namespace storage {
 namespace testing {
 
+using ::testing::Return;
+using ::testing::ReturnRef;
+
 ClientUnitTest::ClientUnitTest()
     : mock_(std::make_shared<testing::MockClient>()),
       client_options_(ClientOptions(oauth2::CreateAnonymousCredentials())) {
   EXPECT_CALL(*mock_, client_options())
-      .WillRepeatedly(::testing::ReturnRef(client_options_));
+      .WillRepeatedly(ReturnRef(client_options_));
+  EXPECT_CALL(*mock_, options)
+      .WillRepeatedly(Return(storage::internal::DefaultOptionsWithCredentials(
+          Options{}
+              .set<UnifiedCredentialsOption>(MakeInsecureCredentials())
+              .set<AuthorityOption>("a-default")
+              .set<UserProjectOption>("u-p-default"))));
 }
 
 Client ClientUnitTest::ClientForMock() {
