@@ -31,7 +31,7 @@ class TempFile {
 
   ~TempFile();
 
-  std::string name() { return name_; }
+  std::string name() const { return name_; }
 
  private:
   std::string name_;
