@@ -18,10 +18,8 @@
 #include "google/cloud/log.h"
 #include "google/cloud/status.h"
 #include "google/cloud/status_or.h"
-#include "google/cloud/testing_util/contains_once.h"
 #include "google/cloud/testing_util/expect_exception.h"
 #include "google/cloud/testing_util/status_matchers.h"
-#include "absl/memory/memory.h"
 #include <gmock/gmock.h>
 #include <sys/types.h>
 #include <algorithm>
@@ -37,7 +35,6 @@ GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN
 namespace {
 
 using ::google::cloud::storage::testing::AclEntityNames;
-using ::google::cloud::testing_util::ContainsOnce;
 using ::google::cloud::testing_util::IsOk;
 using ::google::cloud::testing_util::StatusIs;
 using ::testing::AnyOf;
@@ -108,7 +105,7 @@ TEST_F(ObjectIntegrationTest, FullPatch) {
   // acl() - cannot compare for equality because many fields are updated with
   // unknown values (entity_id, etag, etc)
   EXPECT_THAT(AclEntityNames(patched->acl()),
-              ContainsOnce("allAuthenticatedUsers"));
+              Contains("allAuthenticatedUsers").Times(1));
 
   EXPECT_EQ(desired.cache_control(), patched->cache_control());
   EXPECT_EQ(desired.content_disposition(), patched->content_disposition());
@@ -212,6 +209,31 @@ TEST_F(ObjectIntegrationTest, ListObjectsStartEndOffset) {
   EXPECT_THAT(objects, UnorderedElementsAre(object_prefix + "/foo"));
 }
 
+TEST_F(ObjectIntegrationTest, ListObjectsMatchGlob) {
+  StatusOr<Client> client = MakeIntegrationTestClient();
+  ASSERT_STATUS_OK(client);
+
+  auto object_prefix = MakeRandomObjectName();
+  for (auto const* suffix :
+       {"/foo/1.txt", "/foo/bar/1.txt", "/foo/bar/2.cc", "/qux/quux/3.cc"}) {
+    auto meta =
+        client->InsertObject(bucket_name_, object_prefix + suffix, LoremIpsum(),
+                             storage::IfGenerationMatch(0));
+    ASSERT_STATUS_OK(meta);
+    ScheduleForDelete(*meta);
+  }
+
+  auto reader = client->ListObjects(bucket_name_, Prefix(object_prefix),
+                                    MatchGlob("**/*.cc"));
+  std::vector<std::string> objects;
+  for (auto& o : reader) {
+    ASSERT_STATUS_OK(o);
+    objects.push_back(o->name());
+  }
+  EXPECT_THAT(objects, UnorderedElementsAre(object_prefix + "/foo/bar/2.cc",
+                                            object_prefix + "/qux/quux/3.cc"));
+}
+
 TEST_F(ObjectIntegrationTest, ListObjectsIncludeTrailingDelimiter) {
   StatusOr<Client> client = MakeIntegrationTestClient();
   ASSERT_STATUS_OK(client);
@@ -453,63 +475,6 @@ TEST_F(ObjectIntegrationTest, StreamingWriteEmpty) {
   ASSERT_TRUE(actual.empty());
 }
 
-TEST_F(ObjectIntegrationTest, XmlStreamingWrite) {
-  // This test makes no sense when using the gRPC API.
-  if (UsingGrpc()) GTEST_SKIP();
-
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
-  auto object_name = MakeRandomObjectName();
-
-  // Create the object, but only if it does not exist already.
-  auto os = client->WriteObject(bucket_name_, object_name, IfGenerationMatch(0),
-                                Fields(""));
-  os.exceptions(std::ios_base::failbit);
-  // We will construct the expected response while streaming the data up.
-  std::ostringstream expected;
-
-  WriteRandomLines(os, expected);
-
-  os.Close();
-  ObjectMetadata meta = os.metadata().value();
-  ScheduleForDelete(meta);
-  // When asking for an empty list of fields we should not expect any values:
-  EXPECT_TRUE(meta.bucket().empty());
-  EXPECT_TRUE(meta.name().empty());
-
-  // Create a iostream to read the object back.
-  auto stream = client->ReadObject(bucket_name_, object_name);
-  std::string actual(std::istreambuf_iterator<char>{stream}, {});
-  ASSERT_FALSE(actual.empty());
-  auto expected_str = expected.str();
-  EXPECT_EQ(expected_str.size(), actual.size()) << " meta=" << meta;
-  EXPECT_EQ(expected_str, actual);
-}
-
-TEST_F(ObjectIntegrationTest, XmlReadWrite) {
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
-  auto object_name = MakeRandomObjectName();
-
-  std::string expected = LoremIpsum();
-
-  // Create the object, but only if it does not exist already.
-  StatusOr<ObjectMetadata> meta = client->InsertObject(
-      bucket_name_, object_name, expected, IfGenerationMatch(0), Fields(""));
-  ASSERT_STATUS_OK(meta);
-  ScheduleForDelete(*meta);
-
-  EXPECT_EQ(object_name, meta->name());
-  EXPECT_EQ(bucket_name_, meta->bucket());
-
-  // Create a iostream to read the object back.
-  auto stream = client->ReadObject(bucket_name_, object_name);
-  std::string actual(std::istreambuf_iterator<char>{stream}, {});
-  EXPECT_EQ(expected, actual);
-}
-
 TEST_F(ObjectIntegrationTest, AccessControlCRUD) {
   StatusOr<Client> client = MakeIntegrationTestClient();
   ASSERT_STATUS_OK(client);
@@ -539,7 +504,8 @@ TEST_F(ObjectIntegrationTest, AccessControlCRUD) {
   // Search using the entity name returned by the request, because we use
   // 'project-editors-<project_id>' this different than the original entity
   // name, the server "translates" the project id to a project number.
-  EXPECT_THAT(AclEntityNames(*current_acl), ContainsOnce(result->entity()));
+  EXPECT_THAT(AclEntityNames(*current_acl),
+              Contains(result->entity()).Times(1));
 
   auto get_result =
       client->GetObjectAcl(bucket_name_, object_name, entity_name);
@@ -606,6 +572,7 @@ TEST_F(ObjectIntegrationTest, GetObjectMetadataFailure) {
   EXPECT_THAT(meta, Not(IsOk())) << "value=" << meta.value();
 }
 
+#if GOOGLE_CLOUD_CPP_HAVE_EXCEPTIONS
 TEST_F(ObjectIntegrationTest, StreamingWriteFailure) {
   StatusOr<Client> client = MakeIntegrationTestClient();
   ASSERT_STATUS_OK(client);
@@ -629,18 +596,13 @@ TEST_F(ObjectIntegrationTest, StreamingWriteFailure) {
   os << "Expected failure data:\n" << LoremIpsum();
 
   // This operation should fail because the object already exists.
-  testing_util::ExpectException<std::ios::failure>(
-      [&] { os.Close(); },
-      [&](std::ios::failure const&) {
-        // The GCS server returns a different error code depending on the
-        // protocol (REST vs. gRPC) used
-        EXPECT_THAT(os.metadata().status().code(),
-                    AnyOf(Eq(StatusCode::kFailedPrecondition),
-                          Eq(StatusCode::kAborted)))
-            << " status=" << os.metadata().status();
-      },
-      "" /* the message generated by the C++ runtime is unknown */);
+  ASSERT_THROW(os.Close(), std::ios::failure);
+  // The GCS server returns a different error code depending on the
+  // protocol (REST vs. gRPC) used
+  EXPECT_THAT(os.metadata(), AnyOf(StatusIs(StatusCode::kFailedPrecondition),
+                                   StatusIs(StatusCode::kAborted)));
 }
+#endif  // GOOGLE_CLOUD_CPP_HAVE_EXCEPTIONS
 
 TEST_F(ObjectIntegrationTest, StreamingWriteFailureNoex) {
   StatusOr<Client> client = MakeIntegrationTestClient();
@@ -798,10 +760,8 @@ TEST_F(ObjectIntegrationTest, DeleteAccessControlFailure) {
 }
 
 TEST_F(ObjectIntegrationTest, DeleteResumableUpload) {
-  // TODO(#5673): Enable this.
-  if (!UsingEmulator()) GTEST_SKIP();
   StatusOr<Client> client = MakeIntegrationTestClient(
-      absl::make_unique<LimitedErrorCountRetryPolicy>(1));
+      std::make_unique<LimitedErrorCountRetryPolicy>(1));
   ASSERT_STATUS_OK(client);
 
   auto object_name = MakeRandomObjectName();
@@ -824,7 +784,6 @@ TEST_F(ObjectIntegrationTest, DeleteResumableUpload) {
 }
 
 TEST_F(ObjectIntegrationTest, InsertWithCustomTime) {
-  if (UsingGrpc()) GTEST_SKIP();  // TODO(#4893) - no support in GCS+gRPC
   StatusOr<Client> client = MakeIntegrationTestClient();
   ASSERT_STATUS_OK(client);
 
@@ -847,7 +806,6 @@ TEST_F(ObjectIntegrationTest, InsertWithCustomTime) {
 }
 
 TEST_F(ObjectIntegrationTest, WriteWithCustomTime) {
-  if (UsingGrpc()) GTEST_SKIP();  // TODO(#4893) - no support in GCS+gRPC
   StatusOr<Client> client = MakeIntegrationTestClient();
   ASSERT_STATUS_OK(client);
 
