@@ -119,41 +119,6 @@ TEST_F(ObjectMediaIntegrationTest, ReadRangeJSON) {
   EXPECT_EQ(large_text.substr(1 * kChunk, 1 * kChunk), actual);
 }
 
-/// @test Read a portion of a relatively large object using the XML API.
-TEST_F(ObjectMediaIntegrationTest, ReadRangeXml) {
-  // The emulator always requires multiple iterations to copy this object.
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
-  auto object_name = MakeRandomObjectName();
-
-  // This produces a 64 KiB text object. Normally applications should download
-  // much larger chunks from GCS, but it is really hard to figure out what is
-  // broken when the error messages are in the MiB ranges.
-  std::size_t constexpr kChunk = 16 * 1024L;
-  std::size_t constexpr kLines = 4 * kChunk / 128;
-  std::string large_text;
-  for (std::size_t i = 0; i != kLines; ++i) {
-    auto line = google::cloud::internal::Sample(generator_, 127,
-                                                "abcdefghijklmnopqrstuvwxyz"
-                                                "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-                                                "0123456789");
-    large_text += line + "\n";
-  }
-
-  StatusOr<ObjectMetadata> source_meta = client->InsertObject(
-      bucket_name_, object_name, large_text, IfGenerationMatch(0));
-  ASSERT_STATUS_OK(source_meta);
-  ScheduleForDelete(*source_meta);
-
-  // Create an iostream to read the object back.
-  auto stream = client->ReadObject(bucket_name_, object_name,
-                                   ReadRange(1 * kChunk, 2 * kChunk));
-  std::string actual(std::istreambuf_iterator<char>{stream}, {});
-  EXPECT_EQ(1 * kChunk, actual.size());
-  EXPECT_EQ(large_text.substr(1 * kChunk, 1 * kChunk), actual);
-}
-
 /// @test Read a portion of a relatively large object using the JSON API.
 TEST_F(ObjectMediaIntegrationTest, ReadFromOffsetJSON) {
   // The emulator always requires multiple iterations to copy this object.
@@ -190,41 +155,6 @@ TEST_F(ObjectMediaIntegrationTest, ReadFromOffsetJSON) {
   EXPECT_EQ(large_text.substr(2 * kChunk), actual);
 }
 
-/// @test Read a portion of a relatively large object using the XML API.
-TEST_F(ObjectMediaIntegrationTest, ReadFromOffsetXml) {
-  // The emulator always requires multiple iterations to copy this object.
-  StatusOr<Client> client = MakeIntegrationTestClient();
-  ASSERT_STATUS_OK(client);
-
-  auto object_name = MakeRandomObjectName();
-
-  // This produces a 64 KiB text object. Normally applications should download
-  // much larger chunks from GCS, but it is really hard to figure out what is
-  // broken when the error messages are in the MiB ranges.
-  std::size_t constexpr kChunk = 16 * 1024L;
-  std::size_t constexpr kLines = 4 * kChunk / 128;
-  std::string large_text;
-  for (std::size_t i = 0; i != kLines; ++i) {
-    auto line = google::cloud::internal::Sample(generator_, 127,
-                                                "abcdefghijklmnopqrstuvwxyz"
-                                                "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-                                                "0123456789");
-    large_text += line + "\n";
-  }
-
-  StatusOr<ObjectMetadata> source_meta = client->InsertObject(
-      bucket_name_, object_name, large_text, IfGenerationMatch(0));
-  ASSERT_STATUS_OK(source_meta);
-  ScheduleForDelete(*source_meta);
-
-  // Create an iostream to read the object back.
-  auto stream =
-      client->ReadObject(bucket_name_, object_name, ReadFromOffset(2 * kChunk));
-  std::string actual(std::istreambuf_iterator<char>{stream}, {});
-  EXPECT_EQ(2 * kChunk, actual.size());
-  EXPECT_EQ(large_text.substr(2 * kChunk), actual);
-}
-
 /// @test Read a relatively large object using chunks of different sizes.
 TEST_F(ObjectMediaIntegrationTest, ReadMixedChunks) {
   // The emulator always requires multiple iterations to copy this object.
@@ -362,6 +292,7 @@ TEST_F(ObjectMediaIntegrationTest, ReadFromSpill) {
   // for a regression of #3051.
   std::vector<char> buffer(contents.size() - kUnreadBytes);
   stream.read(buffer.data(), buffer.size());
+  std::string actual{buffer.begin(), buffer.end()};
   EXPECT_FALSE(stream.eof());
   EXPECT_FALSE(stream.fail());
   EXPECT_FALSE(stream.bad());
@@ -370,9 +301,11 @@ TEST_F(ObjectMediaIntegrationTest, ReadFromSpill) {
   // Read the remaining data.
   buffer.resize(contents.size());
   stream.read(buffer.data(), buffer.size());
+  actual.append(buffer.data(), kUnreadBytes);
   EXPECT_TRUE(stream.eof());
   EXPECT_TRUE(stream.fail());
   EXPECT_FALSE(stream.bad());
+  EXPECT_EQ(contents, actual);
   EXPECT_FALSE(stream.IsOpen());
 }
 
@@ -422,6 +355,35 @@ TEST_F(ObjectMediaIntegrationTest, ReadLastChunkReadLast) {
   EXPECT_EQ(large_text.substr(kObjectSize - 129 * kKiB), actual);
 }
 
+/// @test Read the last chunk of an object by setting ReadLast option.
+TEST_F(ObjectMediaIntegrationTest, ReadLastTellg) {
+  StatusOr<Client> client = MakeIntegrationTestClient();
+  ASSERT_STATUS_OK(client);
+
+  auto object_name = MakeRandomObjectName();
+
+  // This produces an object larger than 3MiB, but with a size that is not a
+  // multiple of 128KiB.
+  auto constexpr kKiB = 1024L;
+  auto constexpr kObjectSize = 256 * kKiB;
+  auto const expected = MakeRandomData(kObjectSize);
+  auto insert = client->InsertObject(bucket_name_, object_name, expected,
+                                     IfGenerationMatch(0));
+  ASSERT_STATUS_OK(insert);
+  ScheduleForDelete(*insert);
+
+  // Create an iostream to read the last 129KiB of the object, but simulate an
+  // application that does not know how large that last chunk is.
+  auto is = client->ReadObject(bucket_name_, object_name, ReadLast(129 * kKiB));
+  EXPECT_EQ(is.tellg(), kObjectSize - 129 * kKiB);
+  std::vector<char> buffer(1024 * kKiB);
+  is.read(buffer.data(), 1000);
+  EXPECT_EQ(is.gcount(), 1000);
+  EXPECT_EQ(is.tellg(), kObjectSize - 129 * kKiB + 1000);
+  auto const actual = std::string(buffer.data(), 1000);
+  EXPECT_EQ(actual, expected.substr(kObjectSize - 129 * kKiB, 1000));
+}
+
 /// @test Read an object by chunks of equal size.
 TEST_F(ObjectMediaIntegrationTest, ReadByChunk) {
   StatusOr<Client> client = MakeIntegrationTestClient();
@@ -496,29 +458,6 @@ TEST_F(ObjectMediaIntegrationTest, ConnectionFailureReadJSON) {
 
   auto object_name = MakeRandomObjectName();
 
-  // We force the library to use the JSON API by adding the
-  // `IfGenerationNotMatch()` parameter, both JSON and XML use the same code to
-  // download, but controlling the endpoint for JSON is easier.
-  auto stream =
-      client.ReadObject(bucket_name_, object_name, IfGenerationNotMatch(0));
-  std::string actual(std::istreambuf_iterator<char>{stream}, {});
-  EXPECT_TRUE(actual.empty());
-  EXPECT_TRUE(stream.bad());
-  EXPECT_FALSE(stream.status().ok());
-  EXPECT_EQ(StatusCode::kUnavailable, stream.status().code())
-      << ", status=" << stream.status();
-}
-
-TEST_F(ObjectMediaIntegrationTest, ConnectionFailureReadXML) {
-  ScopedEnvironment emulator("CLOUD_STORAGE_EMULATOR_ENDPOINT", {});
-  Client client{
-      Options{}
-          .set<UnifiedCredentialsOption>(MakeInsecureCredentials())
-          .set<RestEndpointOption>("http://localhost:1")
-          .set<RetryPolicyOption>(LimitedErrorCountRetryPolicy(2).clone())};
-
-  auto object_name = MakeRandomObjectName();
-
   auto stream = client.ReadObject(bucket_name_, object_name);
   std::string actual(std::istreambuf_iterator<char>{stream}, {});
   EXPECT_TRUE(actual.empty());
@@ -538,29 +477,8 @@ TEST_F(ObjectMediaIntegrationTest, ConnectionFailureWriteJSON) {
 
   auto object_name = MakeRandomObjectName();
 
-  // We force the library to use the JSON API by adding the
-  // `IfGenerationNotMatch()` parameter, both JSON and XML use the same code to
-  // download, but controlling the endpoint for JSON is easier.
-  auto stream = client.WriteObject(
-      bucket_name_, object_name, IfGenerationMatch(0), IfGenerationNotMatch(7));
-  EXPECT_TRUE(stream.bad());
-  EXPECT_FALSE(stream.metadata().status().ok());
-  EXPECT_EQ(StatusCode::kUnavailable, stream.metadata().status().code())
-      << ", status=" << stream.metadata().status();
-}
-
-TEST_F(ObjectMediaIntegrationTest, ConnectionFailureWriteXML) {
-  ScopedEnvironment emulator("CLOUD_STORAGE_EMULATOR_ENDPOINT", {});
-  Client client{
-      Options{}
-          .set<UnifiedCredentialsOption>(MakeInsecureCredentials())
-          .set<RestEndpointOption>("http://localhost:1")
-          .set<RetryPolicyOption>(LimitedErrorCountRetryPolicy(2).clone())};
-
-  auto object_name = MakeRandomObjectName();
-
-  auto stream = client.WriteObject(
-      bucket_name_, object_name, IfGenerationMatch(0), IfGenerationNotMatch(7));
+  auto stream =
+      client.WriteObject(bucket_name_, object_name, IfGenerationMatch(0));
   EXPECT_TRUE(stream.bad());
   EXPECT_FALSE(stream.metadata().status().ok());
   EXPECT_EQ(StatusCode::kUnavailable, stream.metadata().status().code())
@@ -606,7 +524,6 @@ TEST_F(ObjectMediaIntegrationTest, ConnectionFailureUploadFile) {
 
 TEST_F(ObjectMediaIntegrationTest, StreamingReadTimeout) {
   if (!UsingEmulator()) GTEST_SKIP();
-
   auto options = ClientOptions::CreateDefaultClientOptions();
   ASSERT_STATUS_OK(options);
 
@@ -688,9 +605,20 @@ TEST_F(ObjectMediaIntegrationTest, StreamingReadInternalError) {
   ASSERT_STATUS_OK(source_meta);
   ScheduleForDelete(*source_meta);
 
-  auto stream = client.ReadObject(
-      bucket_name_, object_name,
-      CustomHeader("x-goog-emulator-instructions", "return-503-after-256K"));
+  auto retry_response = InsertRetryTest(RetryTestRequest{{
+      RetryTestConfiguration{"storage.objects.get",
+                             {
+                                 "return-503",
+                                 "return-503",
+                                 "return-broken-stream-after-256K",
+                                 "return-broken-stream-after-64K",
+                             }},
+  }});
+  ASSERT_STATUS_OK(retry_response);
+
+  auto stream =
+      client.ReadObject(bucket_name_, object_name,
+                        CustomHeader("x-retry-test-id", retry_response->id));
   std::vector<char> actual(64 * 1024);
   for (std::size_t offset = 0;
        offset < contents.size() && !stream.bad() && !stream.eof();
@@ -705,6 +633,70 @@ TEST_F(ObjectMediaIntegrationTest, StreamingReadInternalError) {
   }
 }
 
+TEST_F(ObjectMediaIntegrationTest, StringView) {
+  StatusOr<Client> client = MakeIntegrationTestClient();
+  ASSERT_STATUS_OK(client);
+
+  auto const object_name = MakeRandomObjectName();
+  auto const contents = LoremIpsum();
+
+  // Create the object, but only if it does not exist already.
+  StatusOr<ObjectMetadata> meta =
+      client->InsertObject(bucket_name_, object_name,
+                           absl::string_view(contents), IfGenerationMatch(0));
+  ASSERT_STATUS_OK(meta);
+  ScheduleForDelete(*meta);
+  EXPECT_EQ(object_name, meta->name());
+  EXPECT_EQ(bucket_name_, meta->bucket());
+
+  // Create an iostream to read the object back.
+  auto stream = client->ReadObject(bucket_name_, object_name);
+  std::string actual(std::istreambuf_iterator<char>{stream}, {});
+  EXPECT_EQ(contents, actual);
+}
+
+TEST_F(ObjectMediaIntegrationTest, String) {
+  StatusOr<Client> client = MakeIntegrationTestClient();
+  ASSERT_STATUS_OK(client);
+
+  auto const object_name = MakeRandomObjectName();
+  std::string const contents = LoremIpsum();
+
+  // Create the object, but only if it does not exist already.
+  StatusOr<ObjectMetadata> meta = client->InsertObject(
+      bucket_name_, object_name, contents, IfGenerationMatch(0));
+  ASSERT_STATUS_OK(meta);
+  ScheduleForDelete(*meta);
+  EXPECT_EQ(object_name, meta->name());
+  EXPECT_EQ(bucket_name_, meta->bucket());
+
+  // Create an iostream to read the object back.
+  auto stream = client->ReadObject(bucket_name_, object_name);
+  std::string actual(std::istreambuf_iterator<char>{stream}, {});
+  EXPECT_EQ(contents, actual);
+}
+
+TEST_F(ObjectMediaIntegrationTest, CharConstPointer) {
+  StatusOr<Client> client = MakeIntegrationTestClient();
+  ASSERT_STATUS_OK(client);
+
+  auto const object_name = MakeRandomObjectName();
+  std::string const contents = LoremIpsum();
+
+  // Create the object, but only if it does not exist already.
+  StatusOr<ObjectMetadata> meta = client->InsertObject(
+      bucket_name_, object_name, contents.c_str(), IfGenerationMatch(0));
+  ASSERT_STATUS_OK(meta);
+  ScheduleForDelete(*meta);
+  EXPECT_EQ(object_name, meta->name());
+  EXPECT_EQ(bucket_name_, meta->bucket());
+
+  // Create an iostream to read the object back.
+  auto stream = client->ReadObject(bucket_name_, object_name);
+  std::string actual(std::istreambuf_iterator<char>{stream}, {});
+  EXPECT_EQ(contents, actual);
+}
+
 }  // anonymous namespace
 GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
 }  // namespace storage
