@@ -12,9 +12,10 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/vpcaccess/vpc_access_client.h"
+//! [all]
+#include "google/cloud/vpcaccess/v1/vpc_access_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
   if (argc != 3) {
@@ -22,19 +23,20 @@ int main(int argc, char* argv[]) try {
     return 1;
   }
 
-  namespace vpcaccess = ::google::cloud::vpcaccess;
+  auto const location = google::cloud::Location(argv[1], argv[2]);
+
+  namespace vpcaccess = ::google::cloud::vpcaccess_v1;
   auto client = vpcaccess::VpcAccessServiceClient(
       vpcaccess::MakeVpcAccessServiceConnection());
 
-  auto const parent =
-      std::string("projects/") + argv[1] + "/locations/" + argv[2];
-  for (auto r : client.ListConnectors(parent)) {
-    if (!r) throw std::runtime_error(r.status().message());
-    std::cout << r->DebugString() << "\n";
+  for (auto c : client.ListConnectors(location.FullName())) {
+    if (!c) throw std::move(c).status();
+    std::cout << c->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [all]
