@@ -12,29 +12,31 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-#include "google/cloud/workflows/workflows_client.h"
-#include "google/cloud/project.h"
+//! [START workflows_client_library] [all]
+#include "google/cloud/workflows/v1/workflows_client.h"
+#include "google/cloud/location.h"
 #include <iostream>
-#include <stdexcept>
 
 int main(int argc, char* argv[]) try {
-  if (argc != 2) {
-    std::cerr << "Usage: " << argv[0] << " project-id\n";
+  if (argc != 3) {
+    std::cerr << "Usage: " << argv[0] << " project-id location-id\n";
     return 1;
   }
 
-  namespace workflows = ::google::cloud::workflows;
+  auto const location = google::cloud::Location(argv[1], argv[2]);
+
+  namespace workflows = ::google::cloud::workflows_v1;
   auto client =
       workflows::WorkflowsClient(workflows::MakeWorkflowsConnection());
 
-  auto const project = google::cloud::Project(argv[1]).FullName();
-  for (auto w : client.ListWorkflows(project)) {
-    if (!w) throw std::runtime_error(w.status().message());
+  for (auto w : client.ListWorkflows(location.FullName())) {
+    if (!w) throw std::move(w).status();
     std::cout << w->DebugString() << "\n";
   }
 
   return 0;
-} catch (std::exception const& ex) {
-  std::cerr << "Standard exception raised: " << ex.what() << "\n";
+} catch (google::cloud::Status const& status) {
+  std::cerr << "google::cloud::Status thrown: " << status << "\n";
   return 1;
 }
+//! [END workflows_client_library] [all]
